@@ -1,26 +1,7 @@
-/*
- * Copyright (C) 2008-2021 The QXmpp developers
- *
- * Authors:
- *	Ian Reinhart Geiser
- *  Jeremy Lainé
- *
- * Source:
- *	https://github.com/qxmpp-project/qxmpp
- *
- * This file is a part of QXmpp library.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- */
+// SPDX-FileCopyrightText: 2012 Ian Reinhart Geiser <geiseri@kde.org>
+// SPDX-FileCopyrightText: 2012 Jeremy Lainé <jeremy.laine@m4x.org>
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include "example_3_transferHandling.h"
 
@@ -103,8 +84,9 @@ void xmppClient::slotPresenceReceived(const QXmppPresence &presence)
     // do nothing
     if (m_recipient.isEmpty() ||
         QXmppUtils::jidToBareJid(presence.from()) != m_recipient ||
-        presence.type() != QXmppPresence::Available)
+        presence.type() != QXmppPresence::Available) {
         return;
+    }
 
     // send the file and connect to the job's signals
     QXmppTransferJob *job = transferManager->sendFile(presence.from(), ":/example_3_transferHandling.cpp", "example source code");
