@@ -1,3 +1,7 @@
+// SPDX-FileCopyrightText: 2009 Ian Geiser <ian.geiser@gmail.com>
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
+
 #ifndef REMOTEINTERFACE_H
 #define REMOTEINTERFACE_H
 
@@ -12,8 +16,8 @@ public:
     bool isAuthorized(const QString &jid) const override;
 
     // RPC Interface
-public slots:
-    QString echoString(const QString &message);
+public:
+    Q_SLOT QString echoString(const QString &message);
 };
 
 #endif  // REMOTEINTERFACE_H
