@@ -6,41 +6,44 @@
 #include "QXmppBindIq.h"
 
 #include "QXmppConstants_p.h"
-#include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 #include <QTextStream>
 #include <QXmlStreamWriter>
 
-/// Returns the bound JID.
-///
+using namespace QXmpp::Private;
 
+/// Creates a Bind IQ of type set with the specified resource.
+QXmppBindIq QXmppBindIq::bindAddressIq(const QString &resource)
+{
+    QXmppBindIq iq;
+    iq.setType(QXmppIq::Set);
+    iq.setResource(resource);
+    return iq;
+}
+
+/// Returns the bound JID.
 QString QXmppBindIq::jid() const
 {
     return m_jid;
 }
 
 /// Sets the bound JID.
-///
-/// \param jid
-
 void QXmppBindIq::setJid(const QString &jid)
 {
     m_jid = jid;
 }
 
 /// Returns the requested resource.
-///
-
 QString QXmppBindIq::resource() const
 {
     return m_resource;
 }
 
 /// Sets the requested resource.
-///
-/// \param resource
-
 void QXmppBindIq::setResource(const QString &resource)
 {
     m_resource = resource;
@@ -49,26 +52,25 @@ void QXmppBindIq::setResource(const QString &resource)
 /// \cond
 bool QXmppBindIq::isBindIq(const QDomElement &element)
 {
-    QDomElement bindElement = element.firstChildElement(QStringLiteral("bind"));
-    return (bindElement.namespaceURI() == ns_bind);
+    return isIqType(element, u"bind", ns_bind);
 }
 
 void QXmppBindIq::parseElementFromChild(const QDomElement &element)
 {
-    QDomElement bindElement = element.firstChildElement(QStringLiteral("bind"));
-    m_jid = bindElement.firstChildElement(QStringLiteral("jid")).text();
-    m_resource = bindElement.firstChildElement(QStringLiteral("resource")).text();
+    QDomElement bindElement = firstChildElement(element, u"bind");
+    m_jid = firstChildElement(bindElement, u"jid").text();
+    m_resource = firstChildElement(bindElement, u"resource").text();
 }
 
 void QXmppBindIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement(QStringLiteral("bind"));
-    writer->writeDefaultNamespace(ns_bind);
+    writer->writeStartElement(QSL65("bind"));
+    writer->writeDefaultNamespace(toString65(ns_bind));
     if (!m_jid.isEmpty()) {
-        helperToXmlAddTextElement(writer, QStringLiteral("jid"), m_jid);
+        writeXmlTextElement(writer, u"jid", m_jid);
     }
     if (!m_resource.isEmpty()) {
-        helperToXmlAddTextElement(writer, QStringLiteral("resource"), m_resource);
+        writeXmlTextElement(writer, u"resource", m_resource);
     }
     writer->writeEndElement();
 }
