@@ -5,10 +5,15 @@
 #include "QXmppBitsOfBinaryIq.h"
 
 #include "QXmppConstants_p.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 #include <QSharedData>
 
+using namespace QXmpp::Private;
+
 ///
 /// \class QXmppBitsOfBinaryIq
 ///
@@ -30,26 +35,15 @@ QXmppBitsOfBinaryIq::~QXmppBitsOfBinaryIq() = default;
 ///
 bool QXmppBitsOfBinaryIq::isBitsOfBinaryIq(const QDomElement &element)
 {
-    QDomElement child = element.firstChildElement();
-    while (!child.isNull()) {
-        if (QXmppBitsOfBinaryData::isBitsOfBinaryData(child)) {
-            return true;
-        }
-        child = child.nextSiblingElement();
-    }
-    return false;
+    return isIqType(element, u"data", ns_bob);
 }
 
 /// \cond
 void QXmppBitsOfBinaryIq::parseElementFromChild(const QDomElement &element)
 {
-    QDomElement child = element.firstChildElement();
-    while (!child.isNull()) {
-        if (QXmppBitsOfBinaryData::isBitsOfBinaryData(child)) {
-            QXmppBitsOfBinaryData::parseElementFromChild(child);
-            break;
-        }
-        child = child.nextSiblingElement();
+    auto child = firstChildElement(element, u"data", ns_bob);
+    if (!child.isNull()) {
+        QXmppBitsOfBinaryData::parseElementFromChild(child);
     }
 }
 
