@@ -5,10 +5,14 @@
 #include "QXmppByteStreamIq.h"
 
 #include "QXmppConstants_p.h"
-#include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 
+using namespace QXmpp::Private;
+
 ///
 /// \enum QXmppByteStreamIq::Mode
 ///
@@ -177,64 +181,61 @@ void QXmppByteStreamIq::setStreamHostUsed(const QString &jid)
 ///
 bool QXmppByteStreamIq::isByteStreamIq(const QDomElement &element)
 {
-    return element.firstChildElement(QStringLiteral("query")).namespaceURI() == ns_bytestreams;
+    return isIqType(element, u"query", ns_bytestreams);
 }
 
 /// \cond
 void QXmppByteStreamIq::parseElementFromChild(const QDomElement &element)
 {
-    auto queryElement = element.firstChildElement(QStringLiteral("query"));
-    m_sid = queryElement.attribute(QStringLiteral("sid"));
-    const auto modeStr = queryElement.attribute(QStringLiteral("mode"));
-    if (modeStr == QStringLiteral("tcp")) {
+    auto queryElement = firstChildElement(element, u"query", ns_bytestreams);
+    m_sid = queryElement.attribute(u"sid"_s);
+    const auto modeStr = queryElement.attribute(u"mode"_s);
+    if (modeStr == u"tcp") {
         m_mode = Tcp;
-    } else if (modeStr == QStringLiteral("udp")) {
+    } else if (modeStr == u"udp") {
         m_mode = Udp;
     } else {
         m_mode = None;
     }
 
-    QDomElement hostElement = queryElement.firstChildElement(QStringLiteral("streamhost"));
-    while (!hostElement.isNull()) {
+    for (const auto &hostElement : iterChildElements(queryElement, u"streamhost")) {
         StreamHost streamHost;
-        streamHost.setHost(hostElement.attribute(QStringLiteral("host")));
-        streamHost.setJid(hostElement.attribute(QStringLiteral("jid")));
-        streamHost.setPort(hostElement.attribute(QStringLiteral("port")).toInt());
-        streamHost.setZeroconf(hostElement.attribute(QStringLiteral("zeroconf")));
+        streamHost.setHost(hostElement.attribute(u"host"_s));
+        streamHost.setJid(hostElement.attribute(u"jid"_s));
+        streamHost.setPort(parseInt<quint16>(hostElement.attribute(u"port"_s)).value_or(0));
+        streamHost.setZeroconf(hostElement.attribute(u"zeroconf"_s));
         m_streamHosts.append(streamHost);
-
-        hostElement = hostElement.nextSiblingElement(QStringLiteral("streamhost"));
     }
-    m_activate = queryElement.firstChildElement(QStringLiteral("activate")).text();
-    m_streamHostUsed = queryElement.firstChildElement(QStringLiteral("streamhost-used")).attribute(QStringLiteral("jid"));
+    m_activate = firstChildElement(queryElement, u"activate").text();
+    m_streamHostUsed = firstChildElement(queryElement, u"streamhost-used").attribute(u"jid"_s);
 }
 
 void QXmppByteStreamIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement(QStringLiteral("query"));
-    writer->writeDefaultNamespace(ns_bytestreams);
-    helperToXmlAddAttribute(writer, QStringLiteral("sid"), m_sid);
+    writer->writeStartElement(QSL65("query"));
+    writer->writeDefaultNamespace(toString65(ns_bytestreams));
+    writeOptionalXmlAttribute(writer, u"sid", m_sid);
     QString modeStr;
     if (m_mode == Tcp) {
-        modeStr = QStringLiteral("tcp");
+        modeStr = u"tcp"_s;
     } else if (m_mode == Udp) {
-        modeStr = QStringLiteral("udp");
+        modeStr = u"udp"_s;
     }
-    helperToXmlAddAttribute(writer, QStringLiteral("mode"), modeStr);
+    writeOptionalXmlAttribute(writer, u"mode", modeStr);
     for (const auto &streamHost : m_streamHosts) {
-        writer->writeStartElement(QStringLiteral("streamhost"));
-        helperToXmlAddAttribute(writer, QStringLiteral("host"), streamHost.host());
-        helperToXmlAddAttribute(writer, QStringLiteral("jid"), streamHost.jid());
-        helperToXmlAddAttribute(writer, QStringLiteral("port"), QString::number(streamHost.port()));
-        helperToXmlAddAttribute(writer, QStringLiteral("zeroconf"), streamHost.zeroconf());
+        writer->writeStartElement(QSL65("streamhost"));
+        writeOptionalXmlAttribute(writer, u"host", streamHost.host());
+        writeOptionalXmlAttribute(writer, u"jid", streamHost.jid());
+        writeOptionalXmlAttribute(writer, u"port", QString::number(streamHost.port()));
+        writeOptionalXmlAttribute(writer, u"zeroconf", streamHost.zeroconf());
         writer->writeEndElement();
     }
     if (!m_activate.isEmpty()) {
-        helperToXmlAddTextElement(writer, QStringLiteral("activate"), m_activate);
+        writeXmlTextElement(writer, u"activate", m_activate);
     }
     if (!m_streamHostUsed.isEmpty()) {
-        writer->writeStartElement(QStringLiteral("streamhost-used"));
-        helperToXmlAddAttribute(writer, QStringLiteral("jid"), m_streamHostUsed);
+        writer->writeStartElement(QSL65("streamhost-used"));
+        writeOptionalXmlAttribute(writer, u"jid", m_streamHostUsed);
         writer->writeEndElement();
     }
 
