@@ -7,7 +7,9 @@
 
 #include "QXmppConstants_p.h"
 #include "QXmppDataFormBase.h"
-#include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <optional>
 
@@ -19,8 +21,9 @@
 #include <QStringList>
 #include <QUrl>
 
-struct field_type
-{
+using namespace QXmpp::Private;
+
+struct field_type {
     QXmppDataForm::Field::Type type;
     const char *str;
 };
@@ -41,9 +44,10 @@ static field_type field_types[] = {
 
 std::optional<QXmppDataForm::Field::Type> fieldTypeFromString(const QString &type)
 {
+    const auto typeStr = type.toStdString();
     struct field_type *ptr;
     for (ptr = field_types; ptr->str; ptr++) {
-        if (type == ptr->str) {
+        if (typeStr == ptr->str) {
             return ptr->type;
         }
     }
@@ -55,7 +59,7 @@ QString fieldTypeToString(QXmppDataForm::Field::Type type)
     struct field_type *ptr;
     for (ptr = field_types; ptr->str; ptr++) {
         if (type == ptr->type) {
-            return ptr->str;
+            return QString::fromLocal8Bit(ptr->str);
         }
     }
     return {};
@@ -63,16 +67,16 @@ QString fieldTypeToString(QXmppDataForm::Field::Type type)
 
 std::optional<QXmppDataForm::Type> formTypeFromString(const QString &type)
 {
-    if (type == "form") {
+    if (type == u"form") {
         return QXmppDataForm::Form;
     }
-    if (type == "submit") {
+    if (type == u"submit") {
         return QXmppDataForm::Submit;
     }
-    if (type == "cancel") {
+    if (type == u"cancel") {
         return QXmppDataForm::Cancel;
     }
-    if (type == "result") {
+    if (type == u"result") {
         return QXmppDataForm::Result;
     }
     return {};
@@ -82,13 +86,13 @@ QString formTypeToString(QXmppDataForm::Type type)
 {
     switch (type) {
     case QXmppDataForm::Form:
-        return "form";
+        return u"form"_s;
     case QXmppDataForm::Submit:
-        return "submit";
+        return u"submit"_s;
     case QXmppDataForm::Cancel:
-        return "cancel";
+        return u"cancel"_s;
     case QXmppDataForm::Result:
-        return "result";
+        return u"result"_s;
     default:
         return {};
     }
@@ -770,7 +774,7 @@ QString QXmppDataForm::formType() const
 {
     const auto formTypeItr = std::find_if(d->fields.begin(), d->fields.end(), [](const QXmppDataForm::Field &field) {
         return field.type() == QXmppDataForm::Field::HiddenField &&
-            field.key() == QStringLiteral("FORM_TYPE");
+            field.key() == u"FORM_TYPE";
     });
 
     if (formTypeItr != d->fields.end()) {
@@ -795,66 +799,61 @@ void QXmppDataForm::parse(const QDomElement &element)
     }
 
     /* form type */
-    if (const auto type = formTypeFromString(element.attribute("type"))) {
+    const auto typeStr = element.attribute(u"type"_s);
+    if (const auto type = formTypeFromString(typeStr)) {
         d->type = *type;
     } else {
-        qWarning() << "Unknown form type" << element.attribute("type");
+        qWarning() << "Unknown form type" << typeStr;
         return;
     }
 
     /* form properties */
-    d->title = element.firstChildElement("title").text();
-    d->instructions = element.firstChildElement("instructions").text();
+    d->title = firstChildElement(element, u"title").text();
+    d->instructions = firstChildElement(element, u"instructions").text();
 
-    for (auto fieldElement = element.firstChildElement("field");
-         !fieldElement.isNull();
-         fieldElement = fieldElement.nextSiblingElement("field")) {
+    for (const auto &fieldElement : iterChildElements(element, u"field")) {
         QXmppDataForm::Field field;
 
         /* field type */
-        field.setType(fieldTypeFromString(fieldElement.attribute("type")).value_or(Field::TextSingleField));
+        field.setType(fieldTypeFromString(fieldElement.attribute(u"type"_s)).value_or(Field::TextSingleField));
 
         /* field attributes */
-        field.setLabel(fieldElement.attribute("label"));
-        field.setKey(fieldElement.attribute("var"));
+        field.setLabel(fieldElement.attribute(u"label"_s));
+        field.setKey(fieldElement.attribute(u"var"_s));
 
         /* field value(s) */
         switch (field.type()) {
         case Field::BooleanField: {
-            const auto valueStr = fieldElement.firstChildElement("value").text();
-            field.setValue(valueStr == "1" || valueStr == "true");
+            const auto valueStr = fieldElement.firstChildElement(u"value"_s).text();
+            field.setValue(valueStr == u"1" || valueStr == u"true");
             break;
         }
         case Field::ListMultiField:
         case Field::JidMultiField:
         case Field::TextMultiField: {
             QStringList values;
-            for (auto element = fieldElement.firstChildElement("value");
-                 !element.isNull();
-                 element = element.nextSiblingElement("value")) {
+            for (const auto &element : iterChildElements(fieldElement, u"value")) {
                 values << element.text();
             }
             field.setValue(values);
             break;
         }
         default:
-            field.setValue(fieldElement.firstChildElement("value").text());
+            field.setValue(fieldElement.firstChildElement(u"value"_s).text());
         }
 
         /* field media */
-        if (const auto mediaElement = fieldElement.firstChildElement("media");
-            mediaElement.namespaceURI() == ns_media_element) {
-            field.mediaSize().setHeight(mediaElement.attribute("height", "-1").toInt());
-            field.mediaSize().setWidth(mediaElement.attribute("width", "-1").toInt());
+        if (const auto mediaElement = firstChildElement(fieldElement, u"media", ns_media_element);
+            !mediaElement.isNull()) {
+            field.mediaSize().setHeight(mediaElement.attribute(u"height"_s, u"-1"_s).toInt());
+            field.mediaSize().setWidth(mediaElement.attribute(u"width"_s, u"-1"_s).toInt());
 
             QMimeDatabase database;
 
-            for (auto element = mediaElement.firstChildElement(QStringLiteral("uri"));
-                 !element.isNull();
-                 element = element.nextSiblingElement(QStringLiteral("uri"))) {
+            for (const auto &element : iterChildElements(mediaElement, u"uri")) {
                 field.mediaSources() << MediaSource(
                     QUrl(element.text()),
-                    database.mimeTypeForName(element.attribute(QStringLiteral("type"))));
+                    database.mimeTypeForName(element.attribute(u"type"_s)));
             }
         }
 
@@ -863,20 +862,20 @@ void QXmppDataForm::parse(const QDomElement &element)
         case Field::ListMultiField:
         case Field::ListSingleField: {
             QList<QPair<QString, QString>> options;
-            for (auto element = fieldElement.firstChildElement("option");
-                 !element.isNull();
-                 element = element.nextSiblingElement("option")) {
-                options << qMakePair(element.attribute("label"), element.firstChildElement("value").text());
+            for (const auto &element : iterChildElements(fieldElement, u"option")) {
+                options << qMakePair(element.attribute(u"label"_s),
+                                     firstChildElement(element, u"value").text());
             }
             field.setOptions(options);
+            break;
         }
         default:
             break;
         }
 
         /* other properties */
-        field.setDescription(fieldElement.firstChildElement("description").text());
-        field.setRequired(!fieldElement.firstChildElement("required").isNull());
+        field.setDescription(firstChildElement(fieldElement, u"description").text());
+        field.setRequired(!firstChildElement(fieldElement, u"required").isNull());
 
         d->fields.append(field);
     }
@@ -888,73 +887,73 @@ void QXmppDataForm::toXml(QXmlStreamWriter *writer) const
         return;
     }
 
-    writer->writeStartElement("x");
-    writer->writeDefaultNamespace(ns_data);
+    writer->writeStartElement(QSL65("x"));
+    writer->writeDefaultNamespace(toString65(ns_data));
 
     /* form type */
-    writer->writeAttribute("type", formTypeToString(d->type));
+    writer->writeAttribute(QSL65("type"), formTypeToString(d->type));
 
     /* form properties */
     if (!d->title.isEmpty()) {
-        writer->writeTextElement("title", d->title);
+        writer->writeTextElement(QSL65("title"), d->title);
     }
     if (!d->instructions.isEmpty()) {
-        writer->writeTextElement("instructions", d->instructions);
+        writer->writeTextElement(QSL65("instructions"), d->instructions);
     }
 
     for (const auto &field : d->fields) {
-        writer->writeStartElement("field");
+        writer->writeStartElement(QSL65("field"));
 
         /* field type */
-        writer->writeAttribute("type", fieldTypeToString(field.type()));
+        writer->writeAttribute(QSL65("type"), fieldTypeToString(field.type()));
 
         /* field attributes */
-        helperToXmlAddAttribute(writer, "label", field.label());
-        helperToXmlAddAttribute(writer, "var", field.key());
+        writeOptionalXmlAttribute(writer, u"label", field.label());
+        writeOptionalXmlAttribute(writer, u"var", field.key());
 
         /* field value(s) */
         switch (field.type()) {
         case Field::BooleanField:
-            helperToXmlAddTextElement(writer, "value", field.value().toBool() ? "1" : "0");
+            writeXmlTextElement(writer, u"value", field.value().toBool() ? u"1" : u"0");
             break;
         case Field::ListMultiField:
         case Field::JidMultiField:
         case Field::TextMultiField: {
             const auto values = field.value().toStringList();
             for (const QString &value : values) {
-                helperToXmlAddTextElement(writer, "value", value);
+                writeXmlTextElement(writer, u"value", value);
             }
             break;
         }
         default:
             if (const auto value = field.value().toString(); !value.isEmpty()) {
-                helperToXmlAddTextElement(writer, "value", value);
+                writeXmlTextElement(writer, u"value", value);
             }
         }
 
         /* field media */
         if (!field.mediaSources().isEmpty()) {
-            writer->writeStartElement("media");
-            writer->writeDefaultNamespace(ns_media_element);
+            writer->writeStartElement(QSL65("media"));
+            writer->writeDefaultNamespace(toString65(ns_media_element));
 
             // media width and height
             if (field.mediaSize().width() > 0) {
-                helperToXmlAddAttribute(
+                writeOptionalXmlAttribute(
                     writer,
-                    QStringLiteral("width"),
+                    u"width"_s,
                     QString::number(field.mediaSize().width()));
             }
             if (field.mediaSize().height() > 0) {
-                helperToXmlAddAttribute(
+                writeOptionalXmlAttribute(
                     writer,
-                    QStringLiteral("height"),
+                    u"height"_s,
                     QString::number(field.mediaSize().height()));
             }
 
             const auto sources = field.mediaSources();
             for (const auto &source : sources) {
-                writer->writeStartElement(QStringLiteral("uri"));
-                helperToXmlAddAttribute(writer, QStringLiteral("type"), source.contentType().name());
+                writer->writeStartElement(QSL65("uri"));
+                writeOptionalXmlAttribute(writer, u"type", source.contentType().name());
                 writer->writeCharacters(source.uri().toString());
                 writer->writeEndElement();
             }
@@ -968,11 +967,12 @@ void QXmppDataForm::toXml(QXmlStreamWriter *writer) const
         case Field::ListSingleField: {
             const auto options = field.options();
             for (const auto &option : options) {
-                writer->writeStartElement("option");
-                helperToXmlAddAttribute(writer, "label", option.first);
-                helperToXmlAddTextElement(writer, "value", option.second);
+                writer->writeStartElement(QSL65("option"));
+                writeOptionalXmlAttribute(writer, u"label", option.first);
+                writeXmlTextElement(writer, u"value", option.second);
                 writer->writeEndElement();
             }
+            break;
         }
         default:
             break;
@@ -980,10 +980,10 @@ void QXmppDataForm::toXml(QXmlStreamWriter *writer) const
 
         /* other properties */
         if (!field.description().isEmpty()) {
-            helperToXmlAddTextElement(writer, "description", field.description());
+            writeXmlTextElement(writer, u"description", field.description());
         }
         if (field.isRequired()) {
-            helperToXmlAddTextElement(writer, "required", "");
+            writeXmlTextElement(writer, u"required", u"");
         }
 
         writer->writeEndElement();
