@@ -5,12 +5,16 @@
 #include "QXmppDiscoveryIq.h"
 
 #include "QXmppConstants_p.h"
-#include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QCryptographicHash>
 #include <QDomElement>
 #include <QSharedData>
 
+using namespace QXmpp::Private;
+
 static bool identityLessThan(const QXmppDiscoveryIq::Identity &i1, const QXmppDiscoveryIq::Identity &i2)
 {
     if (i1.category() < i2.category()) {
@@ -386,10 +390,10 @@ QByteArray QXmppDiscoveryIq::verificationString() const
     std::sort(sortedFeatures.begin(), sortedFeatures.end());
     sortedFeatures.removeDuplicates();
     for (const auto &identity : sortedIdentities) {
-        S += QString("%1/%2/%3/%4<").arg(identity.category(), identity.type(), identity.language(), identity.name());
+        S += identity.category() + u'/' + identity.type() + u'/' + identity.language() + u'/' + identity.name() + u'<';
     }
     for (const auto &feature : sortedFeatures) {
-        S += feature + QLatin1String("<");
+        S += feature + u'<';
     }
 
     if (!d->form.isNull()) {
@@ -399,23 +403,23 @@ QByteArray QXmppDiscoveryIq::verificationString() const
             fieldMap.insert(field.key(), field);
         }
 
-        if (fieldMap.contains("FORM_TYPE")) {
-            const QXmppDataForm::Field field = fieldMap.take("FORM_TYPE");
-            S += field.value().toString() + QLatin1String("<");
+        if (fieldMap.contains(u"FORM_TYPE"_s)) {
+            const QXmppDataForm::Field field = fieldMap.take(u"FORM_TYPE"_s);
+            S += field.value().toString() + u"<";
 
             QStringList keys = fieldMap.keys();
             std::sort(keys.begin(), keys.end());
             for (const auto &key : keys) {
                 const QXmppDataForm::Field field = fieldMap.value(key);
-                S += key + QLatin1String("<");
+                S += key + u'<';
                 if (field.value().canConvert<QStringList>()) {
                     QStringList list = field.value().toStringList();
                     list.sort();
-                    S += list.join(QLatin1String("<"));
+                    S += list.join(u'<');
                 } else {
                     S += field.value().toString();
                 }
-                S += QLatin1String("<");
+                S += u'<';
             }
         } else {
             qWarning("QXmppDiscoveryIq form does not contain FORM_TYPE");
@@ -433,91 +437,87 @@ QByteArray QXmppDiscoveryIq::verificationString() const
 ///
 bool QXmppDiscoveryIq::isDiscoveryIq(const QDomElement &element)
 {
-    QDomElement queryElement = element.firstChildElement();
-    return checkIqType(queryElement.tagName(), queryElement.namespaceURI());
+    return isIqType(element, u"query", ns_disco_info) || isIqType(element, u"query", ns_disco_items);
 }
 
 /// \cond
 bool QXmppDiscoveryIq::checkIqType(const QString &tagName, const QString &xmlNamespace)
 {
-    return tagName == QStringLiteral("query") &&
+    return tagName == u"query" &&
         (xmlNamespace == ns_disco_info || xmlNamespace == ns_disco_items);
 }
 
 void QXmppDiscoveryIq::parseElementFromChild(const QDomElement &element)
 {
-    QDomElement queryElement = element.firstChildElement("query");
-    d->queryNode = queryElement.attribute("node");
+    QDomElement queryElement = firstChildElement(element, u"query");
+    d->queryNode = queryElement.attribute(u"node"_s);
     if (queryElement.namespaceURI() == ns_disco_items) {
         d->queryType = ItemsQuery;
     } else {
         d->queryType = InfoQuery;
     }
 
-    QDomElement itemElement = queryElement.firstChildElement();
-    while (!itemElement.isNull()) {
-        if (itemElement.tagName() == "feature") {
-            d->features.append(itemElement.attribute("var"));
-        } else if (itemElement.tagName() == "identity") {
+    for (const auto &itemElement : iterChildElements(queryElement)) {
+        if (itemElement.tagName() == u"feature") {
+            d->features.append(itemElement.attribute(u"var"_s));
+        } else if (itemElement.tagName() == u"identity") {
             QXmppDiscoveryIq::Identity identity;
-            identity.setLanguage(itemElement.attribute("xml:lang"));
-            identity.setCategory(itemElement.attribute("category"));
-            identity.setName(itemElement.attribute("name"));
-            identity.setType(itemElement.attribute("type"));
+            identity.setLanguage(itemElement.attribute(u"xml:lang"_s));
+            identity.setCategory(itemElement.attribute(u"category"_s));
+            identity.setName(itemElement.attribute(u"name"_s));
+            identity.setType(itemElement.attribute(u"type"_s));
 
             // FIXME: for some reason the language does not found,
             // so we are forced to use QDomNamedNodeMap
             QDomNamedNodeMap m(itemElement.attributes());
             for (int i = 0; i < m.size(); ++i) {
-                if (m.item(i).nodeName() == "xml:lang") {
+                if (m.item(i).nodeName() == u"xml:lang") {
                     identity.setLanguage(m.item(i).nodeValue());
                     break;
                 }
             }
 
             d->identities.append(identity);
-        } else if (itemElement.tagName() == "item") {
+        } else if (itemElement.tagName() == u"item") {
             QXmppDiscoveryIq::Item item;
-            item.setJid(itemElement.attribute("jid"));
-            item.setName(itemElement.attribute("name"));
-            item.setNode(itemElement.attribute("node"));
+            item.setJid(itemElement.attribute(u"jid"_s));
+            item.setName(itemElement.attribute(u"name"_s));
+            item.setNode(itemElement.attribute(u"node"_s));
             d->items.append(item);
-        } else if (itemElement.tagName() == "x" &&
+        } else if (itemElement.tagName() == u"x" &&
                    itemElement.namespaceURI() == ns_data) {
             d->form.parse(itemElement);
         }
-        itemElement = itemElement.nextSiblingElement();
     }
 }
 
 void QXmppDiscoveryIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement("query");
-    writer->writeDefaultNamespace(
-        d->queryType == InfoQuery ? ns_disco_info : ns_disco_items);
-    helperToXmlAddAttribute(writer, "node", d->queryNode);
+    writer->writeStartElement(QSL65("query"));
+    writer->writeDefaultNamespace(toString65(d->queryType == InfoQuery ? ns_disco_info : ns_disco_items));
+    writeOptionalXmlAttribute(writer, u"node", d->queryNode);
 
     if (d->queryType == InfoQuery) {
         for (const auto &identity : d->identities) {
-            writer->writeStartElement("identity");
-            helperToXmlAddAttribute(writer, "xml:lang", identity.language());
-            helperToXmlAddAttribute(writer, "category", identity.category());
-            helperToXmlAddAttribute(writer, "name", identity.name());
-            helperToXmlAddAttribute(writer, "type", identity.type());
+            writer->writeStartElement(QSL65(u"identity"));
+            writeOptionalXmlAttribute(writer, u"xml:lang", identity.language());
+            writeOptionalXmlAttribute(writer, u"category", identity.category());
+            writeOptionalXmlAttribute(writer, u"name", identity.name());
+            writeOptionalXmlAttribute(writer, u"type", identity.type());
             writer->writeEndElement();
         }
 
         for (const auto &feature : d->features) {
-            writer->writeStartElement("feature");
-            helperToXmlAddAttribute(writer, "var", feature);
+            writer->writeStartElement(QSL65("feature"));
+            writeOptionalXmlAttribute(writer, u"var", feature);
             writer->writeEndElement();
         }
     } else {
         for (const auto &item : d->items) {
-            writer->writeStartElement("item");
-            helperToXmlAddAttribute(writer, "jid", item.jid());
-            helperToXmlAddAttribute(writer, "name", item.name());
-            helperToXmlAddAttribute(writer, "node", item.node());
+            writer->writeStartElement(QSL65("item"));
+            writeOptionalXmlAttribute(writer, u"jid", item.jid());
+            writeOptionalXmlAttribute(writer, u"name", item.name());
+            writeOptionalXmlAttribute(writer, u"node", item.node());
             writer->writeEndElement();
         }
     }
