@@ -4,11 +4,15 @@
 
 #include "QXmppElement.h"
 
-#include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 #include <QTextStream>
 
+using namespace QXmpp::Private;
+
 class QXmppElementPrivate
 {
 public:
@@ -37,7 +41,7 @@ QXmppElementPrivate::QXmppElementPrivate(const QDomElement &element)
     QString xmlns = element.namespaceURI();
     QString parentns = element.parentNode().namespaceURI();
     if (!xmlns.isEmpty() && xmlns != parentns) {
-        attributes.insert("xmlns", xmlns);
+        attributes.insert(u"xmlns"_s, xmlns);
     }
     QDomNamedNodeMap attrs = element.attributes();
     for (int i = 0; i < attrs.size(); i++) {
@@ -45,8 +49,9 @@ QXmppElementPrivate::QXmppElementPrivate(const QDomElement &element)
         attributes.insert(attr.name(), attr.value());
     }
 
-    QDomNode childNode = element.firstChild();
-    while (!childNode.isNull()) {
+    for (auto childNode = element.firstChild();
+         !childNode.isNull();
+         childNode = childNode.nextSibling()) {
         if (childNode.isElement()) {
             QXmppElementPrivate *child = new QXmppElementPrivate(childNode.toElement());
             child->parent = this;
@@ -54,7 +59,6 @@ QXmppElementPrivate::QXmppElementPrivate(const QDomElement &element)
         } else if (childNode.isText()) {
             value += childNode.toText().data();
         }
-        childNode = childNode.nextSibling();
     }
 
     QTextStream stream(&serializedSource);
@@ -217,7 +221,7 @@ QXmppElement QXmppElement::nextSiblingElement(const QString &name) const
         return QXmppElement();
     }
     const QList<QXmppElementPrivate *> &siblings_d = d->parent->children;
-    for (int i = siblings_d.indexOf(d) + 1; i < siblings_d.size(); i++) {
+    for (qsizetype i = siblings_d.indexOf(d) + 1; i < siblings_d.size(); i++) {
         if (name.isEmpty() || siblings_d[i]->name == name) {
             return QXmppElement(siblings_d[i]);
         }
@@ -289,12 +293,12 @@ void QXmppElement::toXml(QXmlStreamWriter *writer) const
     }
 
     writer->writeStartElement(d->name);
-    if (d->attributes.contains("xmlns")) {
-        writer->writeDefaultNamespace(d->attributes.value("xmlns"));
+    if (d->attributes.contains(u"xmlns"_s)) {
+        writer->writeDefaultNamespace(d->attributes.value(u"xmlns"_s));
     }
     std::for_each(d->attributes.keyBegin(), d->attributes.keyEnd(), [this, writer](const QString &key) {
-        if (key != "xmlns") {
-            helperToXmlAddAttribute(writer, key, d->attributes.value(key));
+        if (key != u"xmlns") {
+            writeOptionalXmlAttribute(writer, key, d->attributes.value(key));
         }
     });
     if (!d->value.isEmpty()) {
