@@ -6,9 +6,14 @@
 
 #include "QXmppConstants_p.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 
+using namespace QXmpp::Private;
+
 ///
 /// Returns the timezone offset in seconds.
 ///
@@ -50,31 +55,30 @@ void QXmppEntityTimeIq::setUtc(const QDateTime &utc)
 ///
 bool QXmppEntityTimeIq::isEntityTimeIq(const QDomElement &element)
 {
-    QDomElement timeElement = element.firstChildElement("time");
-    return timeElement.namespaceURI() == ns_entity_time;
+    return isIqType(element, u"time", ns_entity_time);
 }
 
 /// \cond
 bool QXmppEntityTimeIq::checkIqType(const QString &tagName, const QString &xmlns)
 {
-    return tagName == QStringLiteral("time") && xmlns == ns_entity_time;
+    return tagName == u"time" && xmlns == ns_entity_time;
 }
 
 void QXmppEntityTimeIq::parseElementFromChild(const QDomElement &element)
 {
-    QDomElement timeElement = element.firstChildElement("time");
-    m_tzo = QXmppUtils::timezoneOffsetFromString(timeElement.firstChildElement("tzo").text());
-    m_utc = QXmppUtils::datetimeFromString(timeElement.firstChildElement("utc").text());
+    QDomElement timeElement = firstChildElement(element, u"time");
+    m_tzo = QXmppUtils::timezoneOffsetFromString(firstChildElement(timeElement, u"tzo").text());
+    m_utc = QXmppUtils::datetimeFromString(firstChildElement(timeElement, u"utc").text());
 }
 
 void QXmppEntityTimeIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement("time");
-    writer->writeDefaultNamespace(ns_entity_time);
+    writer->writeStartElement(QSL65("time"));
+    writer->writeDefaultNamespace(toString65(ns_entity_time));
 
     if (m_utc.isValid()) {
-        helperToXmlAddTextElement(writer, "tzo", QXmppUtils::timezoneOffsetToString(m_tzo));
-        helperToXmlAddTextElement(writer, "utc", QXmppUtils::datetimeToString(m_utc));
+        writeXmlTextElement(writer, u"tzo", QXmppUtils::timezoneOffsetToString(m_tzo));
+        writeXmlTextElement(writer, u"utc", QXmppUtils::datetimeToString(m_utc));
     }
     writer->writeEndElement();
 }
