@@ -6,10 +6,15 @@
 #include "QXmppIbbIq.h"
 
 #include "QXmppConstants_p.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 #include <QXmlStreamWriter>
 
+using namespace QXmpp::Private;
+
 ///
 /// \class QXmppIbbOpenIq
 ///
@@ -62,23 +67,22 @@ void QXmppIbbOpenIq::setSid(const QString &sid)
 /// \cond
 bool QXmppIbbOpenIq::isIbbOpenIq(const QDomElement &element)
 {
-    QDomElement openElement = element.firstChildElement("open");
-    return openElement.namespaceURI() == ns_ibb;
+    return isIqType(element, u"open", ns_ibb);
 }
 
 void QXmppIbbOpenIq::parseElementFromChild(const QDomElement &element)
 {
-    QDomElement openElement = element.firstChildElement("open");
-    m_sid = openElement.attribute("sid");
-    m_block_size = openElement.attribute("block-size").toLong();
+    QDomElement openElement = firstChildElement(element, u"open");
+    m_sid = openElement.attribute(u"sid"_s);
+    m_block_size = openElement.attribute(u"block-size"_s).toLong();
 }
 
 void QXmppIbbOpenIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement("open");
-    writer->writeDefaultNamespace(ns_ibb);
-    writer->writeAttribute("sid", m_sid);
-    writer->writeAttribute("block-size", QString::number(m_block_size));
+    writer->writeStartElement(QSL65("open"));
+    writer->writeDefaultNamespace(toString65(ns_ibb));
+    writer->writeAttribute(QSL65("sid"), m_sid);
+    writer->writeAttribute(QSL65("block-size"), QString::number(m_block_size));
     writer->writeEndElement();
 }
 /// \endcond
@@ -117,21 +121,20 @@ void QXmppIbbCloseIq::setSid(const QString &sid)
 /// \cond
 bool QXmppIbbCloseIq::isIbbCloseIq(const QDomElement &element)
 {
-    QDomElement openElement = element.firstChildElement("close");
-    return openElement.namespaceURI() == ns_ibb;
+    return isIqType(element, u"close", ns_ibb);
 }
 
 void QXmppIbbCloseIq::parseElementFromChild(const QDomElement &element)
 {
-    QDomElement openElement = element.firstChildElement("close");
-    m_sid = openElement.attribute("sid");
+    QDomElement openElement = firstChildElement(element, u"close");
+    m_sid = openElement.attribute(u"sid"_s);
 }
 
 void QXmppIbbCloseIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement("close");
-    writer->writeDefaultNamespace(ns_ibb);
-    writer->writeAttribute("sid", m_sid);
+    writer->writeStartElement(QSL65("close"));
+    writer->writeDefaultNamespace(toString65(ns_ibb));
+    writer->writeAttribute(QSL65("sid"), m_sid);
     writer->writeEndElement();
 }
 /// \endcond
@@ -210,25 +213,28 @@ void QXmppIbbDataIq::setPayload(const QByteArray &data)
 /// \cond
 bool QXmppIbbDataIq::isIbbDataIq(const QDomElement &element)
 {
-    QDomElement dataElement = element.firstChildElement("data");
-    return dataElement.namespaceURI() == ns_ibb;
+    return isIqType(element, u"data", ns_ibb);
 }
 
 void QXmppIbbDataIq::parseElementFromChild(const QDomElement &element)
 {
-    QDomElement dataElement = element.firstChildElement("data");
-    m_sid = dataElement.attribute("sid");
-    m_seq = dataElement.attribute("seq").toLong();
+    QDomElement dataElement = firstChildElement(element, u"data");
+    m_sid = dataElement.attribute(u"sid"_s);
+    m_seq = parseInt<uint16_t>(dataElement.attribute(u"seq"_s)).value_or(0);
     m_payload = QByteArray::fromBase64(dataElement.text().toLatin1());
 }
 
 void QXmppIbbDataIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement("data");
-    writer->writeDefaultNamespace(ns_ibb);
-    writer->writeAttribute("sid", m_sid);
-    writer->writeAttribute("seq", QString::number(m_seq));
+    writer->writeStartElement(QSL65("data"));
+    writer->writeDefaultNamespace(toString65(ns_ibb));
+    writer->writeAttribute(QSL65("sid"), m_sid);
+    writer->writeAttribute(QSL65("seq"), QString::number(m_seq));
+#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
     writer->writeCharacters(m_payload.toBase64());
+#else
+    writer->writeCharacters(QString::fromUtf8(m_payload.toBase64()));
+#endif
     writer->writeEndElement();
 }
 /// \endcond
