@@ -5,6 +5,8 @@
 
 #include "QXmppLogger.h"
 
+#include "StringLiterals.h"
+
 #include <iostream>
 
 #include <QChildEvent>
@@ -15,29 +17,27 @@
 
 QXmppLogger *QXmppLogger::m_logger = nullptr;
 
-static const char *typeName(QXmppLogger::MessageType type)
+static QStringView typeName(QXmppLogger::MessageType type)
 {
     switch (type) {
     case QXmppLogger::DebugMessage:
-        return "DEBUG";
+        return u"DEBUG";
     case QXmppLogger::InformationMessage:
-        return "INFO";
+        return u"INFO";
     case QXmppLogger::WarningMessage:
-        return "WARNING";
+        return u"WARNING";
     case QXmppLogger::ReceivedMessage:
-        return "RECEIVED";
+        return u"RECEIVED";
     case QXmppLogger::SentMessage:
-        return "SENT";
+        return u"SENT";
     default:
-        return "";
+        return {};
     }
 }
 
 static QString formatted(QXmppLogger::MessageType type, const QString &text)
 {
-    return QDateTime::currentDateTime().toString() + " " +
-        QString::fromLatin1(typeName(type)) + " " +
-        text;
+    return QDateTime::currentDateTime().toString() + u' ' + typeName(type) + u' ' + text;
 }
 
 static void relaySignals(QXmppLoggable *from, QXmppLoggable *to)
@@ -96,29 +96,31 @@ public:
 };
 
 QXmppLoggerPrivate::QXmppLoggerPrivate()
-    : loggingType(QXmppLogger::NoLogging), logFile(nullptr), logFilePath("QXmppClientLog.log"), messageTypes(QXmppLogger::AnyMessage)
+    : loggingType(QXmppLogger::NoLogging),
+      logFile(nullptr),
+      logFilePath(u"QXmppClientLog.log"_s),
+      messageTypes(QXmppLogger::AnyMessage)
 {
 }
 
+///
 /// Constructs a new QXmppLogger.
 ///
 /// \param parent
-
+///
 QXmppLogger::QXmppLogger(QObject *parent)
-    : QObject(parent), d(new QXmppLoggerPrivate())
+    : QObject(parent),
+      d(std::make_unique<QXmppLoggerPrivate>())
 {
     // make it possible to pass QXmppLogger::MessageType between threads
     qRegisterMetaType<QXmppLogger::MessageType>("QXmppLogger::MessageType");
 }
 
-QXmppLogger::~QXmppLogger()
-{
-    delete d;
-}
+QXmppLogger::~QXmppLogger() = default;
 
+///
 /// Returns the default logger.
 ///
-
 QXmppLogger *QXmppLogger::getLogger()
 {
     if (!m_logger) {
@@ -134,36 +136,46 @@ QXmppLogger::LoggingType QXmppLogger::loggingType()
 }
 
 /// Sets the handler for logging messages.
-///
-/// \param type
-
 void QXmppLogger::setLoggingType(QXmppLogger::LoggingType type)
 {
     if (d->loggingType != type) {
         d->loggingType = type;
         reopen();
+        Q_EMIT loggingTypeChanged();
     }
 }
 
+///
+/// \fn QXmppLogger::loggingTypeChanged()
+///
+/// Emitted when the logging type has been changed.
+///
+/// \since QXmpp 1.7
+///
+
 QXmppLogger::MessageTypes QXmppLogger::messageTypes()
 {
     return d->messageTypes;
 }
 
 /// Sets the types of messages to log.
-///
-/// \param types
-
 void QXmppLogger::setMessageTypes(QXmppLogger::MessageTypes types)
 {
-    d->messageTypes = types;
+    if (d->messageTypes != types) {
+        d->messageTypes = types;
+        Q_EMIT messageTypesChanged();
+    }
 }
 
-/// Add a logging message.
 ///
-/// \param type
-/// \param text
+/// \fn QXmppLogger::messageTypesChanged()
+///
+/// Emitted when the message types have been changed.
+///
+/// \since QXmpp 1.7
+///
 
+/// Add a logging message.
 void QXmppLogger::log(QXmppLogger::MessageType type, const QString &text)
 {
     // filter messages
@@ -190,24 +202,26 @@ void QXmppLogger::log(QXmppLogger::MessageType type, const QString &text)
     }
 }
 
+///
 /// Sets the given \a gauge to \a value.
 ///
 /// NOTE: the base implementation does nothing.
-
+///
 void QXmppLogger::setGauge(const QString &gauge, double value)
 {
-    Q_UNUSED(gauge);
-    Q_UNUSED(value);
+    Q_UNUSED(gauge)
+    Q_UNUSED(value)
 }
 
+///
 /// Updates the given \a counter by \a amount.
 ///
 /// NOTE: the base implementation does nothing.
-
+///
 void QXmppLogger::updateCounter(const QString &counter, qint64 amount)
 {
-    Q_UNUSED(counter);
-    Q_UNUSED(amount);
+    Q_UNUSED(counter)
+    Q_UNUSED(amount)
 }
 
 QString QXmppLogger::logFilePath()
@@ -215,23 +229,31 @@ QString QXmppLogger::logFilePath()
     return d->logFilePath;
 }
 
+///
 /// Sets the path to which logging messages should be written.
 ///
 /// \param path
 ///
 /// \sa setLoggingType()
-
+///
 void QXmppLogger::setLogFilePath(const QString &path)
 {
     if (d->logFilePath != path) {
         d->logFilePath = path;
         reopen();
+        Q_EMIT logFilePathChanged();
     }
 }
 
-/// If logging to a file, causes the file to be re-opened.
+///
+/// \fn QXmppLogger::logFilePathChanged()
+///
+/// Emitted when the log file path has been changed.
+///
+/// \since QXmpp 1.7
 ///
 
+/// If logging to a file, causes the file to be re-opened.
 void QXmppLogger::reopen()
 {
     if (d->logFile) {
