@@ -5,10 +5,14 @@
 #include "QXmppMucIq.h"
 
 #include "QXmppConstants_p.h"
-#include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 
+using namespace QXmpp::Private;
+
 QXmppMucItem::QXmppMucItem()
     : m_affiliation(QXmppMucItem::UnspecifiedAffiliation),
       m_role(QXmppMucItem::UnspecifiedRole)
@@ -16,7 +20,6 @@ QXmppMucItem::QXmppMucItem()
 }
 
 /// Returns true if the current item is null.
-
 bool QXmppMucItem::isNull() const
 {
     return m_actor.isEmpty() &&
@@ -29,7 +32,6 @@ bool QXmppMucItem::isNull() const
 
 /// Returns the actor for this item, for instance the admin who kicked
 /// a user out of a room.
-
 QString QXmppMucItem::actor() const
 {
     return m_actor;
@@ -37,14 +39,12 @@ QString QXmppMucItem::actor() const
 
 /// Sets the \a actor for this item, for instance the admin who kicked
 /// a user out of a room.
-
 void QXmppMucItem::setActor(const QString &actor)
 {
     m_actor = actor;
 }
 
 /// Returns the user's affiliation, i.e. long-lived permissions.
-
 QXmppMucItem::Affiliation QXmppMucItem::affiliation() const
 {
     return m_affiliation;
@@ -53,15 +53,15 @@ QXmppMucItem::Affiliation QXmppMucItem::affiliation() const
 /// \cond
 QXmppMucItem::Affiliation QXmppMucItem::affiliationFromString(const QString &affiliationStr)
 {
-    if (affiliationStr == QStringLiteral("owner")) {
+    if (affiliationStr == u"owner") {
         return QXmppMucItem::OwnerAffiliation;
-    } else if (affiliationStr == QStringLiteral("admin")) {
+    } else if (affiliationStr == u"admin") {
         return QXmppMucItem::AdminAffiliation;
-    } else if (affiliationStr == QStringLiteral("member")) {
+    } else if (affiliationStr == u"member") {
         return QXmppMucItem::MemberAffiliation;
-    } else if (affiliationStr == QStringLiteral("outcast")) {
+    } else if (affiliationStr == u"outcast") {
         return QXmppMucItem::OutcastAffiliation;
-    } else if (affiliationStr == QStringLiteral("none")) {
+    } else if (affiliationStr == u"none") {
         return QXmppMucItem::NoAffiliation;
     } else {
         return QXmppMucItem::UnspecifiedAffiliation;
@@ -72,15 +72,15 @@ QString QXmppMucItem::affiliationToString(Affiliation affiliation)
 {
     switch (affiliation) {
     case QXmppMucItem::OwnerAffiliation:
-        return "owner";
+        return u"owner"_s;
     case QXmppMucItem::AdminAffiliation:
-        return "admin";
+        return u"admin"_s;
     case QXmppMucItem::MemberAffiliation:
-        return "member";
+        return u"member"_s;
     case QXmppMucItem::OutcastAffiliation:
-        return "outcast";
+        return u"outcast"_s;
     case QXmppMucItem::NoAffiliation:
-        return "none";
+        return u"none"_s;
     default:
         return QString();
     }
@@ -88,41 +88,30 @@ QString QXmppMucItem::affiliationToString(Affiliation affiliation)
 /// \endcond
 
 /// Sets the user's affiliation, i.e. long-lived permissions.
-///
-/// \param affiliation
-
 void QXmppMucItem::setAffiliation(Affiliation affiliation)
 {
     m_affiliation = affiliation;
 }
 
 /// Returns the user's real JID.
-
 QString QXmppMucItem::jid() const
 {
     return m_jid;
 }
 
 /// Sets the user's real JID.
-///
-/// \param jid
-
 void QXmppMucItem::setJid(const QString &jid)
 {
     m_jid = jid;
 }
 
 /// Returns the user's nickname.
-
 QString QXmppMucItem::nick() const
 {
     return m_nick;
 }
 
 /// Sets the user's nickname.
-///
-/// \param nick
-
 void QXmppMucItem::setNick(const QString &nick)
 {
     m_nick = nick;
@@ -130,7 +119,6 @@ void QXmppMucItem::setNick(const QString &nick)
 
 /// Returns the reason for this item, for example the reason for kicking
 /// a user out of a room.
-
 QString QXmppMucItem::reason() const
 {
     return m_reason;
@@ -138,14 +126,12 @@ QString QXmppMucItem::reason() const
 
 /// Sets the \a reason for this item, for example the reason for kicking
 /// a user out of a room.
-
 void QXmppMucItem::setReason(const QString &reason)
 {
     m_reason = reason;
 }
 
 /// Returns the user's role, i.e. short-lived permissions.
-
 QXmppMucItem::Role QXmppMucItem::role() const
 {
     return m_role;
@@ -154,13 +140,13 @@ QXmppMucItem::Role QXmppMucItem::role() const
 /// \cond
 QXmppMucItem::Role QXmppMucItem::roleFromString(const QString &roleStr)
 {
-    if (roleStr == QStringLiteral("moderator")) {
+    if (roleStr == u"moderator") {
         return QXmppMucItem::ModeratorRole;
-    } else if (roleStr == QStringLiteral("participant")) {
+    } else if (roleStr == u"participant") {
         return QXmppMucItem::ParticipantRole;
-    } else if (roleStr == QStringLiteral("visitor")) {
+    } else if (roleStr == u"visitor") {
         return QXmppMucItem::VisitorRole;
-    } else if (roleStr == QStringLiteral("none")) {
+    } else if (roleStr == u"none") {
         return QXmppMucItem::NoRole;
     } else {
         return QXmppMucItem::UnspecifiedRole;
@@ -171,13 +157,13 @@ QString QXmppMucItem::roleToString(Role role)
 {
     switch (role) {
     case QXmppMucItem::ModeratorRole:
-        return QStringLiteral("moderator");
+        return u"moderator"_s;
     case QXmppMucItem::ParticipantRole:
-        return QStringLiteral("participant");
+        return u"participant"_s;
     case QXmppMucItem::VisitorRole:
-        return QStringLiteral("visitor");
+        return u"visitor"_s;
     case QXmppMucItem::NoRole:
-        return QStringLiteral("none");
+        return u"none"_s;
     default:
         return QString();
     }
@@ -185,9 +171,6 @@ QString QXmppMucItem::roleToString(Role role)
 /// \endcond
 
 /// Sets the user's role, i.e. short-lived permissions.
-///
-/// \param role
-
 void QXmppMucItem::setRole(Role role)
 {
     m_role = role;
@@ -196,44 +179,40 @@ void QXmppMucItem::setRole(Role role)
 /// \cond
 void QXmppMucItem::parse(const QDomElement &element)
 {
-    m_affiliation = QXmppMucItem::affiliationFromString(element.attribute(QStringLiteral("affiliation")).toLower());
-    m_jid = element.attribute(QStringLiteral("jid"));
-    m_nick = element.attribute(QStringLiteral("nick"));
-    m_role = QXmppMucItem::roleFromString(element.attribute(QStringLiteral("role")).toLower());
-    m_actor = element.firstChildElement(QStringLiteral("actor")).attribute("jid");
-    m_reason = element.firstChildElement(QStringLiteral("reason")).text();
+    m_affiliation = QXmppMucItem::affiliationFromString(element.attribute(u"affiliation"_s).toLower());
+    m_jid = element.attribute(u"jid"_s);
+    m_nick = element.attribute(u"nick"_s);
+    m_role = QXmppMucItem::roleFromString(element.attribute(u"role"_s).toLower());
+    m_actor = element.firstChildElement(u"actor"_s).attribute(u"jid"_s);
+    m_reason = element.firstChildElement(u"reason"_s).text();
 }
 
 void QXmppMucItem::toXml(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement(QStringLiteral("item"));
-    helperToXmlAddAttribute(writer, QStringLiteral("affiliation"), affiliationToString(m_affiliation));
-    helperToXmlAddAttribute(writer, QStringLiteral("jid"), m_jid);
-    helperToXmlAddAttribute(writer, QStringLiteral("nick"), m_nick);
-    helperToXmlAddAttribute(writer, QStringLiteral("role"), roleToString(m_role));
+    writer->writeStartElement(QSL65("item"));
+    writeOptionalXmlAttribute(writer, u"affiliation", affiliationToString(m_affiliation));
+    writeOptionalXmlAttribute(writer, u"jid", m_jid);
+    writeOptionalXmlAttribute(writer, u"nick", m_nick);
+    writeOptionalXmlAttribute(writer, u"role", roleToString(m_role));
     if (!m_actor.isEmpty()) {
-        writer->writeStartElement(QStringLiteral("actor"));
-        helperToXmlAddAttribute(writer, QStringLiteral("jid"), m_actor);
+        writer->writeStartElement(QSL65("actor"));
+        writeOptionalXmlAttribute(writer, u"jid", m_actor);
         writer->writeEndElement();
     }
     if (!m_reason.isEmpty()) {
-        helperToXmlAddTextElement(writer, QStringLiteral("reason"), m_reason);
+        writeXmlTextElement(writer, u"reason", m_reason);
     }
     writer->writeEndElement();
 }
 /// \endcond
 
 /// Returns the IQ's items.
-
 QList<QXmppMucItem> QXmppMucAdminIq::items() const
 {
     return m_items;
 }
 
 /// Sets the IQ's items.
-///
-/// \param items
-
 void QXmppMucAdminIq::setItems(const QList<QXmppMucItem> &items)
 {
     m_items = items;
@@ -242,26 +221,24 @@ void QXmppMucAdminIq::setItems(const QList<QXmppMucItem> &items)
 /// \cond
 bool QXmppMucAdminIq::isMucAdminIq(const QDomElement &element)
 {
-    QDomElement queryElement = element.firstChildElement(QStringLiteral("query"));
+    QDomElement queryElement = element.firstChildElement(u"query"_s);
     return (queryElement.namespaceURI() == ns_muc_admin);
 }
 
 void QXmppMucAdminIq::parseElementFromChild(const QDomElement &element)
 {
-    QDomElement queryElement = element.firstChildElement(QStringLiteral("query"));
-    QDomElement child = queryElement.firstChildElement(QStringLiteral("item"));
-    while (!child.isNull()) {
+    QDomElement queryElement = element.firstChildElement(u"query"_s);
+    for (const auto &child : iterChildElements(queryElement, u"item")) {
         QXmppMucItem item;
         item.parse(child);
         m_items << item;
-        child = child.nextSiblingElement(QStringLiteral("item"));
     }
 }
 
 void QXmppMucAdminIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement(QStringLiteral("query"));
-    writer->writeDefaultNamespace(ns_muc_admin);
+    writer->writeStartElement(QSL65("query"));
+    writer->writeDefaultNamespace(toString65(ns_muc_admin));
     for (const QXmppMucItem &item : m_items) {
         item.toXml(writer);
     }
@@ -270,16 +247,12 @@ void QXmppMucAdminIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 /// \endcond
 
 /// Returns the IQ's data form.
-
 QXmppDataForm QXmppMucOwnerIq::form() const
 {
     return m_form;
 }
 
 /// Sets the IQ's data form.
-///
-/// \param form
-
 void QXmppMucOwnerIq::setForm(const QXmppDataForm &form)
 {
     m_form = form;
@@ -288,20 +261,20 @@ void QXmppMucOwnerIq::setForm(const QXmppDataForm &form)
 /// \cond
 bool QXmppMucOwnerIq::isMucOwnerIq(const QDomElement &element)
 {
-    QDomElement queryElement = element.firstChildElement(QStringLiteral("query"));
+    QDomElement queryElement = element.firstChildElement(u"query"_s);
     return (queryElement.namespaceURI() == ns_muc_owner);
 }
 
 void QXmppMucOwnerIq::parseElementFromChild(const QDomElement &element)
 {
-    QDomElement queryElement = element.firstChildElement(QStringLiteral("query"));
-    m_form.parse(queryElement.firstChildElement(QStringLiteral("x")));
+    QDomElement queryElement = element.firstChildElement(u"query"_s);
+    m_form.parse(queryElement.firstChildElement(u"x"_s));
 }
 
 void QXmppMucOwnerIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement(QStringLiteral("query"));
-    writer->writeDefaultNamespace(ns_muc_owner);
+    writer->writeStartElement(QSL65("query"));
+    writer->writeDefaultNamespace(toString65(ns_muc_owner));
     m_form.toXml(writer);
     writer->writeEndElement();
 }
