@@ -6,9 +6,14 @@
 
 #include "QXmppConstants_p.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 
+using namespace QXmpp::Private;
+
 ///
 /// \class QXmppPingIq
 ///
@@ -27,16 +32,14 @@ QXmppPingIq::QXmppPingIq()
 ///
 bool QXmppPingIq::isPingIq(const QDomElement &element)
 {
-    QDomElement pingElement = element.firstChildElement(QStringLiteral("ping"));
-    return (element.attribute(QStringLiteral("type")) == QStringLiteral("get") &&
-            pingElement.namespaceURI() == ns_ping);
+    return isIqType(element, u"ping", ns_ping) && element.attribute(u"type"_s) == u"get";
 }
 
 /// \cond
 void QXmppPingIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement(QStringLiteral("ping"));
-    writer->writeDefaultNamespace(ns_ping);
+    writer->writeStartElement(QSL65("ping"));
+    writer->writeDefaultNamespace(toString65(ns_ping));
     writer->writeEndElement();
 }
 /// \endcond
