@@ -9,7 +9,9 @@
 #include "QXmppPubSubIq_p.h"
 #include "QXmppPubSubSubscription.h"
 #include "QXmppResultSet.h"
-#include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QSharedData>
 
@@ -61,25 +63,25 @@ using namespace QXmpp::Private;
 /// &lt;item/&gt; is also checked.
 ///
 
-static const QStringList PUBSUB_QUERIES = {
-    QStringLiteral("affiliations"),
-    QStringLiteral("affiliations"),
-    QStringLiteral("configure"),
-    QStringLiteral("create"),
-    QStringLiteral("default"),
-    QStringLiteral("default"),
-    QStringLiteral("delete"),
-    QStringLiteral("items"),
-    QStringLiteral("options"),
-    QStringLiteral("publish"),
-    QStringLiteral("purge"),
-    QStringLiteral("retract"),
-    QStringLiteral("subscribe"),
-    QStringLiteral("subscription"),
-    QStringLiteral("subscriptions"),
-    QStringLiteral("subscriptions"),
-    QStringLiteral("unsubscribe"),
-};
+constexpr auto PUBSUB_QUERIES = to_array<QStringView>({
+    u"affiliations",
+    u"affiliations",
+    u"configure",
+    u"create",
+    u"default",
+    u"default",
+    u"delete",
+    u"items",
+    u"options",
+    u"publish",
+    u"purge",
+    u"retract",
+    u"subscribe",
+    u"subscription",
+    u"subscriptions",
+    u"subscriptions",
+    u"unsubscribe",
+});
 
 namespace QXmpp::Private {
 
@@ -322,7 +324,7 @@ bool PubSubIqBase::isPubSubIq(const QDomElement &element, bool (*isItemValid)(co
 {
     // IQs must have only one direct child element.
     const auto pubSubElement = element.firstChildElement();
-    if (pubSubElement.tagName() != QStringLiteral("pubsub")) {
+    if (pubSubElement.tagName() != u"pubsub") {
         return false;
     }
 
@@ -348,7 +350,7 @@ bool PubSubIqBase::isPubSubIq(const QDomElement &element, bool (*isItemValid)(co
     case Retract:
     case Delete:
     case Purge:
-        if (!queryElement.hasAttribute(QStringLiteral("node"))) {
+        if (!queryElement.hasAttribute(u"node"_s)) {
             return false;
         }
     default:
@@ -361,7 +363,7 @@ bool PubSubIqBase::isPubSubIq(const QDomElement &element, bool (*isItemValid)(co
     case OwnerSubscriptions:
     case Subscribe:
     case Unsubscribe:
-        if (!queryElement.hasAttribute(QStringLiteral("jid"))) {
+        if (!queryElement.hasAttribute(u"jid"_s)) {
             return false;
         }
     default:
@@ -374,9 +376,7 @@ bool PubSubIqBase::isPubSubIq(const QDomElement &element, bool (*isItemValid)(co
     case Publish:
     case Retract:
         // check the items using isItemValid()
-        for (auto itemElement = queryElement.firstChildElement(QStringLiteral("item"));
-             !itemElement.isNull();
-             itemElement = itemElement.nextSiblingElement(QStringLiteral("item"))) {
+        for (const auto &itemElement : iterChildElements(queryElement, u"item")) {
             if (!isItemValid(itemElement)) {
                 return false;
             }
@@ -410,19 +410,8 @@ bool PubSubIqBase::isPubSubIq(const QDomElement &element, bool (*isItemValid)(co
 
 void PubSubIqBase::parseElementFromChild(const QDomElement &element)
 {
-    const auto findChildElement = [](const QDomElement &element, const QString &tag, const QString &namespaceUri) {
-        for (auto subElement = element.firstChildElement(tag);
-             !subElement.isNull();
-             subElement = subElement.nextSiblingElement(tag)) {
-            if (subElement.namespaceURI() == namespaceUri) {
-                return subElement;
-            }
-        }
-        return QDomElement();
-    };
-
     const auto parseDataFormFromChild = [=](const QDomElement &element) -> std::optional<QXmppDataForm> {
-        if (const auto subElement = findChildElement(element, "x", ns_data);
+        if (const auto subElement = firstChildElement(element, u"x", ns_data);
             !subElement.isNull()) {
             QXmppDataForm form;
             form.parse(subElement);
@@ -431,7 +420,7 @@ void PubSubIqBase::parseElementFromChild(const QDomElement &element)
         return {};
     };
 
-    const auto pubSubElement = element.firstChildElement(QStringLiteral("pubsub"));
+    const auto pubSubElement = element.firstChildElement(u"pubsub"_s);
     const auto queryElement = pubSubElement.firstChildElement();
 
     // parse query type
@@ -449,19 +438,19 @@ void PubSubIqBase::parseElementFromChild(const QDomElement &element)
         setSubscription(subscription);
 
         // form inside following <options/>
-        d->dataForm = parseDataFormFromChild(pubSubElement.firstChildElement("options"));
+        d->dataForm = parseDataFormFromChild(firstChildElement(pubSubElement, u"options"));
         return;
     }
 
-    d->queryJid = queryElement.attribute(QStringLiteral("jid"));
-    d->queryNode = queryElement.attribute(QStringLiteral("node"));
+    d->queryJid = queryElement.attribute(u"jid"_s);
+    d->queryNode = queryElement.attribute(u"node"_s);
 
     // parse subid
     switch (d->queryType) {
     case Items:
     case Unsubscribe:
     case Options:
-        d->subscriptionId = queryElement.attribute(QStringLiteral("subid"));
+        d->subscriptionId = queryElement.attribute(u"subid"_s);
     default:
         break;
     }
@@ -470,9 +459,7 @@ void PubSubIqBase::parseElementFromChild(const QDomElement &element)
     switch (d->queryType) {
     case Affiliations:
     case OwnerAffiliations:
-        for (auto subElement = queryElement.firstChildElement();
-             !subElement.isNull();
-             subElement = subElement.nextSiblingElement()) {
+        for (const auto &subElement : iterChildElements(queryElement, u"affiliation")) {
             if (QXmppPubSubAffiliation::isAffiliation(subElement)) {
                 QXmppPubSubAffiliation affiliation;
                 affiliation.parse(subElement);
@@ -483,9 +470,7 @@ void PubSubIqBase::parseElementFromChild(const QDomElement &element)
         break;
     case Items:
         // Result Set Management (incomplete items result received)
-        for (auto rsmEl = pubSubElement.firstChildElement(QStringLiteral("set"));
-             !rsmEl.isNull();
-             rsmEl = rsmEl.nextSiblingElement(QStringLiteral("set"))) {
+        for (const auto &rsmEl : iterChildElements(pubSubElement, u"set")) {
             if (rsmEl.namespaceURI() == ns_rsm) {
                 QXmppResultSetReply reply;
                 reply.parse(rsmEl);
@@ -498,18 +483,16 @@ void PubSubIqBase::parseElementFromChild(const QDomElement &element)
         parseItems(queryElement);
 
         if (d->queryType == Items) {
-            d->maxItems = queryElement.attribute(QStringLiteral("max_items")).toUInt();
+            d->maxItems = queryElement.attribute(u"max_items"_s).toUInt();
         } else if (d->queryType == Publish) {
             // form inside following <publish-options/>
-            d->dataForm = parseDataFormFromChild(pubSubElement.firstChildElement("publish-options"));
+            d->dataForm = parseDataFormFromChild(firstChildElement(pubSubElement, u"publish-options"));
         }
 
         break;
     case Subscriptions:
     case OwnerSubscriptions:
-        for (auto subElement = queryElement.firstChildElement();
-             !subElement.isNull();
-             subElement = subElement.nextSiblingElement()) {
+        for (const auto &subElement : iterChildElements(queryElement)) {
             if (QXmppPubSubSubscription::isSubscription(subElement)) {
                 QXmppPubSubSubscription subscription;
                 subscription.parse(subElement);
@@ -527,12 +510,12 @@ void PubSubIqBase::parseElementFromChild(const QDomElement &element)
         break;
     case Create:
         // form inside following <configure/>
-        d->dataForm = parseDataFormFromChild(pubSubElement.firstChildElement("configure"));
+        d->dataForm = parseDataFormFromChild(firstChildElement(pubSubElement, u"configure"));
         break;
     case Subscribe:
     case Subscription:
         // form inside following <options/>
-        d->dataForm = parseDataFormFromChild(pubSubElement.firstChildElement("options"));
+        d->dataForm = parseDataFormFromChild(firstChildElement(pubSubElement, u"options"));
         break;
     case Delete:
     case Purge:
@@ -543,8 +526,8 @@ void PubSubIqBase::parseElementFromChild(const QDomElement &element)
 
 void PubSubIqBase::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement(QStringLiteral("pubsub"));
-    writer->writeDefaultNamespace(queryTypeIsOwnerIq(d->queryType) ? ns_pubsub_owner : ns_pubsub);
+    writer->writeStartElement(QSL65("pubsub"));
+    writer->writeDefaultNamespace(toString65(queryTypeIsOwnerIq(d->queryType) ? ns_pubsub_owner : ns_pubsub));
 
     // The SubscriptionQuery is special here: The query element is directly
     // handled by a QXmppPubSubSubscription.
@@ -552,16 +535,16 @@ void PubSubIqBase::toXmlElementFromChild(QXmlStreamWriter *writer) const
         subscription().value_or(QXmppPubSubSubscription()).toXml(writer);
     } else {
         // write query type
-        writer->writeStartElement(PUBSUB_QUERIES.at(d->queryType));
-        helperToXmlAddAttribute(writer, QStringLiteral("jid"), d->queryJid);
-        helperToXmlAddAttribute(writer, QStringLiteral("node"), d->queryNode);
+        writer->writeStartElement(toString65(PUBSUB_QUERIES.at(size_t(d->queryType))));
+        writeOptionalXmlAttribute(writer, u"jid", d->queryJid);
+        writeOptionalXmlAttribute(writer, u"node", d->queryNode);
 
         // write subid
         switch (d->queryType) {
         case Items:
         case Unsubscribe:
         case Options:
-            helperToXmlAddAttribute(writer, QStringLiteral("subid"), d->subscriptionId);
+            writeOptionalXmlAttribute(writer, u"subid", d->subscriptionId);
         default:
             break;
         }
@@ -576,7 +559,7 @@ void PubSubIqBase::toXmlElementFromChild(QXmlStreamWriter *writer) const
             break;
         case Items:
             if (d->maxItems > 0) {
-                writer->writeAttribute(QStringLiteral("max_items"), QString::number(d->maxItems));
+                writer->writeAttribute(QSL65("max_items"), QString::number(d->maxItems));
             }
             [[fallthrough]];
         case Publish:
@@ -632,14 +615,14 @@ void PubSubIqBase::toXmlElementFromChild(QXmlStreamWriter *writer) const
 
             switch (d->queryType) {
             case Create:
-                writeForm(writer, *form, QStringLiteral("configure"));
+                writeForm(writer, *form, u"configure"_s);
                 break;
             case Publish:
-                writeForm(writer, *form, QStringLiteral("publish-options"));
+                writeForm(writer, *form, u"publish-options"_s);
                 break;
             case Subscribe:
             case Subscription:
-                writeForm(writer, *form, QStringLiteral("options"));
+                writeForm(writer, *form, u"options"_s);
                 break;
             default:
                 break;
@@ -657,8 +640,8 @@ void PubSubIqBase::toXmlElementFromChild(QXmlStreamWriter *writer) const
 std::optional<PubSubIqBase::QueryType> PubSubIqBase::queryTypeFromDomElement(const QDomElement &element)
 {
     QueryType type;
-    if (auto index = PUBSUB_QUERIES.indexOf(element.tagName()); index != -1) {
-        type = QueryType(index);
+    if (auto queryType = enumFromString<QueryType>(PUBSUB_QUERIES, element.tagName())) {
+        type = *queryType;
     } else {
         return std::nullopt;
     }
