@@ -4,6 +4,8 @@
 
 #include "QXmppSocks.h"
 
+#include "StringLiterals.h"
+
 #include <QDataStream>
 #include <QTcpServer>
 #include <QTcpSocket>
@@ -295,7 +297,7 @@ void QXmppSocksServer::slotReadyRead()
 
         // notify of connection
         m_states.insert(socket, ReadyState);
-        Q_EMIT newConnection(socket, hostName, hostPort);
+        Q_EMIT newConnection(socket, QString::fromUtf8(hostName), hostPort);
 
         // send response
         buffer.resize(3);
