@@ -12,6 +12,9 @@
 #include "QXmppE2eeMetadata.h"
 #include "QXmppStanza_p.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDateTime>
 #include <QDomElement>
@@ -29,108 +32,108 @@ QString conditionToString(QXmppStanza::Error::Condition condition)
     case QXmppStanza::Error::NoCondition:
         return {};
     case QXmppStanza::Error::BadRequest:
-        return QStringLiteral("bad-request");
+        return u"bad-request"_s;
     case QXmppStanza::Error::Conflict:
-        return QStringLiteral("conflict");
+        return u"conflict"_s;
     case QXmppStanza::Error::FeatureNotImplemented:
-        return QStringLiteral("feature-not-implemented");
+        return u"feature-not-implemented"_s;
     case QXmppStanza::Error::Forbidden:
-        return QStringLiteral("forbidden");
+        return u"forbidden"_s;
     case QXmppStanza::Error::Gone:
-        return QStringLiteral("gone");
+        return u"gone"_s;
     case QXmppStanza::Error::InternalServerError:
-        return QStringLiteral("internal-server-error");
+        return u"internal-server-error"_s;
     case QXmppStanza::Error::ItemNotFound:
-        return QStringLiteral("item-not-found");
+        return u"item-not-found"_s;
     case QXmppStanza::Error::JidMalformed:
-        return QStringLiteral("jid-malformed");
+        return u"jid-malformed"_s;
     case QXmppStanza::Error::NotAcceptable:
-        return QStringLiteral("not-acceptable");
+        return u"not-acceptable"_s;
     case QXmppStanza::Error::NotAllowed:
-        return QStringLiteral("not-allowed");
+        return u"not-allowed"_s;
     case QXmppStanza::Error::NotAuthorized:
-        return QStringLiteral("not-authorized");
+        return u"not-authorized"_s;
         QT_WARNING_PUSH
         QT_WARNING_DISABLE_DEPRECATED
     case QXmppStanza::Error::PaymentRequired:
         QT_WARNING_POP
-        return QStringLiteral("payment-required");
+        return u"payment-required"_s;
     case QXmppStanza::Error::PolicyViolation:
-        return QStringLiteral("policy-violation");
+        return u"policy-violation"_s;
     case QXmppStanza::Error::RecipientUnavailable:
-        return QStringLiteral("recipient-unavailable");
+        return u"recipient-unavailable"_s;
     case QXmppStanza::Error::Redirect:
-        return QStringLiteral("redirect");
+        return u"redirect"_s;
     case QXmppStanza::Error::RegistrationRequired:
-        return QStringLiteral("registration-required");
+        return u"registration-required"_s;
     case QXmppStanza::Error::RemoteServerNotFound:
-        return QStringLiteral("remote-server-not-found");
+        return u"remote-server-not-found"_s;
     case QXmppStanza::Error::RemoteServerTimeout:
-        return QStringLiteral("remote-server-timeout");
+        return u"remote-server-timeout"_s;
     case QXmppStanza::Error::ResourceConstraint:
-        return QStringLiteral("resource-constraint");
+        return u"resource-constraint"_s;
     case QXmppStanza::Error::ServiceUnavailable:
-        return QStringLiteral("service-unavailable");
+        return u"service-unavailable"_s;
     case QXmppStanza::Error::SubscriptionRequired:
-        return QStringLiteral("subscription-required");
+        return u"subscription-required"_s;
     case QXmppStanza::Error::UndefinedCondition:
-        return QStringLiteral("undefined-condition");
+        return u"undefined-condition"_s;
     case QXmppStanza::Error::UnexpectedRequest:
-        return QStringLiteral("unexpected-request");
+        return u"unexpected-request"_s;
     }
     return {};
 }
 
 std::optional<QXmppStanza::Error::Condition> conditionFromString(const QString &string)
 {
-    if (string == "bad-request") {
+    if (string == u"bad-request") {
         return QXmppStanza::Error::BadRequest;
-    } else if (string == "conflict") {
+    } else if (string == u"conflict") {
         return QXmppStanza::Error::Conflict;
-    } else if (string == "feature-not-implemented") {
+    } else if (string == u"feature-not-implemented") {
         return QXmppStanza::Error::FeatureNotImplemented;
-    } else if (string == "forbidden") {
+    } else if (string == u"forbidden") {
         return QXmppStanza::Error::Forbidden;
-    } else if (string == "gone") {
+    } else if (string == u"gone") {
         return QXmppStanza::Error::Gone;
-    } else if (string == "internal-server-error") {
+    } else if (string == u"internal-server-error") {
         return QXmppStanza::Error::InternalServerError;
-    } else if (string == "item-not-found") {
+    } else if (string == u"item-not-found") {
         return QXmppStanza::Error::ItemNotFound;
-    } else if (string == "jid-malformed") {
+    } else if (string == u"jid-malformed") {
         return QXmppStanza::Error::JidMalformed;
-    } else if (string == "not-acceptable") {
+    } else if (string == u"not-acceptable") {
         return QXmppStanza::Error::NotAcceptable;
-    } else if (string == "not-allowed") {
+    } else if (string == u"not-allowed") {
         return QXmppStanza::Error::NotAllowed;
-    } else if (string == "not-authorized") {
+    } else if (string == u"not-authorized") {
         return QXmppStanza::Error::NotAuthorized;
-    } else if (string == "payment-required") {
+    } else if (string == u"payment-required") {
         QT_WARNING_PUSH
         QT_WARNING_DISABLE_DEPRECATED
         return QXmppStanza::Error::PaymentRequired;
         QT_WARNING_POP
-    } else if (string == "policy-violation") {
+    } else if (string == u"policy-violation") {
         return QXmppStanza::Error::PolicyViolation;
-    } else if (string == "recipient-unavailable") {
+    } else if (string == u"recipient-unavailable") {
         return QXmppStanza::Error::RecipientUnavailable;
-    } else if (string == "redirect") {
+    } else if (string == u"redirect") {
         return QXmppStanza::Error::Redirect;
-    } else if (string == "registration-required") {
+    } else if (string == u"registration-required") {
         return QXmppStanza::Error::RegistrationRequired;
-    } else if (string == "remote-server-not-found") {
+    } else if (string == u"remote-server-not-found") {
         return QXmppStanza::Error::RemoteServerNotFound;
-    } else if (string == "remote-server-timeout") {
+    } else if (string == u"remote-server-timeout") {
         return QXmppStanza::Error::RemoteServerTimeout;
-    } else if (string == "resource-constraint") {
+    } else if (string == u"resource-constraint") {
         return QXmppStanza::Error::ResourceConstraint;
-    } else if (string == "service-unavailable") {
+    } else if (string == u"service-unavailable") {
         return QXmppStanza::Error::ServiceUnavailable;
-    } else if (string == "subscription-required") {
+    } else if (string == u"subscription-required") {
         return QXmppStanza::Error::SubscriptionRequired;
-    } else if (string == "undefined-condition") {
+    } else if (string == u"undefined-condition") {
         return QXmppStanza::Error::UndefinedCondition;
-    } else if (string == "unexpected-request") {
+    } else if (string == u"unexpected-request") {
         return QXmppStanza::Error::UnexpectedRequest;
     }
     return std::nullopt;
@@ -142,30 +145,30 @@ QString typeToString(QXmppStanza::Error::Type type)
     case QXmppStanza::Error::NoType:
         return {};
     case QXmppStanza::Error::Cancel:
-        return QStringLiteral("cancel");
+        return u"cancel"_s;
     case QXmppStanza::Error::Continue:
-        return QStringLiteral("continue");
+        return u"continue"_s;
     case QXmppStanza::Error::Modify:
-        return QStringLiteral("modify");
+        return u"modify"_s;
     case QXmppStanza::Error::Auth:
-        return QStringLiteral("auth");
+        return u"auth"_s;
     case QXmppStanza::Error::Wait:
-        return QStringLiteral("wait");
+        return u"wait"_s;
     }
     return {};
 }
 
 std::optional<QXmppStanza::Error::Type> typeFromString(const QString &string)
 {
-    if (string == QStringLiteral("cancel")) {
+    if (string == u"cancel") {
         return QXmppStanza::Error::Cancel;
-    } else if (string == QStringLiteral("continue")) {
+    } else if (string == u"continue") {
         return QXmppStanza::Error::Continue;
-    } else if (string == QStringLiteral("modify")) {
+    } else if (string == u"modify") {
         return QXmppStanza::Error::Modify;
-    } else if (string == QStringLiteral("auth")) {
+    } else if (string == u"auth") {
         return QXmppStanza::Error::Auth;
-    } else if (string == QStringLiteral("wait")) {
+    } else if (string == u"wait") {
         return QXmppStanza::Error::Wait;
     }
     return std::nullopt;
@@ -277,23 +280,23 @@ bool QXmppExtendedAddress::isValid() const
 /// \cond
 void QXmppExtendedAddress::parse(const QDomElement &element)
 {
-    d->delivered = element.attribute(QStringLiteral("delivered")) == QStringLiteral("true");
-    d->description = element.attribute(QStringLiteral("desc"));
-    d->jid = element.attribute(QStringLiteral("jid"));
-    d->type = element.attribute(QStringLiteral("type"));
+    d->delivered = element.attribute(u"delivered"_s) == u"true";
+    d->description = element.attribute(u"desc"_s);
+    d->jid = element.attribute(u"jid"_s);
+    d->type = element.attribute(u"type"_s);
 }
 
 void QXmppExtendedAddress::toXml(QXmlStreamWriter *xmlWriter) const
 {
-    xmlWriter->writeStartElement(QStringLiteral("address"));
+    xmlWriter->writeStartElement(QSL65("address"));
     if (d->delivered) {
-        xmlWriter->writeAttribute(QStringLiteral("delivered"), QStringLiteral("true"));
+        xmlWriter->writeAttribute(QSL65("delivered"), u"true"_s);
     }
     if (!d->description.isEmpty()) {
-        xmlWriter->writeAttribute(QStringLiteral("desc"), d->description);
+        xmlWriter->writeAttribute(QSL65("desc"), d->description);
     }
-    xmlWriter->writeAttribute(QStringLiteral("jid"), d->jid);
-    xmlWriter->writeAttribute(QStringLiteral("type"), d->type);
+    xmlWriter->writeAttribute(QSL65("jid"), d->jid);
+    xmlWriter->writeAttribute(QSL65("type"), d->type);
     xmlWriter->writeEndElement();
 }
 /// \endcond
@@ -553,14 +556,13 @@ void QXmppStanza::Error::setRetryDate(const QDateTime &retryDate)
 /// \cond
 void QXmppStanza::Error::parse(const QDomElement &errorElement)
 {
-    d->code = errorElement.attribute(QStringLiteral("code")).toInt();
-    d->type = typeFromString(errorElement.attribute(QStringLiteral("type"))).value_or(NoType);
-    d->by = errorElement.attribute(QStringLiteral("by"));
+    d->code = errorElement.attribute(u"code"_s).toInt();
+    d->type = typeFromString(errorElement.attribute(u"type"_s)).value_or(NoType);
+    d->by = errorElement.attribute(u"by"_s);
 
-    QDomElement element = errorElement.firstChildElement();
-    while (!element.isNull()) {
+    for (const auto &element : iterChildElements(errorElement)) {
         if (element.namespaceURI() == ns_stanza) {
-            if (element.tagName() == QStringLiteral("text")) {
+            if (element.tagName() == u"text") {
                 d->text = element.text();
             } else {
                 d->condition = conditionFromString(element.tagName()).value_or(NoCondition);
@@ -578,18 +580,17 @@ void QXmppStanza::Error::parse(const QDomElement &errorElement)
         } else if (element.namespaceURI() == ns_http_upload) {
             // XEP-0363: HTTP File Upload
             // file is too large
-            if (element.tagName() == QStringLiteral("file-too-large")) {
+            if (element.tagName() == u"file-too-large") {
                 d->fileTooLarge = true;
-                d->maxFileSize = element.firstChildElement(QStringLiteral("max-file-size"))
+                d->maxFileSize = element.firstChildElement(u"max-file-size"_s)
                                      .text()
                                      .toLongLong();
                 // retry later
-            } else if (element.tagName() == QStringLiteral("retry")) {
+            } else if (element.tagName() == u"retry") {
                 d->retryDate = QXmppUtils::datetimeFromString(
-                    element.attribute(QStringLiteral("stamp")));
+                    element.attribute(u"stamp"_s));
             }
         }
-        element = element.nextSiblingElement();
     }
 }
 
@@ -599,19 +600,19 @@ void QXmppStanza::Error::toXml(QXmlStreamWriter *writer) const
         return;
     }
 
-    writer->writeStartElement(QStringLiteral("error"));
-    helperToXmlAddAttribute(writer, QStringLiteral("by"), d->by);
+    writer->writeStartElement(QSL65("error"));
+    writeOptionalXmlAttribute(writer, u"by", d->by);
     if (d->type != NoType) {
-        writer->writeAttribute(QStringLiteral("type"), typeToString(d->type));
+        writer->writeAttribute(QSL65("type"), typeToString(d->type));
     }
 
     if (d->code > 0) {
-        helperToXmlAddAttribute(writer, QStringLiteral("code"), QString::number(d->code));
+        writeOptionalXmlAttribute(writer, u"code", QString::number(d->code));
     }
 
     if (d->condition != NoCondition) {
         writer->writeStartElement(conditionToString(d->condition));
-        writer->writeDefaultNamespace(ns_stanza);
+        writer->writeDefaultNamespace(toString65(ns_stanza));
 
         // redirection URI
         if (!d->redirectionUri.isEmpty() && (d->condition == Gone || d->condition == Redirect)) {
@@ -621,24 +622,24 @@ void QXmppStanza::Error::toXml(QXmlStreamWriter *writer) const
         writer->writeEndElement();
     }
     if (!d->text.isEmpty()) {
-        writer->writeStartElement(QStringLiteral("text"));
-        writer->writeAttribute(QStringLiteral("xml:lang"), QStringLiteral("en"));
-        writer->writeDefaultNamespace(ns_stanza);
+        writer->writeStartElement(QSL65("text"));
+        writer->writeAttribute(QSL65("xml:lang"), u"en"_s);
+        writer->writeDefaultNamespace(toString65(ns_stanza));
         writer->writeCharacters(d->text);
         writer->writeEndElement();
     }
 
     // XEP-0363: HTTP File Upload
     if (d->fileTooLarge) {
-        writer->writeStartElement(QStringLiteral("file-too-large"));
-        writer->writeDefaultNamespace(ns_http_upload);
-        helperToXmlAddTextElement(writer, QStringLiteral("max-file-size"),
-                                  QString::number(d->maxFileSize));
+        writer->writeStartElement(QSL65("file-too-large"));
+        writer->writeDefaultNamespace(toString65(ns_http_upload));
+        writeXmlTextElement(writer, u"max-file-size",
+                            QString::number(d->maxFileSize));
         writer->writeEndElement();
     } else if (!d->retryDate.isNull() && d->retryDate.isValid()) {
-        writer->writeStartElement(QStringLiteral("retry"));
-        writer->writeDefaultNamespace(ns_http_upload);
-        writer->writeAttribute(QStringLiteral("stamp"),
+        writer->writeStartElement(QSL65("retry"));
+        writer->writeDefaultNamespace(toString65(ns_http_upload));
+        writer->writeAttribute(QSL65("stamp"),
                                QXmppUtils::datetimeToString(d->retryDate));
         writer->writeEndElement();
     }
@@ -1015,17 +1016,17 @@ void QXmppStanza::generateAndSetNextId()
 {
     // get back
     ++s_uniqeIdNo;
-    d->id = "qxmpp" + QString::number(s_uniqeIdNo);
+    d->id = u"qxmpp" + QString::number(s_uniqeIdNo);
 }
 
 void QXmppStanza::parse(const QDomElement &element)
 {
-    d->from = element.attribute("from");
-    d->to = element.attribute("to");
-    d->id = element.attribute("id");
-    d->lang = element.attribute("lang");
+    d->from = element.attribute(u"from"_s);
+    d->to = element.attribute(u"to"_s);
+    d->id = element.attribute(u"id"_s);
+    d->lang = element.attribute(u"lang"_s);
 
-    QDomElement errorElement = element.firstChildElement("error");
+    QDomElement errorElement = firstChildElement(element, u"error");
     if (!errorElement.isNull()) {
         Error error;
         error.parse(errorElement);
@@ -1033,14 +1034,12 @@ void QXmppStanza::parse(const QDomElement &element)
     }
 
     // XEP-0033: Extended Stanza Addressing
-    QDomElement addressElement = element.firstChildElement("addresses").firstChildElement("address");
-    while (!addressElement.isNull()) {
+    for (const auto &addressElement : iterChildElements(firstChildElement(element, u"addresses"), u"address")) {
         QXmppExtendedAddress address;
         address.parse(addressElement);
         if (address.isValid()) {
             d->extendedAddresses << address;
         }
-        addressElement = addressElement.nextSiblingElement("address");
     }
 }
 
@@ -1048,8 +1047,8 @@ void QXmppStanza::extensionsToXml(QXmlStreamWriter *xmlWriter, QXmpp::SceMode sc
 {
     // XEP-0033: Extended Stanza Addressing
     if (sceMode & QXmpp::ScePublic && !d->extendedAddresses.isEmpty()) {
-        xmlWriter->writeStartElement("addresses");
-        xmlWriter->writeDefaultNamespace(ns_extended_addressing);
+        xmlWriter->writeStartElement(QSL65("addresses"));
+        xmlWriter->writeDefaultNamespace(toString65(ns_extended_addressing));
         for (const auto &address : d->extendedAddresses) {
             address.toXml(xmlWriter);
         }
