@@ -1,43 +1,29 @@
-/*
- * Copyright (C) 2008-2020 The QXmpp developers
- *
- * Authors:
- *  Manjeet Dahiya
- *  Jeremy Lainé
- *  Georg Rudoy
- *
- * Source:
- *  https://github.com/qxmpp-project/qxmpp
- *
- * This file is a part of QXmpp library.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- */
+// SPDX-FileCopyrightText: 2009 Manjeet Dahiya <manjeetdahiya@gmail.com>
+// SPDX-FileCopyrightText: 2010 Jeremy Lainé <jeremy.laine@m4x.org>
+// SPDX-FileCopyrightText: 2015 Georg Rudoy <0xd34df00d@gmail.com>
+// SPDX-FileCopyrightText: 2019 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2022 Melvin Keskin <melvo@olomono.de>
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #ifndef QXMPPSTANZA_H
 #define QXMPPSTANZA_H
 
+#include <optional>
+
 #include <QByteArray>
 #include <QSharedData>
-#include <QString>
 
 // forward declarations of QXmlStream* classes will not work on Mac, we need to
 // include the whole header.
 // See http://lists.trolltech.com/qt-interest/2008-07/thread00798-0.html
 // for an explanation.
 #include "QXmppElement.h"
+#include "QXmppNonza.h"
 
 #include <QXmlStreamWriter>
 
+class QXmppE2eeMetadata;
 class QXmppExtendedAddressPrivate;
 
 ///
@@ -54,9 +40,11 @@ class QXMPP_EXPORT QXmppExtendedAddress
 public:
     QXmppExtendedAddress();
     QXmppExtendedAddress(const QXmppExtendedAddress &);
+    QXmppExtendedAddress(QXmppExtendedAddress &&);
     ~QXmppExtendedAddress();
 
     QXmppExtendedAddress &operator=(const QXmppExtendedAddress &);
+    QXmppExtendedAddress &operator=(QXmppExtendedAddress &&);
 
     QString description() const;
     void setDescription(const QString &description);
@@ -96,7 +84,7 @@ class QXmppStanzaErrorPrivate;
 ///
 /// \ingroup Stanzas
 ///
-class QXMPP_EXPORT QXmppStanza
+class QXMPP_EXPORT QXmppStanza : public QXmppNonza
 {
 public:
     ///
@@ -110,6 +98,7 @@ public:
         /// The error descriptions are not detailed in here. The exact meaning
         /// can be found in the particular protocols using them.
         enum Type {
+            NoType = -1,
             Cancel,    ///< The error is not temporary.
             Continue,  ///< The error was only a warning.
             Modify,    ///< The request needs to be changed and resent.
@@ -119,6 +108,7 @@ public:
 
         /// A detailed condition of the error
         enum Condition {
+            NoCondition = -1,
             BadRequest,             ///< The request does not contain a valid schema.
             Conflict,               ///< The request conflicts with another.
             FeatureNotImplemented,  ///< The feature is not implemented.
@@ -150,11 +140,16 @@ public:
 
         Error();
         Error(const Error &);
+        Error(Error &&);
         Error(Type type, Condition cond, const QString &text = QString());
         Error(const QString &type, const QString &cond, const QString &text = QString());
+        /// \cond
+        Error(QSharedDataPointer<QXmppStanzaErrorPrivate> d);
+        /// \endcond
         ~Error();
 
         Error &operator=(const Error &);
+        Error &operator=(Error &&);
 
         int code() const;
         void setCode(int code);
@@ -165,8 +160,8 @@ public:
         Condition condition() const;
         void setCondition(Condition cond);
 
-        void setType(Type type);
         Type type() const;
+        void setType(Type type);
 
         QString by() const;
         void setBy(const QString &by);
@@ -190,20 +185,18 @@ public:
         /// \endcond
 
     private:
-        QString getConditionStr() const;
-        void setConditionFromStr(const QString &cond);
-
-        QString getTypeStr() const;
-        void setTypeFromStr(const QString &type);
+        friend class QXmppStanza;
 
         QSharedDataPointer<QXmppStanzaErrorPrivate> d;
     };
 
     QXmppStanza(const QString &from = QString(), const QString &to = QString());
     QXmppStanza(const QXmppStanza &other);
-    virtual ~QXmppStanza();
+    QXmppStanza(QXmppStanza &&);
+    ~QXmppStanza() override;
 
     QXmppStanza &operator=(const QXmppStanza &other);
+    QXmppStanza &operator=(QXmppStanza &&);
 
     QString to() const;
     void setTo(const QString &);
@@ -218,7 +211,9 @@ public:
     void setLang(const QString &);
 
     QXmppStanza::Error error() const;
+    std::optional<Error> errorOptional() const;
     void setError(const QXmppStanza::Error &error);
+    void setError(const std::optional<Error> &error);
 
     QXmppElementList extensions() const;
     void setExtensions(const QXmppElementList &elements);
@@ -226,20 +221,21 @@ public:
     QList<QXmppExtendedAddress> extendedAddresses() const;
     void setExtendedAddresses(const QList<QXmppExtendedAddress> &extendedAddresses);
 
-    virtual bool isXmppStanza() const;
+    std::optional<QXmppE2eeMetadata> e2eeMetadata() const;
+    void setE2eeMetadata(const std::optional<QXmppE2eeMetadata> &e2eeMetadata);
 
     /// \cond
-    virtual void parse(const QDomElement &element);
-    virtual void toXml(QXmlStreamWriter *writer) const = 0;
+    void parse(const QDomElement &element) override;
 
 protected:
-    void extensionsToXml(QXmlStreamWriter *writer) const;
+    void extensionsToXml(QXmlStreamWriter *writer, QXmpp::SceMode = QXmpp::SceAll) const;
     void generateAndSetNextId();
     /// \endcond
 
 private:
     QSharedDataPointer<QXmppStanzaPrivate> d;
     static uint s_uniqeIdNo;
+    friend class TestClient;
 };
 
 Q_DECLARE_METATYPE(QXmppStanza::Error::Type);
