@@ -5,9 +5,16 @@
 #include "QXmppStreamFeatures.h"
 
 #include "QXmppConstants_p.h"
+#include "QXmppGlobal_p.h"
+#include "QXmppSasl_p.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 
+using namespace QXmpp::Private;
+
 class QXmppStreamFeaturesPrivate : public QSharedData
 {
 public:
@@ -24,6 +31,7 @@ public:
     bool rosterVersioningSupported;
     QStringList authMechanisms;
     QStringList compressionMethods;
+    std::optional<Sasl2::StreamFeature> sasl2Feature;
 };
 
 QXmppStreamFeaturesPrivate::QXmppStreamFeaturesPrivate()
@@ -118,6 +126,22 @@ void QXmppStreamFeatures::setAuthMechanisms(const QStringList &mechanisms)
     d->authMechanisms = mechanisms;
 }
 
+///
+/// Returns the \xep{0388, Extensible SASL Profile} stream feature.
+///
+const std::optional<Sasl2::StreamFeature> &QXmppStreamFeatures::sasl2Feature() const
+{
+    return d->sasl2Feature;
+}
+
+///
+/// Sets the \xep{0388, Extensible SASL Profile} stream feature.
+///
+void QXmppStreamFeatures::setSasl2Feature(const std::optional<Sasl2::StreamFeature> &feature)
+{
+    d->sasl2Feature = feature;
+}
+
 ///
 /// Returns the available compression methods
 ///
@@ -261,121 +285,102 @@ void QXmppStreamFeatures::setRosterVersioningSupported(bool supported)
 bool QXmppStreamFeatures::isStreamFeatures(const QDomElement &element)
 {
     return element.namespaceURI() == ns_stream &&
-        element.tagName() == QStringLiteral("features");
+        element.tagName() == u"features";
 }
 
-static QXmppStreamFeatures::Mode readFeature(const QDomElement &element, const char *tagName, const char *tagNs)
+static QXmppStreamFeatures::Mode readFeature(const QDomElement &element, QStringView tagName, QStringView tagNs)
 {
-    QDomElement subElement = element.firstChildElement(tagName);
-    QXmppStreamFeatures::Mode mode = QXmppStreamFeatures::Disabled;
-    while (!subElement.isNull()) {
-        if (subElement.namespaceURI() == tagNs) {
-            if (!subElement.firstChildElement(QStringLiteral("required")).isNull()) {
-                mode = QXmppStreamFeatures::Required;
-            } else if (mode != QXmppStreamFeatures::Required) {
-                mode = QXmppStreamFeatures::Enabled;
-            }
+    if (auto subEl = firstChildElement(element, tagName, tagNs); !subEl.isNull()) {
+        if (!firstChildElement(subEl, u"required").isNull()) {
+            return QXmppStreamFeatures::Required;
         }
-        subElement = subElement.nextSiblingElement(tagName);
+        return QXmppStreamFeatures::Enabled;
     }
-    return mode;
-}
-
-static bool readBooleanFeature(const QDomElement &element, const QString &tagName, const QString &xmlns)
-{
-    auto childElement = element.firstChildElement(tagName);
-    while (!childElement.isNull()) {
-        if (childElement.namespaceURI() == xmlns) {
-            return true;
-        }
-        childElement = childElement.nextSiblingElement(tagName);
-    }
-    return false;
+    return QXmppStreamFeatures::Disabled;
 }
 
 void QXmppStreamFeatures::parse(const QDomElement &element)
 {
-    d->bindMode = readFeature(element, "bind", ns_bind);
-    d->sessionMode = readFeature(element, "session", ns_session);
-    d->nonSaslAuthMode = readFeature(element, "auth", ns_authFeature);
-    d->tlsMode = readFeature(element, "starttls", ns_tls);
-    d->streamManagementMode = readFeature(element, "sm", ns_stream_management);
-    d->csiMode = readFeature(element, "csi", ns_csi);
-    d->registerMode = readFeature(element, "register", ns_register_feature);
-    d->preApprovedSubscriptionsSupported = readBooleanFeature(element, QStringLiteral("sub"), ns_pre_approval);
-    d->rosterVersioningSupported = readBooleanFeature(element, QStringLiteral("ver"), ns_rosterver);
+    d->bindMode = readFeature(element, u"bind", ns_bind);
+    d->sessionMode = readFeature(element, u"session", ns_session);
+    d->nonSaslAuthMode = readFeature(element, u"auth", ns_authFeature);
+    d->tlsMode = readFeature(element, u"starttls", ns_tls);
+    d->streamManagementMode = readFeature(element, u"sm", ns_stream_management);
+    d->csiMode = readFeature(element, u"csi", ns_csi);
+    d->registerMode = readFeature(element, u"register", ns_register_feature);
+    d->preApprovedSubscriptionsSupported = !firstChildElement(element, u"sub", ns_pre_approval).isNull();
+    d->rosterVersioningSupported = !firstChildElement(element, u"ver", ns_rosterver).isNull();
 
     // parse advertised compression methods
-    QDomElement compression = element.firstChildElement(QStringLiteral("compression"));
-    if (compression.namespaceURI() == ns_compressFeature) {
-        QDomElement subElement = compression.firstChildElement(QStringLiteral("method"));
-        while (!subElement.isNull()) {
-            d->compressionMethods << subElement.text();
-            subElement = subElement.nextSiblingElement(QStringLiteral("method"));
-        }
+    auto compression = firstChildElement(element, u"compression", ns_compressFeature);
+    for (const auto &subElement : iterChildElements(compression, u"method")) {
+        d->compressionMethods << subElement.text();
     }
 
     // parse advertised SASL Authentication mechanisms
-    QDomElement mechs = element.firstChildElement(QStringLiteral("mechanisms"));
-    if (mechs.namespaceURI() == ns_sasl) {
-        QDomElement subElement = mechs.firstChildElement(QStringLiteral("mechanism"));
-        while (!subElement.isNull()) {
-            d->authMechanisms << subElement.text();
-            subElement = subElement.nextSiblingElement(QStringLiteral("mechanism"));
-        }
+    auto mechs = firstChildElement(element, u"mechanisms", ns_sasl);
+    for (const auto &subElement : iterChildElements(mechs, u"mechanism")) {
+        d->authMechanisms << subElement.text();
     }
+
+    d->sasl2Feature = Sasl2::StreamFeature::fromDom(firstChildElement(element, u"authentication", ns_sasl_2));
 }
 
-static void writeFeature(QXmlStreamWriter *writer, const char *tagName, const char *tagNs, QXmppStreamFeatures::Mode mode)
+static void writeFeature(QXmlStreamWriter *writer, QStringView tagName, QStringView tagNs, QXmppStreamFeatures::Mode mode)
 {
     if (mode != QXmppStreamFeatures::Disabled) {
-        writer->writeStartElement(tagName);
-        writer->writeDefaultNamespace(tagNs);
+        writer->writeStartElement(toString65(tagName));
+        writer->writeDefaultNamespace(toString65(tagNs));
         if (mode == QXmppStreamFeatures::Required) {
-            writer->writeEmptyElement(QStringLiteral("required"));
+            writer->writeEmptyElement(u"required"_s);
         }
         writer->writeEndElement();
     }
 }
 
-static void writeBoolenFeature(QXmlStreamWriter *writer, const QString &tagName, const QString &xmlns, bool enabled)
+static void writeBoolenFeature(QXmlStreamWriter *writer, QStringView tagName, QStringView xmlns, bool enabled)
 {
     if (enabled) {
-        writer->writeStartElement(tagName);
-        writer->writeDefaultNamespace(xmlns);
+        writer->writeStartElement(toString65(tagName));
+        writer->writeDefaultNamespace(toString65(xmlns));
         writer->writeEndElement();
     }
 }
 
 void QXmppStreamFeatures::toXml(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement(QStringLiteral("stream:features"));
-    writeFeature(writer, "bind", ns_bind, d->bindMode);
-    writeFeature(writer, "session", ns_session, d->sessionMode);
-    writeFeature(writer, "auth", ns_authFeature, d->nonSaslAuthMode);
-    writeFeature(writer, "starttls", ns_tls, d->tlsMode);
-    writeFeature(writer, "sm", ns_stream_management, d->streamManagementMode);
-    writeFeature(writer, "csi", ns_csi, d->csiMode);
-    writeFeature(writer, "register", ns_register_feature, d->registerMode);
-    writeBoolenFeature(writer, QStringLiteral("sub"), ns_pre_approval, d->preApprovedSubscriptionsSupported);
-    writeBoolenFeature(writer, QStringLiteral("ver"), ns_rosterver, d->rosterVersioningSupported);
+    writer->writeStartElement(QSL65("stream:features"));
+    writeFeature(writer, u"bind", ns_bind, d->bindMode);
+    writeFeature(writer, u"session", ns_session, d->sessionMode);
+    writeFeature(writer, u"auth", ns_authFeature, d->nonSaslAuthMode);
+    writeFeature(writer, u"starttls", ns_tls, d->tlsMode);
+    writeFeature(writer, u"sm", ns_stream_management, d->streamManagementMode);
+    writeFeature(writer, u"csi", ns_csi, d->csiMode);
+    writeFeature(writer, u"register", ns_register_feature, d->registerMode);
+    writeBoolenFeature(writer, u"sub", ns_pre_approval, d->preApprovedSubscriptionsSupported);
+    writeBoolenFeature(writer, u"ver", ns_rosterver, d->rosterVersioningSupported);
 
     if (!d->compressionMethods.isEmpty()) {
-        writer->writeStartElement(QStringLiteral("compression"));
-        writer->writeDefaultNamespace(ns_compressFeature);
+        writer->writeStartElement(QSL65("compression"));
+        writer->writeDefaultNamespace(toString65(ns_compressFeature));
         for (const auto &method : std::as_const(d->compressionMethods)) {
-            writer->writeTextElement(QStringLiteral("method"), method);
+            writer->writeTextElement(QSL65("method"), method);
         }
         writer->writeEndElement();
     }
     if (!d->authMechanisms.isEmpty()) {
-        writer->writeStartElement(QStringLiteral("mechanisms"));
-        writer->writeDefaultNamespace(ns_sasl);
+        writer->writeStartElement(QSL65("mechanisms"));
+        writer->writeDefaultNamespace(toString65(ns_sasl));
         for (const auto &mechanism : std::as_const(d->authMechanisms)) {
-            writer->writeTextElement(QStringLiteral("mechanism"), mechanism);
+            writer->writeTextElement(QSL65("mechanism"), mechanism);
         }
         writer->writeEndElement();
     }
+
+    if (d->sasl2Feature) {
+        d->sasl2Feature->toXml(writer);
+    }
+
     writer->writeEndElement();
 }
 /// \endcond
