@@ -1,25 +1,6 @@
-/*
- * Copyright (C) 2008-2021 The QXmpp developers
- *
- * Author:
- *  Manjeet Dahiya
- *
- * Source:
- *  https://github.com/qxmpp-project/qxmpp
- *
- * This file is a part of QXmpp library.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- */
+// SPDX-FileCopyrightText: 2010 Manjeet Dahiya <manjeetdahiya@gmail.com>
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #ifndef QXMPPVCARDIQ_H
 #define QXMPPVCARDIQ_H
@@ -30,6 +11,10 @@
 #include <QDomElement>
 #include <QMap>
 
+namespace QXmpp::Private {
+struct VCardData;
+}
+
 class QXmppVCardAddressPrivate;
 class QXmppVCardEmailPrivate;
 class QXmppVCardPhonePrivate;
@@ -53,9 +38,11 @@ public:
 
     QXmppVCardAddress();
     QXmppVCardAddress(const QXmppVCardAddress &other);
+    QXmppVCardAddress(QXmppVCardAddress &&);
     ~QXmppVCardAddress();
 
     QXmppVCardAddress &operator=(const QXmppVCardAddress &other);
+    QXmppVCardAddress &operator=(QXmppVCardAddress &&);
 
     QString country() const;
     void setCountry(const QString &country);
@@ -279,9 +266,12 @@ public:
 
     /// \cond
     static bool isVCard(const QDomElement &element);
+    static bool checkIqType(const QString &tagName, const QString &xmlNamespace);
     /// \endcond
 
 protected:
+    friend struct QXmpp::Private::VCardData;
+
     /// \cond
     void parseElementFromChild(const QDomElement &) override;
     void toXmlElementFromChild(QXmlStreamWriter *writer) const override;
