@@ -57,8 +57,7 @@
 /// rate limiter.
 ///
 
-struct PastRequest
-{
+struct PastRequest {
     QString bareJid;
     QDateTime timestamp;
 };
@@ -97,7 +96,7 @@ QXmppAttentionManager::~QXmppAttentionManager() = default;
 QStringList QXmppAttentionManager::discoveryFeatures() const
 {
     return {
-        ns_attention
+        ns_attention.toString(),
     };
 }
 
@@ -183,14 +182,18 @@ QString QXmppAttentionManager::requestAttention(const QString &jid, const QStrin
     return {};
 }
 
-void QXmppAttentionManager::setClient(QXmppClient *client)
+void QXmppAttentionManager::onRegistered(QXmppClient *client)
 {
-    QXmppClientExtension::setClient(client);
-
     connect(client, &QXmppClient::messageReceived,
             this, &QXmppAttentionManager::handleMessageReceived);
 }
 
+void QXmppAttentionManager::onUnregistered(QXmppClient *client)
+{
+    disconnect(client, &QXmppClient::messageReceived,
+               this, &QXmppAttentionManager::handleMessageReceived);
+}
+
 void QXmppAttentionManager::handleMessageReceived(const QXmppMessage &message)
 {
     if (!message.isAttentionRequested() || !message.stamp().isNull()) {
