@@ -9,9 +9,14 @@
 #include "QXmppConstants_p.h"
 #include "QXmppIq.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QDomElement>
 
+using namespace QXmpp::Private;
+
 // The QXmppPrivateStorageIq class represents an XML private storage IQ
 // as defined by XEP-0049: Private XML Storage.
 //
@@ -45,21 +50,20 @@ void QXmppPrivateStorageIq::setBookmarks(const QXmppBookmarkSet &bookmarks)
 
 bool QXmppPrivateStorageIq::isPrivateStorageIq(const QDomElement &element)
 {
-    const QDomElement queryElement = element.firstChildElement("query");
-    return queryElement.namespaceURI() == ns_private &&
-        QXmppBookmarkSet::isBookmarkSet(queryElement.firstChildElement());
+    return isIqType(element, u"query", ns_private) &&
+        QXmppBookmarkSet::isBookmarkSet(element.firstChildElement().firstChildElement());
 }
 
 void QXmppPrivateStorageIq::parseElementFromChild(const QDomElement &element)
 {
-    const QDomElement queryElement = element.firstChildElement("query");
+    const QDomElement queryElement = firstChildElement(element, u"query");
     m_bookmarks.parse(queryElement.firstChildElement());
 }
 
 void QXmppPrivateStorageIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement("query");
-    writer->writeDefaultNamespace(ns_private);
+    writer->writeStartElement(QSL65("query"));
+    writer->writeDefaultNamespace(toString65(ns_private));
     m_bookmarks.toXml(writer);
     writer->writeEndElement();
 }
@@ -122,21 +126,9 @@ bool QXmppBookmarkManager::setBookmarks(const QXmppBookmarkSet &bookmarks)
 }
 
 /// \cond
-void QXmppBookmarkManager::setClient(QXmppClient *client)
-{
-
-    QXmppClientExtension::setClient(client);
-
-    connect(client, &QXmppClient::connected,
-            this, &QXmppBookmarkManager::slotConnected);
-
-    connect(client, &QXmppClient::disconnected,
-            this, &QXmppBookmarkManager::slotDisconnected);
-}
-
 bool QXmppBookmarkManager::handleStanza(const QDomElement &stanza)
 {
-    if (stanza.tagName() == "iq") {
+    if (stanza.tagName() == u"iq") {
         if (QXmppPrivateStorageIq::isPrivateStorageIq(stanza)) {
             QXmppPrivateStorageIq iq;
             iq.parse(stanza);
@@ -147,7 +139,7 @@ bool QXmppBookmarkManager::handleStanza(const QDomElement &stanza)
                 Q_EMIT bookmarksReceived(d->bookmarks);
             }
             return true;
-        } else if (!d->pendingId.isEmpty() && stanza.attribute("id") == d->pendingId) {
+        } else if (!d->pendingId.isEmpty() && stanza.attribute(u"id"_s) == d->pendingId) {
             QXmppIq iq;
             iq.parse(stanza);
             if (iq.type() == QXmppIq::Result) {
@@ -160,6 +152,24 @@ bool QXmppBookmarkManager::handleStanza(const QDomElement &stanza)
     }
     return false;
 }
+
+void QXmppBookmarkManager::onRegistered(QXmppClient *client)
+{
+    connect(client, &QXmppClient::connected,
+            this, &QXmppBookmarkManager::slotConnected);
+
+    connect(client, &QXmppClient::disconnected,
+            this, &QXmppBookmarkManager::slotDisconnected);
+}
+
+void QXmppBookmarkManager::onUnregistered(QXmppClient *client)
+{
+    disconnect(client, &QXmppClient::connected,
+               this, &QXmppBookmarkManager::slotConnected);
+
+    disconnect(client, &QXmppClient::disconnected,
+               this, &QXmppBookmarkManager::slotDisconnected);
+}
 /// \endcond
 
 void QXmppBookmarkManager::slotConnected()
