@@ -75,7 +75,8 @@ public Q_SLOTS:
 
 protected:
     /// \cond
-    void setClient(QXmppClient *client) override;
+    void onRegistered(QXmppClient *client) override;
+    void onUnregistered(QXmppClient *client) override;
     /// \endcond
 
 private Q_SLOTS:
@@ -86,7 +87,7 @@ private Q_SLOTS:
     void _q_presenceReceived(const QXmppPresence &presence);
 
 private:
-    QXmppCallManagerPrivate *d;
+    const std::unique_ptr<QXmppCallManagerPrivate> d;
     friend class QXmppCall;
     friend class QXmppCallPrivate;
     friend class QXmppCallManagerPrivate;
