@@ -1,4 +1,6 @@
 // SPDX-FileCopyrightText: 2009 Manjeet Dahiya <manjeetdahiya@gmail.com>
+// SPDX-FileCopyrightText: 2019 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2023 Melvin Keskin <melvo@olomono.de>
 //
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
@@ -16,20 +18,25 @@
 #include "QXmppLogger.h"
 #include "QXmppMessage.h"
 #include "QXmppMessageHandler.h"
-#include "QXmppOutgoingClient.h"
 #include "QXmppPacket_p.h"
 #include "QXmppPromise.h"
 #include "QXmppRosterManager.h"
+#include "QXmppStreamManagement_p.h"
 #include "QXmppTask.h"
-#include "QXmppTlsManager_p.h"
 #include "QXmppUtils.h"
 #include "QXmppVCardManager.h"
 #include "QXmppVersionManager.h"
 
+#include "StringLiterals.h"
+#include "XmppSocket.h"
+
+#include <chrono>
+
 #include <QDomElement>
 #include <QSslSocket>
 #include <QTimer>
 
+using namespace std::chrono_literals;
 using namespace QXmpp::Private;
 using MessageEncryptResult = QXmppE2eeExtension::MessageEncryptResult;
 using IqEncryptResult = QXmppE2eeExtension::IqEncryptResult;
@@ -37,8 +44,8 @@ using IqDecryptResult = QXmppE2eeExtension::IqDecryptResult;
 
 static bool isIqResponse(const QDomElement &el)
 {
-    auto type = el.attribute("type");
-    return el.tagName() == "iq" && (type == "result" || type == "error");
+    auto type = el.attribute(u"type"_s);
+    return el.tagName() == u"iq" && (type == u"result" || type == u"error");
 }
 
 /// \cond
@@ -50,7 +57,6 @@ QXmppClientPrivate::QXmppClientPrivate(QXmppClient *qq)
       receivedConflict(false),
       reconnectionTries(0),
       reconnectionTimer(nullptr),
-      isActive(true),
       q(qq)
 {
 }
@@ -59,22 +65,22 @@ void QXmppClientPrivate::addProperCapability(QXmppPresence &presence)
 {
     auto *ext = q->findExtension<QXmppDiscoveryManager>();
     if (ext) {
-        presence.setCapabilityHash("sha-1");
+        presence.setCapabilityHash(u"sha-1"_s);
         presence.setCapabilityNode(ext->clientCapabilitiesNode());
         presence.setCapabilityVer(ext->capabilities().verificationString());
     }
 }
 
-int QXmppClientPrivate::getNextReconnectTime() const
+std::chrono::milliseconds QXmppClientPrivate::getNextReconnectTime() const
 {
     if (reconnectionTries < 5) {
-        return 10 * 1000;
+        return 10s;
     } else if (reconnectionTries < 10) {
-        return 20 * 1000;
+        return 20s;
     } else if (reconnectionTries < 15) {
-        return 40 * 1000;
+        return 40s;
     } else {
-        return 60 * 1000;
+        return 60s;
     }
 }
 
@@ -82,41 +88,71 @@ QStringList QXmppClientPrivate::discoveryFeatures()
 {
     return {
         // XEP-0004: Data Forms
-        ns_data,
+        ns_data.toString(),
         // XEP-0059: Result Set Management
-        ns_rsm,
+        ns_rsm.toString(),
         // XEP-0066: Out of Band Data
-        ns_oob,
+        ns_oob.toString(),
         // XEP-0071: XHTML-IM
-        ns_xhtml_im,
+        ns_xhtml_im.toString(),
         // XEP-0085: Chat State Notifications
-        ns_chat_states,
+        ns_chat_states.toString(),
         // XEP-0115: Entity Capabilities
-        ns_capabilities,
-        // XEP-0199: XMPP Ping
-        ns_ping,
+        ns_capabilities.toString(),
         // XEP-0249: Direct MUC Invitations
-        ns_conference,
+        ns_conference.toString(),
         // XEP-0308: Last Message Correction
-        ns_message_correct,
+        ns_message_correct.toString(),
         // XEP-0333: Chat Markers
-        ns_chat_markers,
+        ns_chat_markers.toString(),
         // XEP-0334: Message Processing Hints
-        ns_message_processing_hints,
+        ns_message_processing_hints.toString(),
         // XEP-0359: Unique and Stable Stanza IDs
-        ns_sid,
+        ns_sid.toString(),
         // XEP-0367: Message Attaching
-        ns_message_attaching,
+        ns_message_attaching.toString(),
         // XEP-0380: Explicit Message Encryption
-        ns_eme,
+        ns_eme.toString(),
         // XEP-0382: Spoiler messages
-        ns_spoiler,
+        ns_spoiler.toString(),
         // XEP-0428: Fallback Indication
-        ns_fallback_indication,
+        ns_fallback_indication.toString(),
         // XEP-0444: Message Reactions
-        ns_reactions,
+        ns_reactions.toString(),
     };
 }
+
+void QXmppClientPrivate::onErrorOccurred(const QString &text, const QXmppOutgoingClient::ConnectionError &err, QXmppClient::Error oldError)
+{
+    // Skip stream errors that are valid during special procedures such as account
+    // creation/deletion.
+    if (const auto streamError = std::get_if<QXmpp::StreamError>(&err);
+        streamError && ignoredStreamErrors.contains(*streamError)) {
+        return;
+    }
+
+    if (q->configuration().autoReconnectionEnabled()) {
+        if (oldError == QXmppClient::XmppStreamError) {
+            // if we receive a resource conflict, inhibit reconnection
+            if (stream->xmppStreamError() == QXmppStanza::Error::Conflict) {
+                receivedConflict = true;
+            }
+        } else if (oldError == QXmppClient::SocketError && !receivedConflict) {
+            // schedule reconnect
+            reconnectionTimer->start(getNextReconnectTime());
+        } else if (oldError == QXmppClient::KeepAliveError) {
+            // if we got a keepalive error, reconnect in one second
+            reconnectionTimer->start(1s);
+        }
+    }
+
+    // notify managers
+    Q_EMIT q->error(oldError);
+    Q_EMIT q->errorOccurred(QXmppError {
+        text,
+        visit([](const auto &value) { return std::any(value); }, err),
+    });
+}
 /// \endcond
 
 namespace QXmpp::Private::StanzaPipeline {
@@ -152,24 +188,90 @@ bool process(QXmppClient *client, const QList<QXmppClientExtension *> &extension
     return false;
 }
 
-bool process(QXmppClient *client, const QList<QXmppClientExtension *> &extensions, const QDomElement &element)
+bool process(QXmppClient *client, const QList<QXmppClientExtension *> &extensions, QXmppE2eeExtension *e2eeExt, const QDomElement &element)
 {
-    if (element.tagName() != "message") {
+    if (element.tagName() != u"message") {
         return false;
     }
     QXmppMessage message;
-    message.parse(element);
+    if (e2eeExt) {
+        message.parse(element, e2eeExt->isEncrypted(element) ? ScePublic : SceSensitive);
+    } else {
+        message.parse(element);
+    }
     return process(client, extensions, std::move(message));
 }
 
 }  // namespace QXmpp::Private::MessagePipeline
 
+///
+/// \class QXmppClient
+///
+/// \brief Main class for starting and managing connections to XMPP servers.
+///
+/// It provides the user all the required functionality to connect to the
+/// server and perform operations afterwards.
+///
+/// This class will provide the handle/reference to QXmppRosterManager
+/// (roster management), QXmppVCardManager (vCard manager), and
+/// QXmppVersionManager (software version information).
+///
+/// By default, the client will automatically try reconnecting to the server.
+/// You can change that behaviour using
+/// QXmppConfiguration::setAutoReconnectionEnabled().
+///
+/// Not all the managers or extensions have been enabled by default. One can
+/// enable/disable the managers using the functions \c addExtension() and
+/// \c removeExtension(). \c findExtension() can be used to find a
+/// reference/pointer to a particular instantiated and enabled manager.
+///
+/// List of managers enabled by default:
+/// - QXmppRosterManager
+/// - QXmppVCardManager
+/// - QXmppVersionManager
+/// - QXmppDiscoveryManager
+/// - QXmppEntityTimeManager
+///
+/// ## Connection details
+///
+/// If no explicit host and port are configured, the client will look up the SRV records of the
+/// domain of the configured JID. Since QXmpp 1.8 both TCP and direct TLS records are looked up
+/// and connection via direct TLS is preferred as it saves the extra round trip from STARTTLS. See
+/// also \xep{0368, SRV records for XMPP over TLS}.
+///
+/// On connection errors the other SRV records are tested too (if multiple are available).
+///
+/// For servers without SRV records or if looking up the records did not succeed, domain and the
+/// default port of 5223 (TLS) and 5222 (TCP/STARTTLS) are tried.
+///
+/// ## Usage of FAST token-based authentication
+///
+/// QXmpp uses \xep{0484, Fast Authentication Streamlining Tokens} if enabled and supported by the
+/// server. FAST tokens can be requested after a first time authentication using a password or
+/// another strong authentication mechanism. The tokens can then be used to log in, without a
+/// password. The tokens are linked to a specific device ID (set via the SASL 2 user agent) and
+/// only this device can use the token. Tokens also expire and are rotated by the server.
+///
+/// The advantage of this mechanism is that a client does not necessarily need to store the
+/// password of an account and in the future clients that are logged in could be listed and logged
+/// out manually. FAST also allows for performance improvements as it only requires one round trip
+/// for authentication (and may be included in TLS 0-RTT data although that is not implemented in
+/// QXmpp) while other mechanisms like SCRAM need multiple round trips.
+///
+/// FAST itself is enabled by default (see QXmppConfiguration::useFastTokenAuthentication()), but
+/// you also need to set a SASL user agent with a stable device ID, so FAST can be used.
+/// After that you can login and use QXmppCredentials to serialize the token data and store it
+/// permanently. Note that the token may change over time, though.
+///
+/// \ingroup Core
+///
+
 ///
 /// \typedef QXmppClient::IqResult
 ///
-/// Result of an IQ request, either contains the QDomElement of the IQ answer
-/// (with type 'error' or 'result') or it contains the packet error, if the
-/// request couldn't be sent.
+/// Result of an IQ request, either contains the QDomElement of the IQ reponse (in case of an
+/// 'result' IQ type) or it contains an QXmppError with a QXmppStanza::Error (on type 'error') or
+/// a QXmpp::SendError.
 ///
 /// \since QXmpp 1.5
 ///
@@ -184,15 +286,18 @@ bool process(QXmppClient *client, const QList<QXmppClientExtension *> &extension
 /// \since QXmpp 1.5
 ///
 
+///
 /// Creates a QXmppClient object.
-/// \param parent is passed to the QObject's constructor.
-/// The default value is 0.
-
-QXmppClient::QXmppClient(QObject *parent)
+///
+/// \param initialExtensions can be used to set the initial set of extensions.
+/// \param parent is passed to the QObject's constructor. The default value is 0.
+///
+/// \since QXmpp 1.6
+///
+QXmppClient::QXmppClient(InitialExtensions initialExtensions, QObject *parent)
     : QXmppLoggable(parent),
       d(new QXmppClientPrivate(this))
 {
-
     d->stream = new QXmppOutgoingClient(this);
     d->addProperCapability(d->clientPresence);
 
@@ -214,14 +319,15 @@ QXmppClient::QXmppClient(QObject *parent)
     connect(d->stream->socket(), &QAbstractSocket::stateChanged,
             this, &QXmppClient::_q_socketStateChanged);
 
-    connect(d->stream, &QXmppStream::connected,
+    connect(d->stream, &QXmppOutgoingClient::connected,
             this, &QXmppClient::_q_streamConnected);
 
-    connect(d->stream, &QXmppStream::disconnected,
+    connect(d->stream, &QXmppOutgoingClient::disconnected,
             this, &QXmppClient::_q_streamDisconnected);
 
-    connect(d->stream, &QXmppOutgoingClient::error,
-            this, &QXmppClient::_q_streamError);
+    connect(d->stream, &QXmppOutgoingClient::errorOccurred, this, [this](const auto &text, const auto &error, auto oldError) {
+        d->onErrorOccurred(text, error, oldError);
+    });
 
     // reconnection
     d->reconnectionTimer = new QTimer(this);
@@ -232,12 +338,26 @@ QXmppClient::QXmppClient(QObject *parent)
     // logging
     setLogger(QXmppLogger::getLogger());
 
-    addExtension(new QXmppTlsManager);
-    addExtension(new QXmppRosterManager(this));
-    addExtension(new QXmppVCardManager);
-    addExtension(new QXmppVersionManager);
-    addExtension(new QXmppEntityTimeManager());
-    addExtension(new QXmppDiscoveryManager());
+    switch (initialExtensions) {
+    case NoExtensions:
+        break;
+    case BasicExtensions:
+        addNewExtension<QXmppRosterManager>(this);
+        addNewExtension<QXmppVCardManager>();
+        addNewExtension<QXmppVersionManager>();
+        addNewExtension<QXmppEntityTimeManager>();
+        addNewExtension<QXmppDiscoveryManager>();
+        break;
+    }
+}
+
+///
+/// Creates a QXmppClient object.
+/// \param parent is passed to the QObject's constructor.
+///
+QXmppClient::QXmppClient(QObject *parent)
+    : QXmppClient(BasicExtensions, parent)
+{
 }
 
 QXmppClient::~QXmppClient() = default;
@@ -253,19 +373,12 @@ QXmppClient::~QXmppClient() = default;
 ///
 
 /// Registers a new \a extension with the client.
-///
-/// \param extension
-
 bool QXmppClient::addExtension(QXmppClientExtension *extension)
 {
     return insertExtension(d->extensions.size(), extension);
 }
 
 /// Registers a new \a extension with the client at the given \a index.
-///
-/// \param index
-/// \param extension
-
 bool QXmppClient::insertExtension(int index, QXmppClientExtension *extension)
 {
     if (d->extensions.contains(extension)) {
@@ -274,20 +387,20 @@ bool QXmppClient::insertExtension(int index, QXmppClientExtension *extension)
     }
 
     extension->setParent(this);
-    extension->setClient(this);
     d->extensions.insert(index, extension);
+    extension->setClient(this);
     return true;
 }
 
+///
 /// Unregisters the given extension from the client. If the extension
 /// is found, it will be destroyed.
 ///
-/// \param extension
-
 bool QXmppClient::removeExtension(QXmppClientExtension *extension)
 {
     if (d->extensions.contains(extension)) {
         d->extensions.removeAll(extension);
+        extension->setClient(nullptr);
         delete extension;
         return true;
     } else {
@@ -317,16 +430,12 @@ void QXmppClient::setEncryptionExtension(QXmppE2eeExtension *extension)
 }
 
 /// Returns a list containing all the client's extensions.
-///
-
-QList<QXmppClientExtension *> QXmppClient::extensions()
+QList<QXmppClientExtension *> QXmppClient::extensions() const
 {
     return d->extensions;
 }
 
 /// Returns a modifiable reference to the current configuration of QXmppClient.
-/// \return Reference to the QXmppClient's configuration for the connection.
-
 QXmppConfiguration &QXmppClient::configuration()
 {
     return d->stream->configuration();
@@ -347,7 +456,7 @@ void QXmppClient::connectToServer(const QXmppConfiguration &config,
 {
     // reset package cache from last connection
     if (d->stream->configuration().jidBare() != config.jidBare()) {
-        d->stream->resetPacketCache();
+        d->stream->streamAckManager().resetCache();
     }
 
     d->stream->configuration() = config;
@@ -357,11 +466,12 @@ void QXmppClient::connectToServer(const QXmppConfiguration &config,
     d->stream->connectToHost();
 }
 
+///
 /// Overloaded function to simply connect to an XMPP server with a JID and password.
 ///
 /// \param jid JID for the account.
 /// \param password Password for the account.
-
+///
 void QXmppClient::connectToServer(const QString &jid, const QString &password)
 {
     QXmppConfiguration config;
@@ -391,7 +501,7 @@ void QXmppClient::connectToServer(const QString &jid, const QString &password)
 ///
 bool QXmppClient::sendPacket(const QXmppNonza &packet)
 {
-    return d->stream->sendPacket(packet);
+    return d->stream->streamAckManager().sendPacketCompat(packet);
 }
 
 ///
@@ -421,10 +531,10 @@ QXmppTask<QXmpp::SendResult> QXmppClient::sendSensitive(QXmppStanza &&stanza, co
                                QXmlStreamWriter writer(&xml);
                                message->toXml(&writer, QXmpp::ScePublic);
 
-                               d->stream->send(QXmppPacket(xml, true, std::move(interface)));
+                               d->stream->streamAckManager().send(QXmppPacket(xml, true, std::move(interface)));
                            },
                            [&](std::unique_ptr<QXmppIq> &&iq) {
-                               d->stream->send(QXmppPacket(*iq, std::move(interface)));
+                               d->stream->streamAckManager().send(QXmppPacket(*iq, std::move(interface)));
                            },
                            [&](QXmppError &&error) {
                                interface.finish(std::move(error));
@@ -446,7 +556,7 @@ QXmppTask<QXmpp::SendResult> QXmppClient::sendSensitive(QXmppStanza &&stanza, co
                     std::move(dynamic_cast<QXmppIq &&>(stanza)), params));
         }
     }
-    return d->stream->send(stanza);
+    return d->stream->streamAckManager().send(stanza);
 }
 
 ///
@@ -463,7 +573,7 @@ QXmppTask<QXmpp::SendResult> QXmppClient::sendSensitive(QXmppStanza &&stanza, co
 ///
 QXmppTask<QXmpp::SendResult> QXmppClient::send(QXmppStanza &&stanza, const std::optional<QXmppSendStanzaParams> &)
 {
-    return d->stream->send(stanza);
+    return d->stream->streamAckManager().send(stanza);
 }
 
 ///
@@ -492,6 +602,9 @@ QXmppTask<QXmpp::SendResult> QXmppClient::reply(QXmppStanza &&stanza, const std:
 /// QDomElement. If you don't expect a special response, you may want use
 /// sendGenericIq().
 ///
+/// IQs of type 'error' are parsed automatically and returned as QXmppError with a contained
+/// QXmppStanza::Error.
+///
 /// This does not do any end-to-encryption on the IQ.
 ///
 /// \sa sendSensitiveIq()
@@ -514,6 +627,9 @@ QXmppTask<QXmppClient::IqResult> QXmppClient::sendIq(QXmppIq &&iq, const std::op
 /// encrypted or it only makes little sense to do so. This is why the default
 /// sendIq() does not do any additional end-to-end encryption.
 ///
+/// IQs of type 'error' are parsed automatically and returned as QXmppError with a contained
+/// QXmppStanza::Error.
+///
 /// \warning THIS API IS NOT FINALIZED YET!
 ///
 /// \since QXmpp 1.5
@@ -533,13 +649,13 @@ QXmppTask<QXmppClient::IqResult> QXmppClient::sendSensitiveIq(QXmppIq &&iq, cons
                                                   [&](QDomElement &&el) {
                                                       if (!isIqResponse(el)) {
                                                           p.finish(QXmppError {
-                                                              QStringLiteral("Invalid IQ response received."),
+                                                              u"Invalid IQ response received."_s,
                                                               QXmpp::SendError::EncryptionError });
                                                           return;
                                                       }
                                                       if (!d->encryptionExtension) {
                                                           p.finish(QXmppError {
-                                                              QStringLiteral("No decryption extension found."),
+                                                              u"No decryption extension found."_s,
                                                               QXmpp::SendError::EncryptionError });
                                                           return;
                                                       }
@@ -599,6 +715,7 @@ QXmppTask<QXmppClient::EmptyResult> QXmppClient::sendGenericIq(QXmppIq &&iq, con
     });
 }
 
+///
 /// Disconnects the client and the current presence of client changes to
 /// QXmppPresence::Unavailable and status text changes to "Logged out".
 ///
@@ -606,14 +723,13 @@ QXmppTask<QXmppClient::EmptyResult> QXmppClient::sendGenericIq(QXmppIq &&iq, con
 /// QXmppPresence::Available, if you are again calling connectToServer() after
 /// calling the disconnectFromServer() function.
 ///
-
 void QXmppClient::disconnectFromServer()
 {
     // cancel reconnection
     d->reconnectionTimer->stop();
 
     d->clientPresence.setType(QXmppPresence::Unavailable);
-    d->clientPresence.setStatusText("Logged out");
+    d->clientPresence.setStatusText(u"Logged out"_s);
     if (d->stream->isConnected()) {
         sendPacket(d->clientPresence);
     }
@@ -622,15 +738,12 @@ void QXmppClient::disconnectFromServer()
 }
 
 /// Returns true if the client has authenticated with the XMPP server.
-
 bool QXmppClient::isAuthenticated() const
 {
     return d->stream->isAuthenticated();
 }
 
 /// Returns true if the client is connected to the XMPP server.
-///
-
 bool QXmppClient::isConnected() const
 {
     return d->stream->isConnected();
@@ -638,31 +751,26 @@ bool QXmppClient::isConnected() const
 
 ///
 /// Returns true if the current client state is "active", false if it is
-/// "inactive". See \xep{0352}: Client State Indication for details.
-///
-/// On connect this is always reset to true.
+/// "inactive". See \xep{0352, Client State Indication} for details.
 ///
 /// \since QXmpp 1.0
 ///
 bool QXmppClient::isActive() const
 {
-    return d->isActive;
+    return d->stream->csiManager().state() == CsiManager::Active;
 }
 
 ///
-/// Sets the client state as described in \xep{0352}: Client State Indication.
+/// Sets the client state as described in \xep{0352, Client State Indication}.
 ///
-/// On connect this is always reset to true.
+/// Since QXmpp 1.8, the state is restored across reconnects. QXmpp will re-send the state of
+/// 'inactive' on connection if that was set before. Stream resumptions are also handled.
 ///
 /// \since QXmpp 1.0
 ///
 void QXmppClient::setActive(bool active)
 {
-    if (active != d->isActive && isConnected() && d->stream->isClientStateIndicationEnabled()) {
-        d->isActive = active;
-        QString packet = "<%1 xmlns='%2'/>";
-        d->stream->sendData(packet.arg(active ? "active" : "inactive", ns_csi).toUtf8());
-    }
+    d->stream->csiManager().setState(active ? CsiManager::Active : CsiManager::Inactive);
 }
 
 ///
@@ -675,8 +783,8 @@ void QXmppClient::setActive(bool active)
 ///
 QXmppClient::StreamManagementState QXmppClient::streamManagementState() const
 {
-    if (d->stream->isStreamManagementEnabled()) {
-        if (d->stream->isStreamResumed()) {
+    if (d->stream->c2sStreamManager().enabled()) {
+        if (d->stream->c2sStreamManager().streamResumed()) {
             return ResumedStream;
         }
         return NewStream;
@@ -684,19 +792,7 @@ QXmppClient::StreamManagementState QXmppClient::streamManagementState() const
     return NoStreamManagement;
 }
 
-/// Returns the reference to QXmppRosterManager object of the client.
 ///
-/// \return Reference to the roster object of the connected client. Use this to
-/// get the list of friends in the roster and their presence information.
-///
-/// \deprecated This method is deprecated since QXmpp 1.1. Use
-/// \c QXmppClient::findExtension<QXmppRosterManager>() instead.
-
-QXmppRosterManager &QXmppClient::rosterManager()
-{
-    return *findExtension<QXmppRosterManager>();
-}
-
 /// Utility function to send message to all the resources associated with the
 /// specified bareJid. If there are no resources available, that is the contact
 /// is offline or not present in the roster, it will still send a message to
@@ -710,7 +806,7 @@ QXmppRosterManager &QXmppClient::rosterManager()
 ///
 /// \param bareJid bareJid of the receiving entity
 /// \param message Message string to be sent.
-
+///
 void QXmppClient::sendMessage(const QString &bareJid, const QString &message)
 {
     QXmppRosterManager *rosterManager = findExtension<QXmppRosterManager>();
@@ -722,13 +818,18 @@ void QXmppClient::sendMessage(const QString &bareJid, const QString &message)
     if (!resources.isEmpty()) {
         for (const auto &resource : resources) {
             sendPacket(
-                QXmppMessage({}, bareJid + QStringLiteral("/") + resource, message));
+                QXmppMessage({}, bareJid + u"/"_s + resource, message));
         }
     } else {
         sendPacket(QXmppMessage({}, bareJid, message));
     }
 }
 
+QXmppOutgoingClient *QXmppClient::stream() const
+{
+    return d->stream;
+}
+
 QXmppClient::State QXmppClient::state() const
 {
     if (d->stream->isConnected()) {
@@ -742,13 +843,12 @@ QXmppClient::State QXmppClient::state() const
 }
 
 /// Returns the client's current presence.
-///
-
 QXmppPresence QXmppClient::clientPresence() const
 {
     return d->clientPresence;
 }
 
+///
 /// Changes the presence of the connected client.
 ///
 /// The connection to the server will be updated accordingly:
@@ -761,7 +861,6 @@ QXmppPresence QXmppClient::clientPresence() const
 ///
 /// \param presence QXmppPresence object
 ///
-
 void QXmppClient::setClientPresence(const QXmppPresence &presence)
 {
     d->clientPresence = presence;
@@ -786,68 +885,40 @@ void QXmppClient::setClientPresence(const QXmppPresence &presence)
 }
 
 /// Returns the socket error if error() is QXmppClient::SocketError.
-///
-
 QAbstractSocket::SocketError QXmppClient::socketError()
 {
     return d->stream->socket()->error();
 }
 
 /// Returns the human-readable description of the last socket error if error() is QXmppClient::SocketError.
-
 QString QXmppClient::socketErrorString() const
 {
     return d->stream->socket()->errorString();
 }
 
 /// Returns the XMPP stream error if QXmppClient::Error is QXmppClient::XmppStreamError.
-
 QXmppStanza::Error::Condition QXmppClient::xmppStreamError()
 {
     return d->stream->xmppStreamError();
 }
 
-///
-/// Returns the reference to QXmppVCardManager, implementation of \xep{0054}.
-/// http://xmpp.org/extensions/xep-0054.html
-///
-/// \deprecated This method is deprecated since QXmpp 1.1. Use
-/// \c QXmppClient::findExtension<QXmppVCardManager>() instead.
-///
-QXmppVCardManager &QXmppClient::vCardManager()
-{
-    return *findExtension<QXmppVCardManager>();
-}
-
-///
-/// Returns the reference to QXmppVersionManager, implementation of \xep{0092}.
-/// http://xmpp.org/extensions/xep-0092.html
-///
-/// \deprecated This method is deprecated since QXmpp 1.1. Use
-/// \c QXmppClient::findExtension<QXmppVersionManager>() instead.
-///
-QXmppVersionManager &QXmppClient::versionManager()
-{
-    return *findExtension<QXmppVersionManager>();
-}
-
 void QXmppClient::injectIq(const QDomElement &element, const std::optional<QXmppE2eeMetadata> &e2eeMetadata)
 {
-    if (element.tagName() != "iq") {
+    if (element.tagName() != u"iq") {
         return;
     }
     if (!StanzaPipeline::process(d->extensions, element, e2eeMetadata)) {
-        const auto iqType = element.attribute("type");
-        if (iqType == "get" || iqType == "set") {
+        const auto iqType = element.attribute(u"type"_s);
+        if (iqType == u"get" || iqType == u"set") {
             // send error IQ
             using Err = QXmppStanza::Error;
 
             QXmppIq iq(QXmppIq::Error);
-            iq.setTo(element.attribute("from"));
-            iq.setId(element.attribute("id"));
+            iq.setTo(element.attribute(u"from"_s));
+            iq.setId(element.attribute(u"id"_s));
             const auto errMessage = e2eeMetadata.has_value()
-                ? QStringLiteral("Feature not implemented or not supported with end-to-end encryption.")
-                : QStringLiteral("Feature not implemented.");
+                ? u"Feature not implemented or not supported with end-to-end encryption."_s
+                : u"Feature not implemented."_s;
             iq.setError(Err(Err::Cancel, Err::FeatureNotImplemented, errMessage));
             reply(std::move(iq), e2eeMetadata);
         }
@@ -868,6 +939,18 @@ bool QXmppClient::injectMessage(QXmppMessage &&message)
     return handled;
 }
 
+///
+/// Sets stream errors that are ignored if they occur.
+///
+/// \param errors stream errors to be ignored
+///
+/// \since QXmpp 1.9
+///
+void QXmppClient::setIgnoredStreamErrors(const QVector<QXmpp::StreamError> &errors)
+{
+    d->ignoredStreamErrors = errors;
+}
+
 ///
 /// Give extensions a chance to handle incoming stanzas.
 ///
@@ -876,13 +959,13 @@ void QXmppClient::_q_elementReceived(const QDomElement &element, bool &handled)
     // The stanza comes directly from the XMPP stream, so it's not end-to-end
     // encrypted and there's no e2ee metadata (std::nullopt).
     handled = StanzaPipeline::process(d->extensions, element, std::nullopt) ||
-        MessagePipeline::process(this, d->extensions, element);
+        MessagePipeline::process(this, d->extensions, d->encryptionExtension, element);
 }
 
 void QXmppClient::_q_reconnect()
 {
     if (d->stream->configuration().autoReconnectionEnabled()) {
-        debug("Reconnecting to server");
+        debug(u"Reconnecting to server"_s);
         d->stream->connectToHost();
     }
 }
@@ -894,19 +977,20 @@ void QXmppClient::_q_socketStateChanged(QAbstractSocket::SocketState socketState
 }
 
 /// At connection establishment, send initial presence.
-
-void QXmppClient::_q_streamConnected()
+void QXmppClient::_q_streamConnected(const QXmpp::Private::SessionBegin &session)
 {
     d->receivedConflict = false;
     d->reconnectionTries = 0;
-    d->isActive = true;
 
     // notify managers
+    if (session.fastTokenChanged) {
+        Q_EMIT credentialsChanged();
+    }
     Q_EMIT connected();
     Q_EMIT stateChanged(QXmppClient::ConnectedState);
 
     // send initial presence
-    if (d->stream->isAuthenticated()) {
+    if (d->stream->isAuthenticated() && streamManagementState() != ResumedStream) {
         sendPacket(d->clientPresence);
     }
 }
@@ -918,34 +1002,12 @@ void QXmppClient::_q_streamDisconnected()
     Q_EMIT stateChanged(QXmppClient::DisconnectedState);
 }
 
-void QXmppClient::_q_streamError(QXmppClient::Error err)
-{
-    if (d->stream->configuration().autoReconnectionEnabled()) {
-        if (err == QXmppClient::XmppStreamError) {
-            // if we receive a resource conflict, inhibit reconnection
-            if (d->stream->xmppStreamError() == QXmppStanza::Error::Conflict) {
-                d->receivedConflict = true;
-            }
-        } else if (err == QXmppClient::SocketError && !d->receivedConflict) {
-            // schedule reconnect
-            d->reconnectionTimer->start(d->getNextReconnectTime());
-        } else if (err == QXmppClient::KeepAliveError) {
-            // if we got a keepalive error, reconnect in one second
-            d->reconnectionTimer->start(1000);
-        }
-    }
-
-    // notify managers
-    Q_EMIT error(err);
-}
-
 QXmppLogger *QXmppClient::logger() const
 {
     return d->logger;
 }
 
 /// Sets the QXmppLogger associated with the current QXmppClient.
-
 void QXmppClient::setLogger(QXmppLogger *logger)
 {
     if (logger != d->logger) {
