@@ -1,4 +1,6 @@
 // SPDX-FileCopyrightText: 2009 Manjeet Dahiya <manjeetdahiya@gmail.com>
+// SPDX-FileCopyrightText: 2019 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2023 Melvin Keskin <melvo@olomono.de>
 //
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
@@ -10,6 +12,7 @@
 #include "QXmppPresence.h"
 #include "QXmppSendResult.h"
 #include "QXmppSendStanzaParams.h"
+#include "QXmppStreamError.h"
 
 #include <memory>
 #include <variant>
@@ -24,11 +27,10 @@ class QXmppTask;
 class QXmppE2eeExtension;
 class QXmppClientExtension;
 class QXmppClientPrivate;
-class QXmppPresence;
 class QXmppMessage;
+class QXmppOutgoingClient;
+class QXmppPresence;
 class QXmppIq;
-class QXmppStream;
-class QXmppInternalClientExtension;
 
 // managers
 class QXmppDiscoveryIq;
@@ -36,6 +38,10 @@ class QXmppRosterManager;
 class QXmppVCardManager;
 class QXmppVersionManager;
 
+namespace QXmpp::Private {
+struct SessionBegin;
+}
+
 ///
 /// \defgroup Core Core classes
 ///
@@ -52,34 +58,6 @@ class QXmppVersionManager;
 /// QXmppClient::addExtension().
 ///
 
-///
-/// \brief The QXmppClient class is the main class for using QXmpp.
-///
-/// It provides the user all the required functionality to connect to the
-/// server and perform operations afterwards.
-///
-/// This class will provide the handle/reference to QXmppRosterManager
-/// (roster management), QXmppVCardManager (vCard manager), and
-/// QXmppVersionManager (software version information).
-///
-/// By default, the client will automatically try reconnecting to the server.
-/// You can change that behaviour using
-/// QXmppConfiguration::setAutoReconnectionEnabled().
-///
-/// Not all the managers or extensions have been enabled by default. One can
-/// enable/disable the managers using the functions \c addExtension() and
-/// \c removeExtension(). \c findExtension() can be used to find a
-/// reference/pointer to a particular instantiated and enabled manager.
-///
-/// List of managers enabled by default:
-/// - QXmppRosterManager
-/// - QXmppVCardManager
-/// - QXmppVersionManager
-/// - QXmppDiscoveryManager
-/// - QXmppEntityTimeManager
-///
-/// \ingroup Core
-///
 class QXMPP_EXPORT QXmppClient : public QXmppLoggable
 {
     Q_OBJECT
@@ -121,6 +99,16 @@ public:
         ResumedStream
     };
 
+    /// Used to decide which extensions should be added by default.
+    /// \since QXmpp 1.6
+    enum InitialExtensions {
+        /// Creates a client without any extensions.
+        NoExtensions,
+        /// Creates a client with the default set of extensions.
+        BasicExtensions,
+    };
+
+    QXmppClient(InitialExtensions, QObject *parent = nullptr);
     QXmppClient(QObject *parent = nullptr);
     ~QXmppClient() override;
 
@@ -138,7 +126,7 @@ public:
     QXmppE2eeExtension *encryptionExtension() const;
     void setEncryptionExtension(QXmppE2eeExtension *);
 
-    QList<QXmppClientExtension *> extensions();
+    QList<QXmppClientExtension *> extensions() const;
 
     ///
     /// \brief Returns the extension which can be cast into type T*, or 0
@@ -154,7 +142,7 @@ public:
     /// \endcode
     ///
     template<typename T>
-    T *findExtension()
+    T *findExtension() const
     {
         const QList<QXmppClientExtension *> list = extensions();
         for (auto ext : list) {
@@ -182,7 +170,7 @@ public:
     /// \since QXmpp 1.2
     ///
     template<typename T>
-    int indexOfExtension()
+    int indexOfExtension() const
     {
         auto list = extensions();
         for (int i = 0; i < list.size(); ++i) {
@@ -271,6 +259,18 @@ Q_SIGNALS:
     /// know the error.
     void error(QXmppClient::Error);
 
+    /// This signal is emitted when the XMPP connection encounters any fatal error.
+    ///
+    /// \param error Detailed error object with a description. It may contain
+    ///  * QAbstractSocket::SocketError
+    ///  * QXmpp::TimeoutError
+    ///  * QXmpp::StreamError
+    ///  * QXmpp::AuthenticationError
+    ///  * QXmpp::BindError
+    ///
+    /// \since QXmpp 1.7
+    Q_SIGNAL void errorOccurred(const QXmppError &error);
+
     /// This signal is emitted when the logger changes.
     void loggerChanged(QXmppLogger *logger);
 
@@ -301,6 +301,13 @@ Q_SIGNALS:
     /// This signal is emitted when the client state changes.
     void stateChanged(QXmppClient::State state);
 
+    /// Emitted when the credentials, e.g. tokens have changed.
+    ///
+    /// This means that the QXmppCredentials in the QXmppConfiguration of this client has changed.
+    ///
+    /// \since QXmpp 1.8
+    Q_SIGNAL void credentialsChanged();
+
 public Q_SLOTS:
     void connectToServer(const QXmppConfiguration &,
                          const QXmppPresence &initialPresence =
@@ -312,22 +319,25 @@ public Q_SLOTS:
     void sendMessage(const QString &bareJid, const QString &message);
 
 private:
+    QXmppOutgoingClient *stream() const;
     void injectIq(const QDomElement &element, const std::optional<QXmppE2eeMetadata> &e2eeMetadata);
     bool injectMessage(QXmppMessage &&message);
 
+    void setIgnoredStreamErrors(const QVector<QXmpp::StreamError> &);
+
 private Q_SLOTS:
     void _q_elementReceived(const QDomElement &element, bool &handled);
     void _q_reconnect();
     void _q_socketStateChanged(QAbstractSocket::SocketState state);
-    void _q_streamConnected();
+    void _q_streamConnected(const QXmpp::Private::SessionBegin &);
     void _q_streamDisconnected();
-    void _q_streamError(QXmppClient::Error error);
 
 private:
     const std::unique_ptr<QXmppClientPrivate> d;
 
     friend class QXmppClientExtension;
-    friend class QXmppInternalClientExtension;
+    friend class QXmppCarbonManagerV2;
+    friend class QXmppRegistrationManager;
     friend class TestClient;
 };
 
