@@ -1,5 +1,6 @@
 // SPDX-FileCopyrightText: 2020 Manjeet Dahiya <manjeetdahiya@gmail.com>
 // SPDX-FileCopyrightText: 2020 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2023 Melvin Keskin <melvo@olomono.de>
 //
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
@@ -18,13 +19,15 @@
 #ifndef QXMPPCLIENT_P_H
 #define QXMPPCLIENT_P_H
 
+#include "QXmppOutgoingClient.h"
 #include "QXmppPresence.h"
 
+#include <chrono>
+
 class QXmppClient;
 class QXmppClientExtension;
 class QXmppE2eeExtension;
 class QXmppLogger;
-class QXmppOutgoingClient;
 class QTimer;
 
 class QXmppClientPrivate
@@ -38,6 +41,7 @@ public:
     QXmppLogger *logger;
     /// Pointer to the XMPP stream
     QXmppOutgoingClient *stream;
+    QVector<QXmpp::StreamError> ignoredStreamErrors;
 
     QXmppE2eeExtension *encryptionExtension;
 
@@ -46,13 +50,11 @@ public:
     int reconnectionTries;
     QTimer *reconnectionTimer;
 
-    // Client state indication
-    bool isActive;
-
     void addProperCapability(QXmppPresence &presence);
-    int getNextReconnectTime() const;
+    std::chrono::milliseconds getNextReconnectTime() const;
 
     static QStringList discoveryFeatures();
+    void onErrorOccurred(const QString &text, const QXmppOutgoingClient::ConnectionError &err, QXmppClient::Error oldError);
 
 private:
     QXmppClient *q;
