@@ -11,6 +11,8 @@
 #include "QXmppIqHandling.h"
 #include "QXmppUtils.h"
 
+#include "StringLiterals.h"
+
 #include <QDateTime>
 #include <QDomElement>
 
@@ -67,7 +69,7 @@ auto QXmppEntityTimeManager::requestEntityTime(const QString &jid) -> QXmppTask<
 /// \cond
 QStringList QXmppEntityTimeManager::discoveryFeatures() const
 {
-    return QStringList() << ns_entity_time;
+    return { ns_entity_time.toString() };
 }
 
 bool QXmppEntityTimeManager::handleStanza(const QDomElement &element)
@@ -76,7 +78,7 @@ bool QXmppEntityTimeManager::handleStanza(const QDomElement &element)
         return true;
     }
 
-    if (element.tagName() == "iq" && QXmppEntityTimeIq::isEntityTimeIq(element)) {
+    if (element.tagName() == u"iq" && QXmppEntityTimeIq::isEntityTimeIq(element)) {
         QXmppEntityTimeIq entityTime;
         entityTime.parse(element);
         Q_EMIT timeReceived(entityTime);
@@ -90,7 +92,7 @@ std::variant<QXmppEntityTimeIq, QXmppStanza::Error> QXmppEntityTimeManager::hand
 {
     using Err = QXmppStanza::Error;
     if (iq.type() != QXmppIq::Get) {
-        return Err(Err::Cancel, Err::BadRequest, QStringLiteral("Only IQ requests of type 'get' allowed."));
+        return Err(Err::Cancel, Err::BadRequest, u"Only IQ requests of type 'get' allowed."_s);
     }
 
     QXmppEntityTimeIq responseIq;
