@@ -7,7 +7,6 @@
 
 #include "QXmppBitsOfBinaryContentId.h"
 #include "QXmppBitsOfBinaryData.h"
-#include "QXmppClient.h"
 #include "QXmppFileMetadata.h"
 #include "QXmppFileShare.h"
 #include "QXmppFutureUtils_p.h"
@@ -16,6 +15,9 @@
 #include "QXmppUploadRequestManager.h"
 #include "QXmppUtils_p.h"
 
+#include "Algorithms.h"
+#include "StringLiterals.h"
+
 #include <any>
 #include <unordered_map>
 #include <utility>
@@ -36,24 +38,11 @@ using MetadataGeneratorResult = QXmppFileSharingManager::MetadataGeneratorResult
 // The manager generates a hash with each hash algorithm
 static std::vector<HashAlgorithm> hashAlgorithms()
 {
-    return {
-        HashAlgorithm::Sha256,
 #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-        HashAlgorithm::Blake2b_256,
+    return { HashAlgorithm::Sha256, HashAlgorithm::Blake2b_256 };
 #else
-        HashAlgorithm::Sha3_256,
+    return { HashAlgorithm::Sha256, HashAlgorithm::Sha3_256 };
 #endif
-    };
-}
-
-template<typename T, typename Converter>
-auto transform(T &input, Converter convert)
-{
-    using Output = std::decay_t<decltype(convert(input.front()))>;
-    std::vector<Output> output;
-    output.reserve(input.size());
-    std::transform(input.begin(), input.end(), std::back_inserter(output), std::move(convert));
-    return output;
 }
 
 class QXmppFileUploadPrivate
@@ -478,12 +467,9 @@ std::shared_ptr<QXmppFileUpload> QXmppFileSharingManager::uploadFile(std::shared
                     auto &hashValue = hashResult->result;
                     if (std::holds_alternative<std::vector<QXmppHash>>(hashValue)) {
                         const auto &hashesVector = std::get<std::vector<QXmppHash>>(hashValue);
-                        QVector<QXmppHash> hashes;
-                        hashes.reserve(hashesVector.size());
-                        std::transform(hashesVector.begin(), hashesVector.end(),
-                                       std::back_inserter(hashes), [](auto &&hash) {
-                                           return hash;
-                                       });
+                        auto hashes = transform<QVector<QXmppHash>>(hashesVector, [](auto &&hash) {
+                            return hash;
+                        });
                         upload->d->metadata.setHashes(hashes);
                         upload->d->success = true;
                     } else if (std::holds_alternative<Cancelled>(hashValue)) {
@@ -554,7 +540,7 @@ std::shared_ptr<QXmppFileDownload> QXmppFileSharingManager::downloadFile(
 
         // try to do hash verification
         if (filePath.isEmpty()) {
-            warning(QStringLiteral("Can't verify hashes of other io devices than QFile!"));
+            warning(u"Can't verify hashes of other io devices than QFile!"_s);
             download->reportFinished(QXmppFileDownload::Downloaded { QXmppFileDownload::NoStrongHashes });
             return;
         }
@@ -567,7 +553,7 @@ std::shared_ptr<QXmppFileDownload> QXmppFileSharingManager::downloadFile(
 
         download->d->hashesFuture = verifyHashes(
             std::move(file),
-            transform(download->d->hashes, [](auto hash) { return hash; }));
+            transform<std::vector<QXmppHash>>(download->d->hashes, [](auto hash) { return hash; }));
 
         await(download->d->hashesFuture, this, [download](HashVerificationResultPtr hashResult) {
             auto convert = overloaded {
@@ -578,7 +564,7 @@ std::shared_ptr<QXmppFileDownload> QXmppFileSharingManager::downloadFile(
                 },
                 [](HashVerificationResult::NotMatching) {
                     return QXmppError {
-                        QStringLiteral("Checksum does not match"),
+                        u"Checksum does not match"_s,
                         {}
                     };
                 },
