@@ -13,6 +13,10 @@
 #include "QXmppMessage.h"
 #include "QXmppPromise.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "Algorithms.h"
+#include "StringLiterals.h"
 
 #include <unordered_map>
 
@@ -21,24 +25,13 @@
 using namespace QXmpp;
 using namespace QXmpp::Private;
 
-template<typename T, typename Converter>
-auto transform(const T &input, Converter convert)
-{
-    using Output = std::decay_t<decltype(convert(*input.begin()))>;
-    QVector<Output> output;
-    output.reserve(input.size());
-    std::transform(input.begin(), input.end(), std::back_inserter(output), std::move(convert));
-    return output;
-}
-
 template<typename T>
 auto sum(const T &c)
 {
     return std::accumulate(c.begin(), c.end(), 0);
 }
 
-struct MamMessage
-{
+struct MamMessage {
     QDomElement element;
     std::optional<QDateTime> delay;
 };
@@ -58,27 +51,27 @@ QXmppMessage parseMamMessage(const MamMessage &mamMessage, EncryptedType encrypt
 
 std::optional<std::tuple<MamMessage, QString>> parseMamMessageResult(const QDomElement &messageEl)
 {
-    auto resultElement = messageEl.firstChildElement("result");
-    if (resultElement.isNull() || resultElement.namespaceURI() != ns_mam) {
+    auto resultElement = firstChildElement(messageEl, u"result", ns_mam);
+    if (resultElement.isNull()) {
         return {};
     }
 
-    auto forwardedElement = resultElement.firstChildElement("forwarded");
-    if (forwardedElement.isNull() || forwardedElement.namespaceURI() != ns_forwarding) {
+    auto forwardedElement = firstChildElement(resultElement, u"forwarded", ns_forwarding);
+    if (forwardedElement.isNull()) {
         return {};
     }
 
-    auto queryId = resultElement.attribute("queryid");
+    auto queryId = resultElement.attribute(u"queryid"_s);
 
-    auto messageElement = forwardedElement.firstChildElement("message");
+    auto messageElement = firstChildElement(forwardedElement, u"message", ns_client);
     if (messageElement.isNull()) {
         return {};
     }
 
     auto parseDelay = [](const auto &forwardedEl) -> std::optional<QDateTime> {
-        auto delayEl = forwardedEl.firstChildElement("delay");
-        if (!delayEl.isNull() && delayEl.namespaceURI() == ns_delayed_delivery) {
-            return QXmppUtils::datetimeFromString(delayEl.attribute("stamp"));
+        auto delayEl = firstChildElement(forwardedEl, u"delay", ns_delayed_delivery);
+        if (!delayEl.isNull()) {
+            return QXmppUtils::datetimeFromString(delayEl.attribute(u"stamp"_s));
         }
         return {};
     };
@@ -86,8 +79,7 @@ std::optional<std::tuple<MamMessage, QString>> parseMamMessageResult(const QDomE
     return { { MamMessage { messageElement, parseDelay(forwardedElement) }, queryId } };
 }
 
-struct RetrieveRequestState
-{
+struct RetrieveRequestState {
     QXmppPromise<QXmppMamManager::RetrieveResult> promise;
     QXmppMamResultIq iq;
     QVector<MamMessage> messages;
@@ -150,12 +142,12 @@ QXmppMamManager::~QXmppMamManager() = default;
 QStringList QXmppMamManager::discoveryFeatures() const
 {
     // XEP-0313: Message Archive Management
-    return QStringList() << ns_mam;
+    return { ns_mam.toString() };
 }
 
 bool QXmppMamManager::handleStanza(const QDomElement &element)
 {
-    if (element.tagName() == "message") {
+    if (element.tagName() == u"message") {
         if (auto result = parseMamMessageResult(element)) {
             auto &[message, queryId] = *result;
 
@@ -190,27 +182,27 @@ static QXmppMamQueryIq buildRequest(const QString &to,
     QList<QXmppDataForm::Field> fields;
 
     QXmppDataForm::Field hiddenField(QXmppDataForm::Field::HiddenField);
-    hiddenField.setKey("FORM_TYPE");
-    hiddenField.setValue(ns_mam);
+    hiddenField.setKey(u"FORM_TYPE"_s);
+    hiddenField.setValue(ns_mam.toString());
     fields << hiddenField;
 
     if (!jid.isEmpty()) {
         QXmppDataForm::Field jidField;
-        jidField.setKey("with");
+        jidField.setKey(u"with"_s);
         jidField.setValue(jid);
         fields << jidField;
     }
 
     if (start.isValid()) {
         QXmppDataForm::Field startField;
-        startField.setKey("start");
+        startField.setKey(u"start"_s);
         startField.setValue(QXmppUtils::datetimeToString(start));
         fields << startField;
     }
 
     if (end.isValid()) {
         QXmppDataForm::Field endField;
-        endField.setKey("end");
+        endField.setKey(u"end"_s);
         endField.setValue(QXmppUtils::datetimeToString(end));
         fields << endField;
     }
@@ -281,8 +273,7 @@ QString QXmppMamManager::retrieveArchivedMessages(const QString &to,
 /// \param resultSetQuery Optional Result Set Management query. This can be used
 ///                       to limit the number of results and to page through the
 ///                       archive.
-/// \return query id of the request. This can be used to associate the
-///         corresponding resultsRecieved signal.
+/// \return result of the query
 ///
 /// \since QXmpp 1.5
 ///
@@ -316,70 +307,62 @@ QXmppTask<QXmppMamManager::RetrieveResult> QXmppMamManager::retrieveMessages(con
         auto &iq = state.iq;
         iq.parse(std::get<QDomElement>(result));
 
-        // handle MAM error result IQ
-        if (iq.type() == QXmppIq::Error) {
-            state.promise.finish(QXmppError { iq.error().text(), iq.error() });
-            d->ongoingRequests.erase(itr);
-            return;
-        }
-
         // decrypt encrypted messages
         if (auto *e2eeExt = client()->encryptionExtension()) {
             // initialize processed messages (we need random access because
             // decryptMessage() may finish in random order)
             state.processedMessages.resize(state.messages.size());
+            state.runningDecryptionJobs = state.messages.size();
 
-            // check for encrypted messages (once)
-            auto messagesEncrypted = transform(state.messages, [&](const auto &m) {
-                return e2eeExt->isEncrypted(m.element);
-            });
-            auto encryptedCount = sum(messagesEncrypted);
+            const auto size = state.messages.size();
+            for (qsizetype i = 0; i < size; i++) {
+                const auto &message = state.messages.at(i);
 
-            // We can't do this on the fly (with ++ and --) in the for loop
-            // because some decryptMessage() jobs could finish instantly
-            state.runningDecryptionJobs = encryptedCount;
+                // decrypt message if needed
+                if (e2eeExt->isEncrypted(message.element)) {
+                    e2eeExt->decryptMessage(parseMamMessage(state.messages.at(i), Encrypted)).then(this, [this, i, queryId](auto result) {
+                        // find state (again)
+                        auto itr = d->ongoingRequests.find(queryId.toStdString());
+                        Q_ASSERT(itr != d->ongoingRequests.end());
 
-            int size = state.messages.size();
-            for (auto i = 0; i < size; i++) {
-                if (!messagesEncrypted[i]) {
-                    continue;
-                }
+                        auto &state = itr->second;
 
-                e2eeExt->decryptMessage(parseMamMessage(state.messages.at(i), Encrypted)).then(this, [this, i, queryId](auto result) {
-                    auto itr = d->ongoingRequests.find(queryId.toStdString());
-                    Q_ASSERT(itr != d->ongoingRequests.end());
+                        // store decrypted message, fallback to encrypted message
+                        if (std::holds_alternative<QXmppMessage>(result)) {
+                            state.processedMessages[i] = std::get<QXmppMessage>(std::move(result));
+                        } else {
+                            warning(u"Error decrypting message."_s);
+                            state.processedMessages[i] = parseMamMessage(state.messages[i], Unencrypted);
+                        }
 
-                    auto &state = itr->second;
+                        // finish promise on last job
+                        state.runningDecryptionJobs--;
+                        if (state.runningDecryptionJobs == 0) {
+                            state.finish();
+                            d->ongoingRequests.erase(itr);
+                        }
+                    });
+                } else {
+                    state.processedMessages[i] = parseMamMessage(state.messages.at(i), Unencrypted);
 
-                    // store decrypted message, fallback to encrypted message
-                    if (std::holds_alternative<QXmppMessage>(result)) {
-                        state.processedMessages[i] = std::get<QXmppMessage>(std::move(result));
-                    } else {
-                        warning(QStringLiteral("Error decrypting message."));
-                        state.processedMessages[i] = parseMamMessage(state.messages[i], Unencrypted);
-                    }
-
-                    // finish promise if this was the last job
+                    // finish promise on last job (may be needed if no messages are encrypted or
+                    // decryption finishes instantly)
                     state.runningDecryptionJobs--;
                     if (state.runningDecryptionJobs == 0) {
                         state.finish();
                         d->ongoingRequests.erase(itr);
                     }
-                });
+                }
             }
+        } else {
+            // for the case without decryption
+            state.processedMessages = transform<QVector<QXmppMessage>>(state.messages, [](const auto &m) {
+                return parseMamMessage(m, Unencrypted);
+            });
 
-            // finishing the promise is done after decryptMessage()
-            if (encryptedCount > 0) {
-                return;
-            }
+            state.finish();
+            d->ongoingRequests.erase(itr);
         }
-
-        // for the case without decryption, finish here
-        state.processedMessages = transform(state.messages, [](const auto &m) {
-            return parseMamMessage(m, Unencrypted);
-        });
-        state.finish();
-        d->ongoingRequests.erase(itr);
     });
 
     return task;
