@@ -16,7 +16,6 @@
 
 /// Constructs a QXmppMessageReceiptManager to handle incoming and outgoing
 /// message delivery receipts.
-
 QXmppMessageReceiptManager::QXmppMessageReceiptManager()
     : QXmppClientExtension()
 {
@@ -25,7 +24,7 @@ QXmppMessageReceiptManager::QXmppMessageReceiptManager()
 /// \cond
 QStringList QXmppMessageReceiptManager::discoveryFeatures() const
 {
-    return QStringList(ns_message_receipts);
+    return { ns_message_receipts.toString() };
 }
 
 bool QXmppMessageReceiptManager::handleMessage(const QXmppMessage &message)
@@ -48,6 +47,10 @@ bool QXmppMessageReceiptManager::handleMessage(const QXmppMessage &message)
         QXmppMessage receipt;
         receipt.setTo(message.from());
         receipt.setReceiptId(message.id());
+
+        // Advise the server to store the receipt even if the message has no body.
+        receipt.addHint(QXmppMessage::Store);
+
         client()->reply(std::move(receipt), message.e2eeMetadata());
     }
 
