@@ -23,7 +23,7 @@ public:
 class QXmppMucRoomPrivate
 {
 public:
-    QString ownJid() const { return jid + "/" + nickName; }
+    QString ownJid() const { return jid + u'/' + nickName; }
     QXmppClient *client;
     QXmppDiscoveryManager *discoManager;
     QXmppMucRoom::Actions allowedActions;
@@ -75,17 +75,18 @@ QList<QXmppMucRoom *> QXmppMucManager::rooms() const
 QStringList QXmppMucManager::discoveryFeatures() const
 {
     // XEP-0045: Multi-User Chat
-    return QStringList()
-        << ns_muc
-        << ns_muc_admin
-        << ns_muc_owner
-        << ns_muc_user
-        << ns_conference;
+    return {
+        ns_muc.toString(),
+        ns_muc_admin.toString(),
+        ns_muc_owner.toString(),
+        ns_muc_user.toString(),
+        ns_conference.toString(),
+    };
 }
 
 bool QXmppMucManager::handleStanza(const QDomElement &element)
 {
-    if (element.tagName() == "iq") {
+    if (element.tagName() == u"iq") {
         if (QXmppMucAdminIq::isMucAdminIq(element)) {
             QXmppMucAdminIq iq;
             iq.parse(element);
@@ -118,13 +119,17 @@ bool QXmppMucManager::handleStanza(const QDomElement &element)
     return false;
 }
 
-void QXmppMucManager::setClient(QXmppClient *client)
+void QXmppMucManager::onRegistered(QXmppClient *client)
 {
-    QXmppClientExtension::setClient(client);
-
     connect(client, &QXmppClient::messageReceived,
             this, &QXmppMucManager::_q_messageReceived);
 }
+
+void QXmppMucManager::onUnregistered(QXmppClient *client)
+{
+    disconnect(client, &QXmppClient::messageReceived,
+               this, &QXmppMucManager::_q_messageReceived);
+}
 /// \endcond
 
 void QXmppMucManager::_q_messageReceived(const QXmppMessage &msg)
@@ -186,12 +191,13 @@ QXmppMucRoom::Actions QXmppMucRoom::allowedActions() const
     return d->allowedActions;
 }
 
+///
 /// Bans the specified user from the chat room.
 ///
 /// The specified \a jid is the Bare JID of the form "user@host".
 ///
 /// \return true if the request was sent, false otherwise
-
+///
 bool QXmppMucRoom::ban(const QString &jid, const QString &reason)
 {
     if (!QXmppUtils::jidToResource(jid).isEmpty()) {
@@ -222,10 +228,11 @@ QString QXmppMucRoom::jid() const
     return d->jid;
 }
 
+///
 /// Joins the chat room.
 ///
 /// \return true if the request was sent, false otherwise
-
+///
 bool QXmppMucRoom::join()
 {
     if (isJoined() || d->nickName.isEmpty()) {
@@ -241,12 +248,13 @@ bool QXmppMucRoom::join()
     return d->client->sendPacket(packet);
 }
 
+///
 /// Kicks the specified user from the chat room.
 ///
 /// The specified \a jid is the Occupant JID of the form "room@service/nick".
 ///
 /// \return true if the request was sent, false otherwise
-
+///
 bool QXmppMucRoom::kick(const QString &jid, const QString &reason)
 {
     QXmppMucItem item;
@@ -262,12 +270,13 @@ bool QXmppMucRoom::kick(const QString &jid, const QString &reason)
     return d->client->sendPacket(iq);
 }
 
+///
 /// Leaves the chat room.
 ///
 /// \param message An optional message.
 ///
 /// \return true if the request was sent, false otherwise
-
+///
 bool QXmppMucRoom::leave(const QString &message)
 {
     QXmppPresence packet;
@@ -287,13 +296,14 @@ QString QXmppMucRoom::nickName() const
     return d->nickName;
 }
 
+///
 /// Invites a user to the chat room.
 ///
 /// \param jid
 /// \param reason
 ///
 /// \return true if the request was sent, false otherwise
-
+///
 bool QXmppMucRoom::sendInvitation(const QString &jid, const QString &reason)
 {
     QXmppMessage message;
@@ -322,12 +332,13 @@ bool QXmppMucRoom::sendMessage(const QString &text)
     return d->client->sendPacket(msg);
 }
 
+///
 /// Sets your own nickname.
 ///
 /// You need to set your nickname before calling join().
 ///
 /// \param nickName
-
+///
 void QXmppMucRoom::setNickName(const QString &nickName)
 {
     if (nickName == d->nickName) {
@@ -337,7 +348,7 @@ void QXmppMucRoom::setNickName(const QString &nickName)
     // if we had already joined the room, request nickname change
     if (isJoined()) {
         QXmppPresence packet = d->client->clientPresence();
-        packet.setTo(d->jid + "/" + nickName);
+        packet.setTo(d->jid + u'/' + nickName);
         packet.setType(QXmppPresence::Available);
         d->client->sendPacket(packet);
     } else {
@@ -346,10 +357,11 @@ void QXmppMucRoom::setNickName(const QString &nickName)
     }
 }
 
+///
 /// Returns the "Full JID" of the given participant.
 ///
 /// The specified \a jid is the Occupant JID of the form "room@service/nick".
-
+///
 QString QXmppMucRoom::participantFullJid(const QString &jid) const
 {
     if (d->participants.contains(jid)) {
@@ -359,10 +371,11 @@ QString QXmppMucRoom::participantFullJid(const QString &jid) const
     }
 }
 
+///
 /// Returns the presence for the given participant.
 ///
 /// The specified \a jid is the Occupant JID of the form "room@service/nick".
-
+///
 QXmppPresence QXmppMucRoom::participantPresence(const QString &jid) const
 {
     if (d->participants.contains(jid)) {
@@ -385,10 +398,11 @@ QString QXmppMucRoom::password() const
     return d->password;
 }
 
+///
 /// Sets the chat room password.
 ///
 /// \param password
-
+///
 void QXmppMucRoom::setPassword(const QString &password)
 {
     d->password = password;
@@ -399,10 +413,11 @@ QString QXmppMucRoom::subject() const
     return d->subject;
 }
 
+///
 /// Sets the chat room's subject.
 ///
 /// \param subject
-
+///
 void QXmppMucRoom::setSubject(const QString &subject)
 {
     QXmppMessage msg;
@@ -412,12 +427,13 @@ void QXmppMucRoom::setSubject(const QString &subject)
     d->client->sendPacket(msg);
 }
 
+///
 /// Request the configuration form for the chat room.
 ///
 /// \return true if the request was sent, false otherwise
 ///
 /// \sa configurationReceived()
-
+///
 bool QXmppMucRoom::requestConfiguration()
 {
     QXmppMucOwnerIq iq;
@@ -425,12 +441,13 @@ bool QXmppMucRoom::requestConfiguration()
     return d->client->sendPacket(iq);
 }
 
+///
 /// Send the configuration form for the chat room.
 ///
 /// \param form
 ///
 /// \return true if the request was sent, false otherwise
-
+///
 bool QXmppMucRoom::setConfiguration(const QXmppDataForm &form)
 {
     QXmppMucOwnerIq iqPacket;
@@ -440,12 +457,13 @@ bool QXmppMucRoom::setConfiguration(const QXmppDataForm &form)
     return d->client->sendPacket(iqPacket);
 }
 
+///
 /// Request the room's permissions.
 ///
 /// \return true if the request was sent, false otherwise
 ///
 /// \sa permissionsReceived()
-
+///
 bool QXmppMucRoom::requestPermissions()
 {
     QList<QXmppMucItem::Affiliation> affiliations;
@@ -471,12 +489,13 @@ bool QXmppMucRoom::requestPermissions()
     return true;
 }
 
+///
 /// Sets the room's permissions.
 ///
 /// \param permissions
 ///
 /// \return true if the request was sent, false otherwise
-
+///
 bool QXmppMucRoom::setPermissions(const QList<QXmppMucItem> &permissions)
 {
     QList<QXmppMucItem> items;
@@ -542,7 +561,7 @@ void QXmppMucRoom::_q_discoveryInfoReceived(const QXmppDiscoveryIq &iq)
         QString name;
         const auto &identities = iq.identities();
         for (const auto &identity : identities) {
-            if (identity.category() == "conference") {
+            if (identity.category() == u"conference") {
                 name = identity.name();
                 break;
             }
