@@ -1,25 +1,6 @@
-/*
- * Copyright (C) 2008-2021 The QXmpp developers
- *
- * Authors:
- *  Ian Reinhart Geiser
- *
- * Source:
- *  https://github.com/qxmpp-project/qxmpp
- *
- * This file is a part of QXmpp library.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- */
+// SPDX-FileCopyrightText: 2009 Ian Reinhart Geiser <geiseri@kde.org>
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include "QXmppRemoteMethod.h"
 
@@ -58,7 +39,7 @@ void QXmppRemoteMethod::gotError(const QXmppRpcErrorIq &iq)
         m_result.hasError = true;
         m_result.errorMessage = iq.error().text();
         m_result.code = iq.error().type();
-        emit callDone();
+        Q_EMIT callDone();
     }
 }
 
@@ -67,7 +48,8 @@ void QXmppRemoteMethod::gotResult(const QXmppRpcResponseIq &iq)
     if (iq.id() == m_payload.id()) {
         m_result.hasError = false;
         // FIXME: we don't handle multiple responses
-        m_result.result = iq.values().first();
-        emit callDone();
+        const auto values = iq.values();
+        m_result.result = values.first();
+        Q_EMIT callDone();
     }
 }
