@@ -1,21 +1,71 @@
 // SPDX-FileCopyrightText: 2010 Manjeet Dahiya <manjeetdahiya@gmail.com>
 // SPDX-FileCopyrightText: 2010 Jeremy Lainé <jeremy.laine@m4x.org>
 // SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2024 Filipe Azevedo <pasnox@gmail.com>
 //
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include "QXmppRosterManager.h"
 
+#include "QXmppAccountMigrationManager.h"
 #include "QXmppClient.h"
+#include "QXmppConstants_p.h"
 #include "QXmppFutureUtils_p.h"
+#include "QXmppMovedManager.h"
 #include "QXmppPresence.h"
 #include "QXmppRosterIq.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "Algorithms.h"
+#include "StringLiterals.h"
 
 #include <QDomElement>
 
+using namespace QXmpp;
 using namespace QXmpp::Private;
 
+namespace QXmpp::Private {
+
+struct RosterData {
+    using Items = QList<QXmppRosterIq::Item>;
+
+    Items items;
+
+    static std::variant<RosterData, QXmppError> fromDom(const QDomElement &el)
+    {
+        if (el.tagName() != u"roster" && el.namespaceURI() != ns_qxmpp_export) {
+            return QXmppError { u"Invalid element."_s, {} };
+        }
+
+        RosterData d;
+
+        for (const auto &itemEl : iterChildElements(el, u"item", ns_roster)) {
+            QXmppRosterIq::Item item;
+            item.parse(itemEl);
+            d.items.push_back(std::move(item));
+        }
+
+        return d;
+    }
+
+    void toXml(QXmlStreamWriter &writer) const
+    {
+        writer.writeStartElement(QSL65("roster"));
+        for (const auto &item : items) {
+            item.toXml(&writer, true);
+        }
+        writer.writeEndElement();
+    }
+};
+
+static void serializeRosterData(const RosterData &d, QXmlStreamWriter &writer)
+{
+    d.toXml(writer);
+}
+
+}  // namespace QXmpp::Private
+
 ///
 /// \fn QXmppRosterManager::subscriptionRequestReceived
 ///
@@ -24,11 +74,16 @@ using namespace QXmpp::Private;
 /// The user can either accept the request by calling acceptSubscription() or refuse it
 /// by calling refuseSubscription().
 ///
-/// \note If QXmppConfiguration::autoAcceptSubscriptions() is set to true, this
-/// signal will not be emitted.
+/// Since *QXmpp 1.10.2* only verified \xep{0283, Moved} old JIDs are passed in \a presence. If
+/// verification fails or the given old JID is not valid, the attribute is cleared in the
+/// QXmppPresence. See \ref rostermanager_moved "above" for more details.
+///
+/// \note If QXmppConfiguration::autoAcceptSubscriptions() is set to true or the subscription
+/// request is automatically accepted by the QXmppMovedManager, this signal will not be emitted.
 ///
 /// \param subscriberBareJid bare JID that wants to subscribe to the user's presence
-/// \param presence presence stanza containing the reason / message (presence.statusText())
+/// \param presence presence stanza, e.g. containing the message (presence.statusText()),
+/// \xep{0283, Moved} old JID or other information
 ///
 /// \since QXmpp 1.5
 ///
@@ -48,9 +103,6 @@ public:
 
     // flag to store that the roster has been populated
     bool isRosterReceived;
-
-    // id of the initial roster request
-    QString rosterReqId;
 };
 
 QXmppRosterManagerPrivate::QXmppRosterManagerPrivate()
@@ -62,7 +114,6 @@ void QXmppRosterManagerPrivate::clear()
 {
     entries.clear();
     presences.clear();
-    rosterReqId.clear();
     isRosterReceived = false;
 }
 
@@ -72,6 +123,8 @@ void QXmppRosterManagerPrivate::clear()
 QXmppRosterManager::QXmppRosterManager(QXmppClient *client)
     : d(std::make_unique<QXmppRosterManagerPrivate>())
 {
+    QXmppExportData::registerExtension<RosterData, RosterData::fromDom, serializeRosterData>(u"roster", ns_qxmpp_export);
+
     connect(client, &QXmppClient::connected,
             this, &QXmppRosterManager::_q_connected);
 
@@ -116,18 +169,21 @@ void QXmppRosterManager::_q_connected()
         d->clear();
     }
 
-    if (!d->isRosterReceived) {
-        QXmppRosterIq roster;
-        roster.setType(QXmppIq::Get);
-        roster.setFrom(client()->configuration().jid());
+    if (!d->isRosterReceived && client()->isAuthenticated()) {
+        requestRoster().then(this, [this](auto &&result) {
+            if (auto *rosterIq = std::get_if<QXmppRosterIq>(&result)) {
+                // reset entries
+                d->entries.clear();
+                const auto items = rosterIq->items();
+                for (const auto &item : items) {
+                    d->entries.insert(item.bareJid(), item);
+                }
 
-        // TODO: Request MIX annotations only when the server supports MIX-PAM.
-        roster.setMixAnnotate(true);
-
-        d->rosterReqId = roster.id();
-        if (client()->isAuthenticated()) {
-            client()->sendPacket(roster);
-        }
+                // notify
+                d->isRosterReceived = true;
+                Q_EMIT rosterReceived();
+            }
+        });
     }
 }
 
@@ -142,13 +198,13 @@ void QXmppRosterManager::_q_disconnected()
 /// \cond
 bool QXmppRosterManager::handleStanza(const QDomElement &element)
 {
-    if (element.tagName() != "iq" || !QXmppRosterIq::isRosterIq(element)) {
+    if (element.tagName() != u"iq" || !QXmppRosterIq::isRosterIq(element)) {
         return false;
     }
 
     // Security check: only server should send this iq
     // from() should be either empty or bareJid of the user
-    const auto fromJid = element.attribute("from");
+    const auto fromJid = element.attribute(u"from"_s);
     if (!fromJid.isEmpty() && QXmppUtils::jidToBareJid(fromJid) != client()->configuration().jidBare()) {
         return false;
     }
@@ -156,11 +212,6 @@ bool QXmppRosterManager::handleStanza(const QDomElement &element)
     QXmppRosterIq rosterIq;
     rosterIq.parse(element);
 
-    bool isInitial = (d->rosterReqId == rosterIq.id());
-    if (isInitial) {
-        d->rosterReqId.clear();
-    }
-
     switch (rosterIq.type()) {
     case QXmppIq::Set: {
         // send result iq
@@ -189,17 +240,6 @@ bool QXmppRosterManager::handleStanza(const QDomElement &element)
                 }
             }
         }
-    } break;
-    case QXmppIq::Result: {
-        const auto items = rosterIq.items();
-        for (const auto &item : items) {
-            const auto bareJid = item.bareJid();
-            d->entries.insert(bareJid, item);
-        }
-        if (isInitial) {
-            d->isRosterReceived = true;
-            Q_EMIT rosterReceived();
-        }
         break;
     }
     default:
@@ -229,23 +269,54 @@ void QXmppRosterManager::_q_presenceReceived(const QXmppPresence &presence)
         d->presences[bareJid].remove(resource);
         Q_EMIT presenceChanged(bareJid, resource);
         break;
-    case QXmppPresence::Subscribe:
-        if (client()->configuration().autoAcceptSubscriptions()) {
-            // accept subscription request
-            acceptSubscription(bareJid);
-
-            // ask for reciprocal subscription
-            subscribe(bareJid);
-        } else {
-            Q_EMIT subscriptionReceived(bareJid);
-            Q_EMIT subscriptionRequestReceived(bareJid, presence);
-        }
+    case QXmppPresence::Subscribe: {
+        handleSubscriptionRequest(bareJid, presence);
         break;
+    }
     default:
         break;
     }
 }
 
+void QXmppRosterManager::handleSubscriptionRequest(const QString &bareJid, const QXmppPresence &presence)
+{
+    auto notifyOnSubscriptionRequest = [this, bareJid](const QXmppPresence &presence) {
+        Q_EMIT subscriptionReceived(bareJid);
+        Q_EMIT subscriptionRequestReceived(bareJid, presence);
+    };
+
+    // Automatically accept all incoming subscription requests if enabled.
+    if (client()->configuration().autoAcceptSubscriptions()) {
+        acceptSubscription(bareJid);
+        subscribe(bareJid);
+        return;
+    }
+
+    // check for XEP-0283: Moved subscription requests and verify them
+    if (auto *movedManager = client()->findExtension<QXmppMovedManager>(); movedManager && !presence.oldJid().isEmpty()) {
+        movedManager->processSubscriptionRequest(presence).then(this, [this, notifyOnSubscriptionRequest](QXmppPresence &&presence) mutable {
+            notifyOnSubscriptionRequest(presence);
+        });
+    } else {
+        auto safePresence = presence;
+        safePresence.setOldJid({});
+
+        notifyOnSubscriptionRequest(safePresence);
+    }
+}
+
+QXmppTask<QXmppRosterManager::RosterResult> QXmppRosterManager::requestRoster()
+{
+    QXmppRosterIq iq;
+    iq.setType(QXmppIq::Get);
+    iq.setFrom(client()->configuration().jid());
+
+    // TODO: Request MIX annotations only when the server supports MIX-PAM.
+    iq.setMixAnnotate(true);
+
+    return chainIq<RosterResult>(client()->sendIq(std::move(iq)), this);
+}
+
 ///
 /// Adds a new item to the roster without sending any subscription requests.
 ///
@@ -307,10 +378,9 @@ QXmppTask<QXmppRosterManager::Result> QXmppRosterManager::removeRosterItem(const
 ///
 QXmppTask<QXmppRosterManager::Result> QXmppRosterManager::renameRosterItem(const QString &bareJid, const QString &name)
 {
-    using Error = QXmppStanza::Error;
     if (!d->entries.contains(bareJid)) {
         return makeReadyTask<Result>(
-            QXmppError { QStringLiteral("The roster doesn't contain this user."), {} });
+            QXmppError { u"The roster doesn't contain this user."_s, {} });
     }
 
     auto item = d->entries.value(bareJid);
@@ -478,6 +548,74 @@ bool QXmppRosterManager::unsubscribe(const QString &bareJid, const QString &reas
     return client()->sendPacket(packet);
 }
 
+void QXmppRosterManager::onRegistered(QXmppClient *client)
+{
+    // data import/export
+    if (auto manager = client->findExtension<QXmppAccountMigrationManager>()) {
+        using ImportResult = std::variant<Success, QXmppError>;
+        auto importData = [this, client, manager](const RosterData &data) -> QXmppTask<ImportResult> {
+            if (data.items.isEmpty()) {
+                return makeReadyTask<ImportResult>(Success());
+            }
+
+            QXmppPromise<ImportResult> promise;
+            auto counter = std::make_shared<int>(data.items.size());
+
+            for (const auto &item : std::as_const(data.items)) {
+                Q_ASSERT(!item.isMixChannel());
+
+                QXmppRosterIq iq;
+                iq.addItem(item);
+                iq.setType(QXmppIq::Set);
+
+                client->sendGenericIq(std::move(iq)).then(this, [promise, counter](auto &&result) mutable {
+                    if (promise.task().isFinished()) {
+                        return;
+                    }
+
+                    if (auto error = std::get_if<QXmppError>(&result); error) {
+                        return promise.finish(std::move(*error));
+                    }
+
+                    if ((--(*counter)) == 0) {
+                        return promise.finish(Success());
+                    }
+                });
+            }
+
+            return promise.task();
+        };
+        auto exportData = [this]() {
+            return chainMapSuccess(requestRoster(), this, [](QXmppRosterIq &&iq) -> RosterData {
+                const auto items = transformFilter<RosterData::Items>(iq.items(), [](const auto &item) -> std::optional<QXmppRosterIq::Item> {
+                    if (item.isMixChannel()) {
+                        return {};
+                    }
+
+                    auto fixed = item;
+
+                    // We don't want this to be sent while importing.
+                    // See https://datatracker.ietf.org/doc/html/rfc6121#section-2.1.2.2
+                    fixed.setSubscriptionStatus({});
+
+                    return fixed;
+                });
+
+                return { items };
+            });
+        };
+
+        manager->registerExportData<RosterData>(importData, exportData);
+    }
+}
+
+void QXmppRosterManager::onUnregistered(QXmppClient *client)
+{
+    if (auto manager = client->findExtension<QXmppAccountMigrationManager>()) {
+        manager->unregisterExportData<RosterData>();
+    }
+}
+
 ///
 /// Function to get all the bareJids present in the roster.
 ///
