@@ -1,26 +1,9 @@
-/*
- * Copyright (C) 2008-2021 The QXmpp developers
- *
- * Authors:
- *  Manjeet Dahiya
- *  Jeremy Lainé
- *
- * Source:
- *  https://github.com/qxmpp-project/qxmpp
- *
- * This file is a part of QXmpp library.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- */
+// SPDX-FileCopyrightText: 2010 Manjeet Dahiya <manjeetdahiya@gmail.com>
+// SPDX-FileCopyrightText: 2010 Jeremy Lainé <jeremy.laine@m4x.org>
+// SPDX-FileCopyrightText: 2021 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2024 Filipe Azevedo <pasnox@gmail.com>
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #ifndef QXMPPROSTERMANAGER_H
 #define QXMPPROSTERMANAGER_H
@@ -28,11 +11,16 @@
 #include "QXmppClientExtension.h"
 #include "QXmppPresence.h"
 #include "QXmppRosterIq.h"
+#include "QXmppSendResult.h"
+
+#include <variant>
 
 #include <QMap>
 #include <QObject>
 #include <QStringList>
 
+template<typename T>
+class QXmppTask;
 class QXmppRosterManagerPrivate;
 
 ///
@@ -63,6 +51,33 @@ class QXmppRosterManagerPrivate;
 /// The \c presenceChanged() signal is emitted whenever the presence for a
 /// roster item changes.
 ///
+/// \anchor rostermanager_moved
+/// ## XEP-0283: Moved
+///
+/// \xep{0283, Moved} provides a way to inform other users that an account has moved. This allows a
+/// presence subscription request from a new account to be automatically linked to the old account.
+/// However, this requires verification via a corresponding PubSub node on the old account.
+///
+/// The roster manager automatically checks entries using the QXmppMovedManager. For this to work,
+/// the moved manager must be added to the QXmppClient. If the moved manager is not found or the
+/// specified old JID is incorrect, the entry in the presence subscription request will be removed.
+///
+/// The received and verified presence subscription request is emitted via
+/// subscriptionRequestReceived() and the old JID can be found in QXmppPresence::oldJid().
+/// It is safe to assume that this old JID is valid since QXmpp 1.10.2.
+///
+/// Accepting moved subscription requests automatically is discouraged in
+/// [section 4.3](https://xmpp.org/extensions/xep-0283.html#receive-notification-client) of the
+/// XEP.
+///
+/// ### Behaviour in previous versions
+///
+/// Support in the roster manager and in QXmppPresence was initially added in 1.9.0.
+///
+/// In QXmpp 1.9.0 until 1.9.4 and 1.10.0 until 1.10.1 subscription requests with a Moved element
+/// are verified and automatically accepted without emitting subscriptionRequestReceived(). If
+/// verification fails, the **invalid** old JID is passed in subscriptionRequestReceived()!
+///
 /// \ingroup Managers
 ///
 class QXMPP_EXPORT QXmppRosterManager : public QXmppClientExtension
@@ -70,7 +85,10 @@ class QXMPP_EXPORT QXmppRosterManager : public QXmppClientExtension
     Q_OBJECT
 
 public:
-    QXmppRosterManager(QXmppClient *stream);
+    /// Empty result containing QXmpp::Success or a QXmppError
+    using Result = std::variant<QXmpp::Success, QXmppError>;
+
+    explicit QXmppRosterManager(QXmppClient *stream);
     ~QXmppRosterManager() override;
 
     bool isRosterReceived() const;
@@ -83,6 +101,12 @@ public:
     QXmppPresence getPresence(const QString &bareJid,
                               const QString &resource) const;
 
+    QXmppTask<Result> addRosterItem(const QString &bareJid, const QString &name = {}, const QSet<QString> &groups = {});
+    QXmppTask<Result> removeRosterItem(const QString &bareJid);
+    QXmppTask<Result> renameRosterItem(const QString &bareJid, const QString &name);
+    QXmppTask<QXmpp::SendResult> subscribeTo(const QString &bareJid, const QString &reason = {});
+    QXmppTask<QXmpp::SendResult> unsubscribeFrom(const QString &bareJid, const QString &reason = {});
+
     /// \cond
     bool handleStanza(const QDomElement &element) override;
     /// \endcond
@@ -116,9 +140,13 @@ Q_SIGNALS:
     /// by calling refuseSubscription().
     ///
     /// \note If you set QXmppConfiguration::autoAcceptSubscriptions() to true, this
-    /// signal will not be emitted.
+    /// signal will not be emitted. This is only valid for non moved or verified moved subscription.
+    /// If the subscription is a moved one and the roster's old-jid's subscription is not either from
+    /// or both then QXmppConfiguration::autoAcceptSubscriptions() is ignored.
     void subscriptionReceived(const QString &bareJid);
 
+    void subscriptionRequestReceived(const QString &subscriberBareJid, const QXmppPresence &presence);
+
     /// This signal is emitted when the roster entry of a particular bareJid is
     /// added as a result of roster push.
     void itemAdded(const QString &bareJid);
@@ -131,13 +159,24 @@ Q_SIGNALS:
     /// removed as a result of roster push.
     void itemRemoved(const QString &bareJid);
 
+protected:
+    void onRegistered(QXmppClient *client) override;
+    void onUnregistered(QXmppClient *client) override;
+
 private Q_SLOTS:
     void _q_connected();
     void _q_disconnected();
     void _q_presenceReceived(const QXmppPresence &);
 
 private:
-    QXmppRosterManagerPrivate *d;
+    using RosterResult = std::variant<QXmppRosterIq, QXmppError>;
+
+    void handleSubscriptionRequest(const QString &bareJid, const QXmppPresence &presence);
+    QXmppTask<RosterResult> requestRoster();
+
+    const std::unique_ptr<QXmppRosterManagerPrivate> d;
+
+    friend class QXmppMixManager;
 };
 
 #endif  // QXMPPROSTER_H
