@@ -10,6 +10,8 @@
 #include "QXmppRemoteMethod.h"
 #include "QXmppRpcIq.h"
 
+#include "StringLiterals.h"
+
 /// Constructs a QXmppRpcManager.
 
 QXmppRpcManager::QXmppRpcManager()
@@ -22,7 +24,7 @@ QXmppRpcManager::QXmppRpcManager()
 
 void QXmppRpcManager::addInvokableInterface(QXmppInvokable *interface)
 {
-    m_interfaces[interface->metaObject()->className()] = interface;
+    m_interfaces[QString::fromUtf8(interface->metaObject()->className())] = interface;
 }
 
 /// Invokes a remote interface using RPC.
@@ -33,7 +35,7 @@ void QXmppRpcManager::invokeInterfaceMethod(const QXmppRpcInvokeIq &iq)
 {
     QXmppStanza::Error error;
 
-    const QStringList methodBits = iq.method().split('.');
+    const QStringList methodBits = iq.method().split(u'.');
     if (methodBits.size() != 2) {
         return;
     }
@@ -139,15 +141,17 @@ QXmppRemoteMethodResult QXmppRpcManager::callRemoteMethod(const QString &jid,
 /// \cond
 QStringList QXmppRpcManager::discoveryFeatures() const
 {
-    // XEP-0009: Jabber-RPC
-    return QStringList() << ns_rpc;
+    return {
+        // XEP-0009: Jabber-RPC
+        ns_rpc.toString(),
+    };
 }
 
 QList<QXmppDiscoveryIq::Identity> QXmppRpcManager::discoveryIdentities() const
 {
     QXmppDiscoveryIq::Identity identity;
-    identity.setCategory("automation");
-    identity.setType("rpc");
+    identity.setCategory(u"automation"_s);
+    identity.setType(u"rpc"_s);
     return QList<QXmppDiscoveryIq::Identity>() << identity;
 }
 
