@@ -13,6 +13,9 @@
 #include "QXmppStun.h"
 #include "QXmppTransferManager_p.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QCryptographicHash>
 #include <QDomElement>
@@ -27,6 +30,8 @@
 #include <QTimer>
 #include <QUrl>
 
+using namespace QXmpp::Private;
+
 // time to try to connect to a SOCKS host (7 seconds)
 const int socksTimeout = 7000;
 
@@ -35,7 +40,7 @@ static QString streamHash(const QString &sid, const QString &initiatorJid, const
     QCryptographicHash hash(QCryptographicHash::Sha1);
     QString str = sid + initiatorJid + targetJid;
     hash.addData(str.toLatin1());
-    return hash.result().toHex();
+    return QString::fromUtf8(hash.result().toHex());
 }
 
 class QXmppTransferFileInfoPrivate : public QSharedData
@@ -169,31 +174,31 @@ bool QXmppTransferFileInfo::operator==(const QXmppTransferFileInfo &other) const
 /// \cond
 void QXmppTransferFileInfo::parse(const QDomElement &element)
 {
-    d->date = QXmppUtils::datetimeFromString(element.attribute("date"));
-    d->hash = QByteArray::fromHex(element.attribute("hash").toLatin1());
-    d->name = element.attribute("name");
-    d->size = element.attribute("size").toLongLong();
-    d->description = element.firstChildElement("desc").text();
+    d->date = QXmppUtils::datetimeFromString(element.attribute(u"date"_s));
+    d->hash = QByteArray::fromHex(element.attribute(u"hash"_s).toLatin1());
+    d->name = element.attribute(u"name"_s);
+    d->size = element.attribute(u"size"_s).toLongLong();
+    d->description = firstChildElement(element, u"desc").text();
 }
 
 void QXmppTransferFileInfo::toXml(QXmlStreamWriter *writer) const
 {
-    writer->writeStartElement("file");
-    writer->writeDefaultNamespace(ns_stream_initiation_file_transfer);
+    writer->writeStartElement(QSL65("file"));
+    writer->writeDefaultNamespace(toString65(ns_stream_initiation_file_transfer));
     if (d->date.isValid()) {
-        writer->writeAttribute("date", QXmppUtils::datetimeToString(d->date));
+        writer->writeAttribute(QSL65("date"), QXmppUtils::datetimeToString(d->date));
     }
     if (!d->hash.isEmpty()) {
-        writer->writeAttribute("hash", d->hash.toHex());
+        writer->writeAttribute(QSL65("hash"), QString::fromUtf8(d->hash.toHex()));
     }
     if (!d->name.isEmpty()) {
-        writer->writeAttribute("name", d->name);
+        writer->writeAttribute(QSL65("name"), d->name);
     }
     if (d->size > 0) {
-        writer->writeAttribute("size", QString::number(d->size));
+        writer->writeAttribute(QSL65("size"), QString::number(d->size));
     }
     if (!d->description.isEmpty()) {
-        writer->writeTextElement("desc", d->description);
+        writer->writeTextElement(QSL65("desc"), d->description);
     }
     writer->writeEndElement();
 }
@@ -284,7 +289,7 @@ void QXmppTransferJob::accept(const QString &filePath)
     if (d->direction == IncomingDirection && d->state == OfferState && !d->iodevice) {
         auto *file = new QFile(filePath, this);
         if (!file->open(QIODevice::WriteOnly)) {
-            warning(QString("Could not write to %1").arg(filePath));
+            warning(u"Could not write to %1"_s.arg(filePath));
             abort();
             return;
         }
@@ -487,7 +492,7 @@ void QXmppTransferIncomingJob::connectToNextHost()
 
     // try next host
     m_candidateHost = m_streamCandidates.takeFirst();
-    info(QString("Connecting to streamhost: %1 (%2 %3)").arg(m_candidateHost.jid(), m_candidateHost.host(), QString::number(m_candidateHost.port())));
+    info(u"Connecting to streamhost: %1 (%2 %3)"_s.arg(m_candidateHost.jid(), m_candidateHost.host(), QString::number(m_candidateHost.port())));
 
     const QString hostName = streamHash(d->sid,
                                         d->jid,
@@ -538,7 +543,7 @@ void QXmppTransferIncomingJob::_q_candidateReady()
         return;
     }
 
-    info(QString("Connected to streamhost: %1 (%2 %3)").arg(m_candidateHost.jid(), m_candidateHost.host(), QString::number(m_candidateHost.port())));
+    info(u"Connected to streamhost: %1 (%2 %3)"_s.arg(m_candidateHost.jid(), m_candidateHost.host(), QString::number(m_candidateHost.port())));
 
     setState(QXmppTransferJob::TransferState);
     d->socksSocket = m_candidateClient;
@@ -564,7 +569,7 @@ void QXmppTransferIncomingJob::_q_candidateDisconnected()
         return;
     }
 
-    warning(QString("Failed to connect to streamhost: %1 (%2 %3)").arg(m_candidateHost.jid(), m_candidateHost.host(), QString::number(m_candidateHost.port())));
+    warning(u"Failed to connect to streamhost: %1 (%2 %3)"_s.arg(m_candidateHost.jid(), m_candidateHost.host(), QString::number(m_candidateHost.port())));
 
     m_candidateClient->deleteLater();
     m_candidateClient = nullptr;
@@ -608,7 +613,7 @@ QXmppTransferOutgoingJob::QXmppTransferOutgoingJob(const QString &jid, QXmppClie
 
 void QXmppTransferOutgoingJob::connectToProxy()
 {
-    info(QString("Connecting to proxy: %1 (%2 %3)").arg(d->socksProxy.jid(), d->socksProxy.host(), QString::number(d->socksProxy.port())));
+    info(u"Connecting to proxy: %1 (%2 %3)"_s.arg(d->socksProxy.jid(), d->socksProxy.host(), QString::number(d->socksProxy.port())));
 
     const QString hostName = streamHash(d->sid,
                                         d->client->configuration().jid(),
@@ -854,7 +859,7 @@ void QXmppTransferManager::byteStreamResultReceived(const QXmppByteStreamIq &iq)
 
     // direction connection, start sending data
     if (!job->d->socksSocket) {
-        warning("Client says they connected to our SOCKS server, but they did not");
+        warning(u"Client says they connected to our SOCKS server, but they did not"_s);
         job->terminate(QXmppTransferJob::ProtocolError);
         return;
     }
@@ -893,16 +898,16 @@ void QXmppTransferManager::byteStreamSetReceived(const QXmppByteStreamIq &iq)
 QStringList QXmppTransferManager::discoveryFeatures() const
 {
     return {
-        ns_ibb,                              // XEP-0047: In-Band Bytestreams
-        ns_bytestreams,                      // XEP-0065: SOCKS5 Bytestreams
-        ns_stream_initiation,                // XEP-0095: Stream Initiation
-        ns_stream_initiation_file_transfer,  // XEP-0096: SI File Transfer
+        ns_ibb.toString(),                              // XEP-0047: In-Band Bytestreams
+        ns_bytestreams.toString(),                      // XEP-0065: SOCKS5 Bytestreams
+        ns_stream_initiation.toString(),                // XEP-0095: Stream Initiation
+        ns_stream_initiation_file_transfer.toString(),  // XEP-0096: SI File Transfer
     };
 }
 
 bool QXmppTransferManager::handleStanza(const QDomElement &element)
 {
-    if (element.tagName() != "iq") {
+    if (element.tagName() != u"iq") {
         return false;
     }
 
@@ -941,13 +946,16 @@ bool QXmppTransferManager::handleStanza(const QDomElement &element)
     return false;
 }
 
-void QXmppTransferManager::setClient(QXmppClient *client)
+void QXmppTransferManager::onRegistered(QXmppClient *client)
 {
-    QXmppClientExtension::setClient(client);
-
     // XEP-0047: In-Band Bytestreams
     connect(client, &QXmppClient::iqReceived, this, &QXmppTransferManager::_q_iqReceived);
 }
+
+void QXmppTransferManager::onUnregistered(QXmppClient *client)
+{
+    disconnect(client, &QXmppClient::iqReceived, this, &QXmppTransferManager::_q_iqReceived);
+}
 /// \endcond
 
 void QXmppTransferManager::ibbCloseIqReceived(const QXmppIbbCloseIq &iq)
@@ -1109,7 +1117,7 @@ void QXmppTransferManager::_q_iqReceived(const QXmppIq &iq)
                     job->startSending();
                 } else if (iq.type() == QXmppIq::Error) {
                     // proxy stream not activated, terminate
-                    warning("Could not activate SOCKS5 proxy bytestream");
+                    warning(u"Could not activate SOCKS5 proxy bytestream"_s);
                     job->terminate(QXmppTransferJob::ProtocolError);
                 }
             } else {
@@ -1214,11 +1222,11 @@ void QXmppTransferManager::_q_jobStateChanged(QXmppTransferJob::State state)
     form.setType(QXmppDataForm::Submit);
 
     QXmppDataForm::Field methodField(QXmppDataForm::Field::ListSingleField);
-    methodField.setKey("stream-method");
+    methodField.setKey(u"stream-method"_s);
     if (job->method() == QXmppTransferJob::InBandMethod) {
-        methodField.setValue(ns_ibb);
+        methodField.setValue(ns_ibb.toString());
     } else if (job->method() == QXmppTransferJob::SocksMethod) {
-        methodField.setValue(ns_bytestreams);
+        methodField.setValue(ns_bytestreams.toString());
     }
     form.setFields(QList<QXmppDataForm::Field>() << methodField);
 
@@ -1249,7 +1257,7 @@ void QXmppTransferManager::_q_jobStateChanged(QXmppTransferJob::State state)
 QXmppTransferJob *QXmppTransferManager::sendFile(const QString &jid, const QString &filePath, const QString &description)
 {
     if (QXmppUtils::jidToResource(jid).isEmpty()) {
-        warning("The file recipient's JID must be a full JID");
+        warning(u"The file recipient's JID must be a full JID"_s);
         return nullptr;
     }
 
@@ -1264,7 +1272,7 @@ QXmppTransferJob *QXmppTransferManager::sendFile(const QString &jid, const QStri
     // open file
     QIODevice *device = new QFile(filePath, this);
     if (!device->open(QIODevice::ReadOnly)) {
-        warning(QString("Could not read from %1").arg(filePath));
+        warning(u"Could not read from %1"_s.arg(filePath));
         delete device;
         device = nullptr;
     }
@@ -1302,7 +1310,7 @@ QXmppTransferJob *QXmppTransferManager::sendFile(const QString &jid, const QStri
 QXmppTransferJob *QXmppTransferManager::sendFile(const QString &jid, QIODevice *device, const QXmppTransferFileInfo &fileInfo, const QString &sid)
 {
     if (QXmppUtils::jidToResource(jid).isEmpty()) {
-        warning("The file recipient's JID must be a full JID");
+        warning(u"The file recipient's JID must be a full JID"_s);
         return nullptr;
     }
 
@@ -1328,12 +1336,12 @@ QXmppTransferJob *QXmppTransferManager::sendFile(const QString &jid, QIODevice *
     form.setType(QXmppDataForm::Form);
 
     QXmppDataForm::Field methodField(QXmppDataForm::Field::ListSingleField);
-    methodField.setKey("stream-method");
+    methodField.setKey(u"stream-method"_s);
     if (d->supportedMethods & QXmppTransferJob::InBandMethod) {
-        methodField.setOptions(methodField.options() << qMakePair(QString(), QString::fromLatin1(ns_ibb)));
+        methodField.setOptions(methodField.options() << qMakePair(QString(), ns_ibb.toString()));
     }
     if (d->supportedMethods & QXmppTransferJob::SocksMethod) {
-        methodField.setOptions(methodField.options() << qMakePair(QString(), QString::fromLatin1(ns_bytestreams)));
+        methodField.setOptions(methodField.options() << qMakePair(QString(), ns_bytestreams.toString()));
     }
     form.setFields(QList<QXmppDataForm::Field>() << methodField);
 
@@ -1369,7 +1377,7 @@ void QXmppTransferManager::_q_socksServerConnected(QTcpSocket *socket, const QSt
             return;
         }
     }
-    warning("QXmppSocksServer got a connection for a unknown stream");
+    warning(u"QXmppSocksServer got a connection for a unknown stream"_s);
     socket->close();
 }
 
@@ -1397,7 +1405,7 @@ void QXmppTransferManager::socksServerSendOffer(QXmppTransferJob *job)
 
     // check we have some stream hosts
     if (!streamHosts.size()) {
-        warning("Could not determine local stream hosts");
+        warning(u"Could not determine local stream hosts"_s);
         job->terminate(QXmppTransferJob::ProtocolError);
         return;
     }
@@ -1434,7 +1442,7 @@ void QXmppTransferManager::streamInitiationResultReceived(const QXmppStreamIniti
     const auto &fields = form.fields();
 
     for (const auto &field : fields) {
-        if (field.key() == "stream-method") {
+        if (field.key() == u"stream-method") {
             if ((field.value().toString() == ns_ibb) &&
                 (d->supportedMethods & QXmppTransferJob::InBandMethod)) {
                 job->d->method = QXmppTransferJob::InBandMethod;
@@ -1472,7 +1480,7 @@ void QXmppTransferManager::streamInitiationResultReceived(const QXmppStreamIniti
             socksServerSendOffer(job);
         }
     } else {
-        warning("QXmppTransferManager received an unsupported method");
+        warning(u"QXmppTransferManager received an unsupported method"_s);
         job->terminate(QXmppTransferJob::ProtocolError);
     }
 }
@@ -1518,7 +1526,7 @@ void QXmppTransferManager::streamInitiationSetReceived(const QXmppStreamInitiati
     const auto &form = iq.featureForm();
     const auto &fields = form.fields();
     for (const auto &field : fields) {
-        if (field.key() == "stream-method") {
+        if (field.key() == u"stream-method") {
             QPair<QString, QString> option;
             const auto &options = field.options();
             for (const auto &option : options) {
