@@ -11,6 +11,9 @@
 #include "QXmppPubSubEvent.h"
 #include "QXmppTrustManager.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "StringLiterals.h"
 
 #include <QStringBuilder>
 
@@ -25,7 +28,7 @@ using Manager = QXmppOmemoManager;
 using ManagerPrivate = QXmppOmemoManagerPrivate;
 
 // default label used for the own device
-const auto DEVICE_LABEL = QStringLiteral("QXmpp");
+const auto DEVICE_LABEL = u"QXmpp"_s;
 
 class QXmppOmemoOwnDevicePrivate : public QSharedData
 {
@@ -250,9 +253,11 @@ void QXmppOmemoDevice::setTrustLevel(TrustLevel trustLevel)
 /// client->addNewExtension<QXmppAtmManager>(trustStorage);
 /// \endcode
 ///
-/// Afterwards, the OMEMO manager using its storage must be added to the client:
+/// Afterwards, the OMEMO manager using its storage must be added to the client and activated to be
+/// used for encryption:
 /// \code
 /// auto *manager = client->addNewExtension<QXmppOmemoManager>(omemoStorage);
+/// client->setEncryptionExtension(manager);
 /// \endcode
 ///
 /// You can set a security policy used by OMEMO.
@@ -353,29 +358,28 @@ QXmppTask<bool> Manager::load()
     QXmppPromise<bool> interface;
 
     auto future = d->omemoStorage->allData();
-    future.then(this, [=](QXmppOmemoStorage::OmemoData omemoData) mutable {
+    future.then(this, [=, this](QXmppOmemoStorage::OmemoData omemoData) mutable {
         const auto &optionalOwnDevice = omemoData.ownDevice;
         if (optionalOwnDevice) {
             d->ownDevice = *optionalOwnDevice;
         } else {
-            debug("Device could not be loaded because it is not stored");
+            debug(u"Device could not be loaded because it is not stored"_s);
             interface.finish(false);
             return;
         }
 
         const auto &signedPreKeyPairs = omemoData.signedPreKeyPairs;
         if (signedPreKeyPairs.isEmpty()) {
-            warning("Signed Pre keys could not be loaded because none is stored");
+            warning(u"Signed Pre keys could not be loaded because none is stored"_s);
             interface.finish(false);
             return;
         } else {
             d->signedPreKeyPairs = signedPreKeyPairs;
-            d->renewSignedPreKeyPairs();
         }
 
         const auto &preKeyPairs = omemoData.preKeyPairs;
         if (preKeyPairs.isEmpty()) {
-            warning("Pre keys could not be loaded because none is stored");
+            warning(u"Pre keys could not be loaded because none is stored"_s);
             interface.finish(false);
             return;
         } else {
@@ -404,7 +408,7 @@ QXmppTask<bool> Manager::setUp()
     QXmppPromise<bool> interface;
 
     auto future = d->setUpDeviceId();
-    future.then(this, [=](bool isDeviceIdSetUp) mutable {
+    future.then(this, [=, this](bool isDeviceIdSetUp) mutable {
         if (isDeviceIdSetUp) {
             // The identity key pair in its deserialized form is not stored as a
             // member variable because it is only needed by
@@ -415,9 +419,9 @@ QXmppTask<bool> Manager::setUp()
                 d->updateSignedPreKeyPair(identityKeyPair.get()) &&
                 d->updatePreKeyPairs(PRE_KEY_INITIAL_CREATION_COUNT)) {
                 auto future = d->omemoStorage->setOwnDevice(d->ownDevice);
-                future.then(this, [=]() mutable {
+                future.then(this, [=, this]() mutable {
                     auto future = d->publishOmemoData();
-                    future.then(this, [=](bool isPublished) mutable {
+                    future.then(this, [=, this](bool isPublished) mutable {
                         d->isStarted = isPublished;
                         interface.finish(std::move(isPublished));
                     });
@@ -440,12 +444,12 @@ QXmppTask<bool> Manager::setUp()
 ///
 QXmppTask<QByteArray> Manager::ownKey()
 {
-    return d->trustManager->ownKey(ns_omemo_2);
+    return d->trustManager->ownKey(ns_omemo_2.toString());
 }
 
 ///
 /// Returns the JIDs of all key owners mapped to the IDs of their keys with
-/// specific trust levels.
+/// specific trust levels except for the own key.
 ///
 /// If no trust levels are passed, all keys are returned.
 ///
@@ -455,16 +459,17 @@ QXmppTask<QByteArray> Manager::ownKey()
 ///
 /// \param trustLevels trust levels of the keys
 ///
-/// \return the key owner JIDs mapped to their keys with specific trust levels
+/// \return the key owner JIDs mapped to their keys with specific trust levels except for the own
+///         key
 ///
 QXmppTask<QHash<QXmpp::TrustLevel, QMultiHash<QString, QByteArray>>> Manager::keys(QXmpp::TrustLevels trustLevels)
 {
-    return d->trustManager->keys(ns_omemo_2, trustLevels);
+    return d->trustManager->keys(ns_omemo_2.toString(), trustLevels);
 }
 
 ///
 /// Returns the IDs of keys mapped to their trust levels for specific key
-/// owners.
+/// owners except for the own key.
 ///
 /// If no trust levels are passed, all keys for jids are returned.
 ///
@@ -475,11 +480,11 @@ QXmppTask<QHash<QXmpp::TrustLevel, QMultiHash<QString, QByteArray>>> Manager::ke
 /// \param jids key owners' bare JIDs
 /// \param trustLevels trust levels of the keys
 ///
-/// \return the key IDs mapped to their trust levels for specific key owners
+/// \return the key IDs mapped to their trust levels for specific key owners except for the own key
 ///
 QXmppTask<QHash<QString, QHash<QByteArray, QXmpp::TrustLevel>>> Manager::keys(const QList<QString> &jids, QXmpp::TrustLevels trustLevels)
 {
-    return d->trustManager->keys(ns_omemo_2, jids, trustLevels);
+    return d->trustManager->keys(ns_omemo_2.toString(), jids, trustLevels);
 }
 
 ///
@@ -567,8 +572,7 @@ void Manager::setMaximumDevicesPerStanza(int maximum)
 QXmppTask<QVector<Manager::DevicesResult>> Manager::requestDeviceLists(const QList<QString> &jids)
 {
     if (const auto jidsCount = jids.size()) {
-        struct State
-        {
+        struct State {
             int processed = 0;
             int jidsCount = 0;
             QXmppPromise<QVector<Manager::DevicesResult>> interface;
@@ -615,8 +619,7 @@ QXmppTask<QVector<Manager::DevicesResult>> Manager::requestDeviceLists(const QLi
 QXmppTask<QVector<Manager::DevicesResult>> Manager::subscribeToDeviceLists(const QList<QString> &jids)
 {
     if (const auto jidsCount = jids.size()) {
-        struct State
-        {
+        struct State {
             int processed = 0;
             int jidsCount = 0;
             QXmppPromise<QVector<Manager::DevicesResult>> interface;
@@ -683,7 +686,7 @@ QXmppOmemoOwnDevice Manager::ownDevice()
 ///
 /// You must build sessions before you can get devices with corresponding keys.
 ///
-/// /\return all devices except the own device
+/// \return all devices except the own device
 ///
 QXmppTask<QVector<QXmppOmemoDevice>> Manager::devices()
 {
@@ -710,7 +713,7 @@ QXmppTask<QVector<QXmppOmemoDevice>> Manager::devices(const QList<QString> &jids
     QXmppPromise<QVector<QXmppOmemoDevice>> interface;
 
     auto future = keys(jids);
-    future.then(this, [=](QHash<QString, QHash<QByteArray, TrustLevel>> keys) mutable {
+    future.then(this, [=, this](QHash<QString, QHash<QByteArray, TrustLevel>> keys) mutable {
         QVector<QXmppOmemoDevice> devices;
 
         for (const auto &jid : jids) {
@@ -759,17 +762,17 @@ QXmppTask<QXmppPubSubManager::Result> Manager::removeContactDevices(const QStrin
     Q_ASSERT_X(jid != d->ownBareJid(), "Removing contact device", "Own JID passed");
 
     auto future = d->unsubscribeFromDeviceList(jid);
-    future.then(this, [=](QXmppPubSubManager::Result result) mutable {
+    future.then(this, [=, this](QXmppPubSubManager::Result result) mutable {
         if (std::holds_alternative<QXmppError>(result)) {
-            warning("Contact '" % jid % "' could not be removed because the device list subscription could not be removed");
+            warning(u"Contact '" + jid + u"' could not be removed because the device list subscription could not be removed");
             interface.finish(std::move(result));
         } else {
             d->devices.remove(jid);
 
             auto future = d->omemoStorage->removeDevices(jid);
-            future.then(this, [=]() mutable {
-                auto future = d->trustManager->removeKeys(ns_omemo_2, jid);
-                future.then(this, [=]() mutable {
+            future.then(this, [=, this]() mutable {
+                auto future = d->trustManager->removeKeys(ns_omemo_2.toString(), jid);
+                future.then(this, [=, this]() mutable {
                     interface.finish(std::move(result));
                     Q_EMIT devicesRemoved(jid);
                 });
@@ -863,9 +866,9 @@ QXmppTask<void> Manager::buildMissingSessions(const QList<QString> &jids)
     for (const auto &jid : jids) {
         // Do not exceed the maximum of manageable devices.
         if (devicesCount > d->maximumDevicesPerStanza - devicesCount) {
-            warning("Sessions could not be built for all JIDs because their devices are "
-                    "altogether more than the maximum of manageable devices " %
-                    QString::number(d->maximumDevicesPerStanza) %
+            warning(u"Sessions could not be built for all JIDs because their devices are "
+                    "altogether more than the maximum of manageable devices " +
+                    QString::number(d->maximumDevicesPerStanza) +
                     u" - Use QXmppOmemoManager::setMaximumDevicesPerStanza() to increase the maximum");
             break;
         } else {
@@ -902,6 +905,22 @@ QXmppTask<void> Manager::buildMissingSessions(const QList<QString> &jids)
     return interface.task();
 }
 
+///
+/// Resets all local OMEMO data for this device and the trust data used by OMEMO.
+///
+/// ATTENTION: This should only be called after an account is removed from the server since the data
+/// on the server is not reset.
+///
+/// Call \c setUp() once this method is finished and you are logged in to a new account if you want
+/// to set up everything for it.
+///
+/// \since QXmpp 1.9
+///
+QXmppTask<void> QXmppOmemoManager::resetOwnDeviceLocally()
+{
+    return d->resetOwnDeviceLocally();
+}
+
 ///
 /// Resets all OMEMO data for this device and the trust data used by OMEMO.
 ///
@@ -918,6 +937,8 @@ QXmppTask<void> Manager::buildMissingSessions(const QList<QString> &jids)
 /// Existing sessions are reset, which might lead to undecryptable incoming
 /// stanzas until everything is set up again.
 ///
+/// \return whether the action was successful
+///
 QXmppTask<bool> Manager::resetOwnDevice()
 {
     return d->resetOwnDevice();
@@ -940,6 +961,8 @@ QXmppTask<bool> Manager::resetOwnDevice()
 /// Existing sessions are reset, which might lead to undecryptable incoming
 /// stanzas until everything is set up again.
 ///
+/// \return whether the action was successful
+///
 QXmppTask<bool> Manager::resetAll()
 {
     return d->resetAll();
@@ -952,7 +975,7 @@ QXmppTask<bool> Manager::resetAll()
 ///
 QXmppTask<void> Manager::setSecurityPolicy(QXmpp::TrustSecurityPolicy securityPolicy)
 {
-    return d->trustManager->setSecurityPolicy(ns_omemo_2, securityPolicy);
+    return d->trustManager->setSecurityPolicy(ns_omemo_2.toString(), securityPolicy);
 }
 
 ///
@@ -962,7 +985,7 @@ QXmppTask<void> Manager::setSecurityPolicy(QXmpp::TrustSecurityPolicy securityPo
 ///
 QXmppTask<QXmpp::TrustSecurityPolicy> Manager::securityPolicy()
 {
-    return d->trustManager->securityPolicy(ns_omemo_2);
+    return d->trustManager->securityPolicy(ns_omemo_2.toString());
 }
 
 ///
@@ -975,7 +998,7 @@ QXmppTask<QXmpp::TrustSecurityPolicy> Manager::securityPolicy()
 ///
 QXmppTask<void> Manager::setTrustLevel(const QMultiHash<QString, QByteArray> &keyIds, QXmpp::TrustLevel trustLevel)
 {
-    return d->trustManager->setTrustLevel(ns_omemo_2, keyIds, trustLevel);
+    return d->trustManager->setTrustLevel(ns_omemo_2.toString(), keyIds, trustLevel);
 }
 
 ///
@@ -990,7 +1013,7 @@ QXmppTask<void> Manager::setTrustLevel(const QMultiHash<QString, QByteArray> &ke
 ///
 QXmppTask<QXmpp::TrustLevel> Manager::trustLevel(const QString &keyOwnerJid, const QByteArray &keyId)
 {
-    return d->trustManager->trustLevel(ns_omemo_2, keyOwnerJid, keyId);
+    return d->trustManager->trustLevel(ns_omemo_2.toString(), keyOwnerJid, keyId);
 }
 
 /// \cond
@@ -1019,12 +1042,11 @@ QXmppTask<QXmppE2eeExtension::MessageDecryptResult> QXmppOmemoManager::decryptMe
 {
     if (!d->isStarted) {
         return makeReadyTask<MessageDecryptResult>(QXmppError {
-            QStringLiteral("OMEMO manager must be started before decrypting"),
+            u"OMEMO manager must be started before decrypting"_s,
             SendError::EncryptionError });
     }
 
-    auto omemoElement = message.omemoElement();
-    if (!omemoElement) {
+    if (!message.omemoElement()) {
         return makeReadyTask<MessageDecryptResult>(NotEncrypted());
     }
 
@@ -1033,7 +1055,7 @@ QXmppTask<QXmppE2eeExtension::MessageDecryptResult> QXmppOmemoManager::decryptMe
             return std::move(*message);
         }
         return QXmppError {
-            QStringLiteral("Couldn't decrypt message"),
+            u"Couldn't decrypt message"_s,
             {}
         };
     });
@@ -1045,7 +1067,7 @@ QXmppTask<QXmppE2eeExtension::IqEncryptResult> Manager::encryptIq(QXmppIq &&iq,
 
     if (!d->isStarted) {
         interface.finish(QXmppError {
-            QStringLiteral("OMEMO manager must be started before encrypting"),
+            u"OMEMO manager must be started before encrypting"_s,
             SendError::EncryptionError });
     } else {
         std::optional<TrustLevels> acceptedTrustLevels;
@@ -1062,7 +1084,7 @@ QXmppTask<QXmppE2eeExtension::IqEncryptResult> Manager::encryptIq(QXmppIq &&iq,
         future.then(this, [=, iq = std::move(iq)](std::optional<QXmppOmemoElement> omemoElement) mutable {
             if (!omemoElement) {
                 interface.finish(QXmppError {
-                    QStringLiteral("OMEMO element could not be created"),
+                    u"OMEMO element could not be created"_s,
                     SendError::EncryptionError });
 
             } else {
@@ -1087,7 +1109,7 @@ QXmppTask<QXmppE2eeExtension::IqDecryptResult> Manager::decryptIq(const QDomElem
     if (!d->isStarted) {
         // TODO: Add decryption queue to avoid this error
         return makeReadyTask<IqDecryptResult>(QXmppError {
-            QStringLiteral("OMEMO manager must be started before decrypting"),
+            u"OMEMO manager must be started before decrypting"_s,
             SendError::EncryptionError });
     }
 
@@ -1098,7 +1120,7 @@ QXmppTask<QXmppE2eeExtension::IqDecryptResult> Manager::decryptIq(const QDomElem
                 return result->iq;
             }
             return QXmppError {
-                QStringLiteral("OMEMO message could not be decrypted"),
+                u"OMEMO message could not be decrypted"_s,
                 SendError::EncryptionError
             };
         });
@@ -1109,14 +1131,7 @@ QXmppTask<QXmppE2eeExtension::IqDecryptResult> Manager::decryptIq(const QDomElem
 
 bool QXmppOmemoManager::isEncrypted(const QDomElement &el)
 {
-    for (auto subEl = el.firstChildElement();
-         !subEl.isNull();
-         subEl = subEl.nextSiblingElement()) {
-        if (subEl.tagName() == "encrypted" && subEl.namespaceURI() == ns_omemo_2) {
-            return true;
-        }
-    }
-    return false;
+    return !firstChildElement(el, u"encrypted", ns_omemo_2).isNull();
 }
 
 bool QXmppOmemoManager::isEncrypted(const QXmppMessage &message)
@@ -1127,33 +1142,33 @@ bool QXmppOmemoManager::isEncrypted(const QXmppMessage &message)
 QStringList Manager::discoveryFeatures() const
 {
     return {
-        QString(ns_omemo_2_devices) % "+notify"
+        ns_omemo_2_devices + u"+notify"
     };
 }
 
 bool Manager::handleStanza(const QDomElement &stanza)
 {
-    if (stanza.tagName() != "iq" || !QXmppOmemoIq::isOmemoIq(stanza)) {
+    if (stanza.tagName() != u"iq" || !QXmppOmemoIq::isOmemoIq(stanza)) {
         return false;
     }
 
     // TODO: Queue incoming IQs until OMEMO is initialized
     if (!d->isStarted) {
-        warning("Couldn't decrypt incoming IQ because the manager isn't initialized yet.");
+        warning(u"Couldn't decrypt incoming IQ because the manager isn't initialized yet."_s);
         return false;
     }
 
-    auto type = stanza.attribute("type");
-    if (type != "get" && type != "set") {
+    auto type = stanza.attribute(u"type"_s);
+    if (type != u"get" && type != u"set") {
         // ignore incoming result and error IQs (they are handled via Client::sendIq())
         return false;
     }
 
-    d->decryptIq(stanza).then(this, [=](auto result) {
+    d->decryptIq(stanza).then(this, [=, this](auto result) {
         if (result) {
             injectIq(result->iq, result->e2eeMetadata);
         } else {
-            warning("Could not decrypt incoming OMEMO IQ.");
+            warning(u"Could not decrypt incoming OMEMO IQ."_s);
         }
     });
     return true;
@@ -1163,9 +1178,11 @@ bool Manager::handleMessage(const QXmppMessage &message)
 {
     if (d->isStarted && message.omemoElement()) {
         auto future = d->decryptMessage(message);
-        future.then(this, [=](std::optional<QXmppMessage> optionalDecryptedMessage) mutable {
+        future.then(this, [this, message](std::optional<QXmppMessage> optionalDecryptedMessage) {
             if (optionalDecryptedMessage) {
                 injectMessage(std::move(*optionalDecryptedMessage));
+            } else {
+                Q_EMIT client() -> messageReceived(message);
             }
         });
 
@@ -1226,11 +1243,8 @@ bool Manager::handleMessage(const QXmppMessage &message)
 ///
 
 /// \cond
-void Manager::setClient(QXmppClient *client)
+void Manager::onRegistered(QXmppClient *client)
 {
-    QXmppClientExtension::setClient(client);
-    client->setEncryptionExtension(this);
-
     d->trustManager = client->findExtension<QXmppTrustManager>();
     if (!d->trustManager) {
         qFatal("QXmppTrustManager is not available, it must be added to the client before adding QXmppOmemoManager");
@@ -1241,8 +1255,8 @@ void Manager::setClient(QXmppClient *client)
         qFatal("QXmppPubSubManager is not available, it must be added to the client before adding QXmppOmemoManager");
     }
 
-    connect(d->trustManager, &QXmppTrustManager::trustLevelsChanged, this, [=](const QHash<QString, QMultiHash<QString, QByteArray>> &modifiedKeys) {
-        const auto &modifiedOmemoKeys = modifiedKeys.value(ns_omemo_2);
+    connect(d->trustManager, &QXmppTrustManager::trustLevelsChanged, this, [=, this](const QHash<QString, QMultiHash<QString, QByteArray>> &modifiedKeys) {
+        const auto &modifiedOmemoKeys = modifiedKeys.value(ns_omemo_2.toString());
 
         if (!modifiedOmemoKeys.isEmpty()) {
             Q_EMIT trustLevelsChanged(modifiedOmemoKeys);
@@ -1270,6 +1284,12 @@ void Manager::setClient(QXmppClient *client)
     });
 }
 
+void Manager::onUnregistered(QXmppClient *client)
+{
+    // TODO: Proper clean up of connections (currently no issue because extensions are deleted
+    // on removal)
+}
+
 bool Manager::handlePubSubEvent(const QDomElement &element, const QString &pubSubService, const QString &nodeName)
 {
     if (nodeName == ns_omemo_2_devices && QXmppPubSubEvent<QXmppOmemoDeviceListItem>::isPubSubEvent(element)) {
