@@ -7,21 +7,28 @@
 #include "QXmppConstants_p.h"
 #include "QXmppDialback.h"
 #include "QXmppOutgoingServer.h"
-#include "QXmppStartTlsPacket.h"
 #include "QXmppStreamFeatures.h"
 #include "QXmppUtils.h"
+#include "QXmppUtils_p.h"
+
+#include "Stream.h"
+#include "StringLiterals.h"
+#include "XmppSocket.h"
 
 #include <QDomElement>
 #include <QHostAddress>
 #include <QSslKey>
 #include <QSslSocket>
 
+using namespace QXmpp::Private;
+
 class QXmppIncomingServerPrivate
 {
 public:
     QXmppIncomingServerPrivate(QXmppIncomingServer *qq);
     QString origin() const;
 
+    XmppSocket socket;
     QSet<QString> authenticated;
     QString domain;
     QString localStreamId;
@@ -31,72 +38,100 @@ private:
 };
 
 QXmppIncomingServerPrivate::QXmppIncomingServerPrivate(QXmppIncomingServer *qq)
-    : q(qq)
+    : q(qq),
+      socket(qq)
 {
 }
 
 QString QXmppIncomingServerPrivate::origin() const
 {
-    QSslSocket *socket = q->socket();
-    if (socket) {
-        return socket->peerAddress().toString() + " " + QString::number(socket->peerPort());
+    auto *tcpSocket = socket.socket();
+    if (tcpSocket) {
+        return tcpSocket->peerAddress().toString() + u' ' + QString::number(tcpSocket->peerPort());
     } else {
-        return "<unknown>";
+        return u"<unknown>"_s;
     }
 }
 
+///
 /// Constructs a new incoming server stream.
 ///
 /// \param socket The socket for the XMPP stream.
 /// \param domain The local domain.
 /// \param parent The parent QObject for the stream (optional).
 ///
-
 QXmppIncomingServer::QXmppIncomingServer(QSslSocket *socket, const QString &domain, QObject *parent)
-    : QXmppStream(parent)
+    : QXmppLoggable(parent),
+      d(std::make_unique<QXmppIncomingServerPrivate>(this))
 {
+    connect(&d->socket, &XmppSocket::started, this, &QXmppIncomingServer::handleStart);
+    connect(&d->socket, &XmppSocket::stanzaReceived, this, &QXmppIncomingServer::handleStanza);
+    connect(&d->socket, &XmppSocket::streamReceived, this, &QXmppIncomingServer::handleStream);
+    connect(&d->socket, &XmppSocket::streamClosed, this, &QXmppIncomingServer::disconnectFromHost);
 
-    d = new QXmppIncomingServerPrivate(this);
     d->domain = domain;
 
     if (socket) {
         connect(socket, &QAbstractSocket::disconnected,
                 this, &QXmppIncomingServer::slotSocketDisconnected);
 
-        setSocket(socket);
+        d->socket.setSocket(socket);
     }
 
-    info(QString("Incoming server connection from %1").arg(d->origin()));
+    info(u"Incoming server connection from %1"_s.arg(d->origin()));
 }
 
-/// Destroys the current stream.
+QXmppIncomingServer::~QXmppIncomingServer() = default;
 
-QXmppIncomingServer::~QXmppIncomingServer()
+/// Returns true if the socket is connected and the remote server is
+/// authenticated.
+bool QXmppIncomingServer::isConnected() const
 {
-    delete d;
+    return d->socket.isConnected() && !d->authenticated.isEmpty();
+}
+
+/// Disconnects from the remote host.
+void QXmppIncomingServer::disconnectFromHost()
+{
+    d->socket.disconnectFromHost();
 }
 
 /// Returns the stream's identifier.
-///
-
 QString QXmppIncomingServer::localStreamId() const
 {
     return d->localStreamId;
 }
 
-/// \cond
+/// Sends an XMPP packet to the peer.
+bool QXmppIncomingServer::sendPacket(const QXmppNonza &nonza)
+{
+    return d->socket.sendData(serializeXml(nonza));
+}
+
+/// Sends raw data to the peer.
+bool QXmppIncomingServer::sendData(const QByteArray &data)
+{
+    return d->socket.sendData(data);
+}
+
+/// Handles a stream start event, which occurs when the underlying transport
+/// becomes ready (socket connected, encryption started).
+void QXmppIncomingServer::handleStart()
+{
+}
+
 void QXmppIncomingServer::handleStream(const QDomElement &streamElement)
 {
-    const QString from = streamElement.attribute("from");
+    const QString from = streamElement.attribute(u"from"_s);
     if (!from.isEmpty()) {
-        info(QString("Incoming server stream from %1 on %2").arg(from, d->origin()));
+        info(u"Incoming server stream from %1 on %2"_s.arg(from, d->origin()));
     }
 
     // start stream
-    d->localStreamId = QXmppUtils::generateStanzaHash().toLatin1();
-    QString data = QString("<?xml version='1.0'?><stream:stream"
-                           " xmlns='%1' xmlns:db='%2' xmlns:stream='%3'"
-                           " id='%4' version=\"1.0\">")
+    d->localStreamId = QXmppUtils::generateStanzaHash();
+    QString data = u"<?xml version='1.0'?><stream:stream"
+                   " xmlns='%1' xmlns:db='%2' xmlns:stream='%3'"
+                   " id='%4' version=\"1.0\">"_s
                        .arg(
                            ns_server,
                            ns_server_dialback,
@@ -106,7 +141,8 @@ void QXmppIncomingServer::handleStream(const QDomElement &streamElement)
 
     // send stream features
     QXmppStreamFeatures features;
-    if (!socket()->isEncrypted() && !socket()->localCertificate().isNull() && !socket()->privateKey().isNull()) {
+    auto *socket = d->socket.socket();
+    if (!socket->isEncrypted() && !socket->localCertificate().isNull() && !socket->privateKey().isNull()) {
         features.setTlsMode(QXmppStreamFeatures::Enabled);
     }
     sendPacket(features);
@@ -114,10 +150,10 @@ void QXmppIncomingServer::handleStream(const QDomElement &streamElement)
 
 void QXmppIncomingServer::handleStanza(const QDomElement &stanza)
 {
-    if (QXmppStartTlsPacket::isStartTlsPacket(stanza, QXmppStartTlsPacket::StartTls)) {
-        sendPacket(QXmppStartTlsPacket(QXmppStartTlsPacket::Proceed));
-        socket()->flush();
-        socket()->startServerEncryption();
+    if (StarttlsRequest::fromDom(stanza)) {
+        sendData(serializeXml(StarttlsProceed()));
+        d->socket.socket()->flush();
+        d->socket.socket()->startServerEncryption();
         return;
     } else if (QXmppDialback::isDialback(stanza)) {
         QXmppDialback request;
@@ -127,13 +163,13 @@ void QXmppIncomingServer::handleStanza(const QDomElement &stanza)
             request.from().isEmpty() ||
             request.to() != d->domain ||
             request.key().isEmpty()) {
-            warning(QString("Invalid dialback received on %1").arg(d->origin()));
+            warning(u"Invalid dialback received on %1"_s.arg(d->origin()));
             return;
         }
 
         const QString domain = request.from();
         if (request.command() == QXmppDialback::Result) {
-            debug(QString("Received a dialback result from '%1' on %2").arg(domain, d->origin()));
+            debug(u"Received a dialback result from '%1' on %2"_s.arg(domain, d->origin()));
 
             // establish dialback connection
             auto *stream = new QXmppOutgoingServer(d->domain, this);
@@ -142,34 +178,20 @@ void QXmppIncomingServer::handleStanza(const QDomElement &stanza)
             stream->setVerify(d->localStreamId, request.key());
             stream->connectToHost(domain);
         } else if (request.command() == QXmppDialback::Verify) {
-            debug(QString("Received a dialback verify from '%1' on %2").arg(domain, d->origin()));
+            debug(u"Received a dialback verify from '%1' on %2"_s.arg(domain, d->origin()));
             Q_EMIT dialbackRequestReceived(request);
         }
 
-    } else if (d->authenticated.contains(QXmppUtils::jidToDomain(stanza.attribute("from")))) {
+    } else if (d->authenticated.contains(QXmppUtils::jidToDomain(stanza.attribute(u"from"_s)))) {
         // relay stanza if the remote party is authenticated
         Q_EMIT elementReceived(stanza);
     } else {
-        warning(QString("Received an element from unverified domain '%1' on %2").arg(QXmppUtils::jidToDomain(stanza.attribute("from")), d->origin()));
+        warning(u"Received an element from unverified domain '%1' on %2"_s.arg(QXmppUtils::jidToDomain(stanza.attribute(u"from"_s)), d->origin()));
         disconnectFromHost();
     }
 }
-/// \endcond
-
-/// Returns true if the socket is connected and the remote server is
-/// authenticated.
-///
-
-bool QXmppIncomingServer::isConnected() const
-{
-    return QXmppStream::isConnected() && !d->authenticated.isEmpty();
-}
 
 /// Handles a dialback response received from the authority server.
-///
-/// \param response
-///
-
 void QXmppIncomingServer::slotDialbackResponseReceived(const QXmppDialback &dialback)
 {
     auto *stream = qobject_cast<QXmppOutgoingServer *>(sender());
@@ -189,15 +211,15 @@ void QXmppIncomingServer::slotDialbackResponseReceived(const QXmppDialback &dial
     sendPacket(response);
 
     // check for success
-    if (response.type() == QLatin1String("valid")) {
-        info(QString("Verified incoming domain '%1' on %2").arg(dialback.from(), d->origin()));
+    if (response.type() == u"valid") {
+        info(u"Verified incoming domain '%1' on %2"_s.arg(dialback.from(), d->origin()));
         const bool wasConnected = !d->authenticated.isEmpty();
         d->authenticated.insert(dialback.from());
         if (!wasConnected) {
             Q_EMIT connected();
         }
     } else {
-        warning(QString("Failed to verify incoming domain '%1' on %2").arg(dialback.from(), d->origin()));
+        warning(u"Failed to verify incoming domain '%1' on %2"_s.arg(dialback.from(), d->origin()));
         disconnectFromHost();
     }
 
@@ -208,6 +230,6 @@ void QXmppIncomingServer::slotDialbackResponseReceived(const QXmppDialback &dial
 
 void QXmppIncomingServer::slotSocketDisconnected()
 {
-    info(QString("Socket disconnected from %1").arg(d->origin()));
+    info(u"Socket disconnected from %1"_s.arg(d->origin()));
     Q_EMIT disconnected();
 }
