@@ -10,11 +10,12 @@
 #include "QXmppIncomingServer.h"
 #include "QXmppIq.h"
 #include "QXmppOutgoingServer.h"
-#include "QXmppPresence.h"
 #include "QXmppServerExtension.h"
 #include "QXmppServerPlugin.h"
 #include "QXmppUtils.h"
 
+#include "StringLiterals.h"
+
 #include <QCoreApplication>
 #include <QDomElement>
 #include <QFileInfo>
@@ -24,7 +25,7 @@
 #include <QSslKey>
 #include <QSslSocket>
 
-static void helperToXmlAddDomElement(QXmlStreamWriter *stream, const QDomElement &element, const QStringList &omitNamespaces)
+static void helperToXmlAddDomElement(QXmlStreamWriter *stream, const QDomElement &element, const QVector<QStringView> &omitNamespaces)
 {
     stream->writeStartElement(element.tagName());
 
@@ -43,7 +44,7 @@ static void helperToXmlAddDomElement(QXmlStreamWriter *stream, const QDomElement
     QDomNode childNode = element.firstChild();
     while (!childNode.isNull()) {
         if (childNode.isElement()) {
-            helperToXmlAddDomElement(stream, childNode.toElement(), QStringList() << xmlns);
+            helperToXmlAddDomElement(stream, childNode.toElement(), { xmlns });
         } else if (childNode.isText()) {
             stream->writeCharacters(childNode.toText().data());
         }
@@ -110,7 +111,7 @@ bool QXmppServerPrivate::routeData(const QString &to, const QByteArray &data)
 {
     // refuse to route packets to empty destination, own domain or sub-domains
     const QString toDomain = QXmppUtils::jidToDomain(to);
-    if (to.isEmpty() || to == domain || toDomain.endsWith("." + domain)) {
+    if (to.isEmpty() || to == domain || toDomain.endsWith(QString(u"." + domain))) {
         return false;
     }
 
@@ -149,16 +150,16 @@ bool QXmppServerPrivate::routeData(const QString &to, const QByteArray &data)
         // if we did not find an outgoing server,
         // we need to establish the S2S connection
         auto *conn = new QXmppOutgoingServer(domain, nullptr);
-        conn->setLocalStreamKey(QXmppUtils::generateStanzaHash().toLatin1());
+        conn->setLocalStreamKey(QXmppUtils::generateStanzaHash());
         conn->moveToThread(q->thread());
         conn->setParent(q);
 
-        QObject::connect(conn, &QXmppStream::disconnected,
+        QObject::connect(conn, &QXmppOutgoingServer::disconnected,
                          q, &QXmppServer::_q_outgoingServerDisconnected);
 
         // add stream
         outgoingServers.insert(conn);
-        Q_EMIT q->setGauge("outgoing-server.count", outgoingServers.size());
+        Q_EMIT q->setGauge(u"outgoing-server.count"_s, outgoingServers.size());
 
         // queue data and connect to remote server
         QMetaObject::invokeMethod(conn, "queueData", Q_ARG(QByteArray, data));
@@ -173,11 +174,6 @@ bool QXmppServerPrivate::routeData(const QString &to, const QByteArray &data)
 }
 
 /// Handles an incoming XML element.
-///
-/// \param server
-/// \param stream
-/// \param element
-
 static void handleStanza(QXmppServer *server, const QDomElement &element)
 {
     // try extensions
@@ -190,9 +186,9 @@ static void handleStanza(QXmppServer *server, const QDomElement &element)
 
     // default handlers
     const QString domain = server->domain();
-    const QString to = element.attribute("to");
+    const QString to = element.attribute(u"to"_s);
     if (to == domain) {
-        if (element.tagName() == QLatin1String("iq")) {
+        if (element.tagName() == u"iq") {
             // we do not support the given IQ
             QXmppIq request;
             request.parse(element);
@@ -212,7 +208,7 @@ static void handleStanza(QXmppServer *server, const QDomElement &element)
     } else {
 
         // route element or reply on behalf of missing peer
-        if (!server->sendElement(element) && element.tagName() == QLatin1String("iq")) {
+        if (!server->sendElement(element) && element.tagName() == u"iq") {
             QXmppIq request;
             request.parse(element);
 
@@ -243,9 +239,6 @@ void QXmppServerPrivate::warning(const QString &message)
 }
 
 /// Load the server's extensions.
-///
-/// \param server
-
 void QXmppServerPrivate::loadExtensions(QXmppServer *server)
 {
     if (!loaded) {
@@ -266,13 +259,12 @@ void QXmppServerPrivate::loadExtensions(QXmppServer *server)
 }
 
 /// Start the server's extensions.
-
 void QXmppServerPrivate::startExtensions()
 {
     if (!started) {
         for (auto *extension : std::as_const(extensions)) {
             if (!extension->start()) {
-                warning(QString("Could not start extension %1").arg(extension->extensionName()));
+                warning(u"Could not start extension %1"_s.arg(extension->extensionName()));
             }
         }
         started = true;
@@ -280,8 +272,6 @@ void QXmppServerPrivate::startExtensions()
 }
 
 /// Stop the server's extensions (in reverse order).
-///
-
 void QXmppServerPrivate::stopExtensions()
 {
     if (started) {
@@ -293,34 +283,25 @@ void QXmppServerPrivate::stopExtensions()
 }
 
 /// Constructs a new XMPP server instance.
-///
-/// \param parent
-
 QXmppServer::QXmppServer(QObject *parent)
-    : QXmppLoggable(parent), d(new QXmppServerPrivate(this))
+    : QXmppLoggable(parent),
+      d(std::make_unique<QXmppServerPrivate>(this))
 {
     qRegisterMetaType<QDomElement>("QDomElement");
 }
 
-/// Destroys an XMPP server instance.
-///
-
 QXmppServer::~QXmppServer()
 {
     close();
-    delete d;
 }
 
 /// Registers a new extension with the server.
-///
-/// \param extension
-
 void QXmppServer::addExtension(QXmppServerExtension *extension)
 {
     if (!extension || d->extensions.contains(extension)) {
         return;
     }
-    d->info(QString("Added extension %1").arg(extension->extensionName()));
+    d->info(u"Added extension %1"_s.arg(extension->extensionName()));
     extension->setParent(this);
     extension->setServer(this);
 
@@ -336,8 +317,6 @@ void QXmppServer::addExtension(QXmppServerExtension *extension)
 }
 
 /// Returns the list of loaded extensions.
-///
-
 QList<QXmppServerExtension *> QXmppServer::extensions()
 {
     d->loadExtensions(this);
@@ -345,17 +324,12 @@ QList<QXmppServerExtension *> QXmppServer::extensions()
 }
 
 /// Returns the server's domain.
-///
-
 QString QXmppServer::domain() const
 {
     return d->domain;
 }
 
 /// Sets the server's domain.
-///
-/// \param domain
-
 void QXmppServer::setDomain(const QString &domain)
 {
     d->domain = domain;
@@ -367,9 +341,6 @@ QXmppLogger *QXmppServer::logger()
 }
 
 /// Sets the QXmppLogger associated with the server.
-///
-/// \param logger
-
 void QXmppServer::setLogger(QXmppLogger *logger)
 {
     if (logger != d->logger) {
@@ -397,39 +368,29 @@ void QXmppServer::setLogger(QXmppLogger *logger)
 }
 
 /// Returns the password checker used to verify client credentials.
-///
-
 QXmppPasswordChecker *QXmppServer::passwordChecker()
 {
     return d->passwordChecker;
 }
 
 /// Sets the password checker used to verify client credentials.
-///
-/// \param checker
-///
-
 void QXmppServer::setPasswordChecker(QXmppPasswordChecker *checker)
 {
     d->passwordChecker = checker;
 }
 
 /// Returns the statistics for the server.
-
 QVariantMap QXmppServer::statistics() const
 {
     QVariantMap stats;
-    stats["version"] = qApp->applicationVersion();
-    stats["incoming-clients"] = d->incomingClients.size();
-    stats["incoming-servers"] = d->incomingServers.size();
-    stats["outgoing-servers"] = d->outgoingServers.size();
+    stats[u"version"_s] = qApp->applicationVersion();
+    stats[u"incoming-clients"_s] = d->incomingClients.size();
+    stats[u"incoming-servers"_s] = d->incomingServers.size();
+    stats[u"outgoing-servers"_s] = d->outgoingServers.size();
     return stats;
 }
 
 /// Sets the path for additional SSL CA certificates.
-///
-/// \param path
-
 void QXmppServer::addCaCertificates(const QString &path)
 {
     // load certificates
@@ -438,7 +399,7 @@ void QXmppServer::addCaCertificates(const QString &path)
     } else if (QFileInfo(path).isReadable()) {
         d->caCertificates = QSslCertificate::fromPath(path);
     } else {
-        d->warning(QString("SSL CA certificates are not readable %1").arg(path));
+        d->warning(u"SSL CA certificates are not readable %1"_s.arg(path));
         d->caCertificates = QList<QSslCertificate>();
     }
 
@@ -452,9 +413,6 @@ void QXmppServer::addCaCertificates(const QString &path)
 }
 
 /// Sets the path for the local SSL certificate.
-///
-/// \param path
-
 void QXmppServer::setLocalCertificate(const QString &path)
 {
     // load certificate
@@ -465,7 +423,7 @@ void QXmppServer::setLocalCertificate(const QString &path)
     } else if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
         d->localCertificate = QSslCertificate(file.readAll());
     } else {
-        d->warning(QString("SSL certificate is not readable %1").arg(path));
+        d->warning(u"SSL certificate is not readable %1"_s.arg(path));
         d->localCertificate = QSslCertificate();
     }
 
@@ -499,9 +457,6 @@ void QXmppServer::setLocalCertificate(const QSslCertificate &certificate)
 }
 
 /// Sets the path for the local SSL private key.
-///
-/// \param path
-
 void QXmppServer::setPrivateKey(const QString &path)
 {
     // load key
@@ -512,7 +467,7 @@ void QXmppServer::setPrivateKey(const QString &path)
     } else if (file.open(QIODevice::ReadOnly)) {
         d->privateKey = QSslKey(file.readAll(), QSsl::Rsa);
     } else {
-        d->warning(QString("SSL key is not readable %1").arg(path));
+        d->warning(u"SSL key is not readable %1"_s.arg(path));
         d->privateKey = QSslKey();
     }
 
@@ -546,17 +501,13 @@ void QXmppServer::setPrivateKey(const QSslKey &key)
 }
 
 /// Listen for incoming XMPP client connections.
-///
-/// \param address
-/// \param port
-
 bool QXmppServer::listenForClients(const QHostAddress &address, quint16 port)
 {
     bool check;
     Q_UNUSED(check)
 
     if (d->domain.isEmpty()) {
-        d->warning("No domain was specified!");
+        d->warning(u"No domain was specified!"_s);
         return false;
     }
 
@@ -571,7 +522,7 @@ bool QXmppServer::listenForClients(const QHostAddress &address, quint16 port)
     Q_ASSERT(check);
 
     if (!server->listen(address, port)) {
-        d->warning(QString("Could not start listening for C2S on %1 %2").arg(address.toString(), QString::number(port)));
+        d->warning(u"Could not start listening for C2S on %1 %2"_s.arg(address.toString(), QString::number(port)));
         delete server;
         return false;
     }
@@ -584,8 +535,6 @@ bool QXmppServer::listenForClients(const QHostAddress &address, quint16 port)
 }
 
 /// Closes the server.
-///
-
 void QXmppServer::close()
 {
     // prevent new connections
@@ -617,17 +566,13 @@ void QXmppServer::close()
 }
 
 /// Listen for incoming XMPP server connections.
-///
-/// \param address
-/// \param port
-
 bool QXmppServer::listenForServers(const QHostAddress &address, quint16 port)
 {
     bool check;
     Q_UNUSED(check)
 
     if (d->domain.isEmpty()) {
-        d->warning("No domain was specified!");
+        d->warning(u"No domain was specified!"_s);
         return false;
     }
 
@@ -642,7 +587,7 @@ bool QXmppServer::listenForServers(const QHostAddress &address, quint16 port)
     Q_ASSERT(check);
 
     if (!server->listen(address, port)) {
-        d->warning(QString("Could not start listening for S2S on %1 %2").arg(address.toString(), QString::number(port)));
+        d->warning(u"Could not start listening for S2S on %1 %2"_s.arg(address.toString(), QString::number(port)));
         delete server;
         return false;
     }
@@ -655,25 +600,18 @@ bool QXmppServer::listenForServers(const QHostAddress &address, quint16 port)
 }
 
 /// Route an XMPP stanza.
-///
-/// \param element
-
 bool QXmppServer::sendElement(const QDomElement &element)
 {
     // serialize data
     QByteArray data;
     QXmlStreamWriter xmlStream(&data);
-    const QStringList omitNamespaces = QStringList() << ns_client << ns_server;
-    helperToXmlAddDomElement(&xmlStream, element, omitNamespaces);
+    helperToXmlAddDomElement(&xmlStream, element, { ns_client, ns_server });
 
     // route data
-    return d->routeData(element.attribute("to"), data);
+    return d->routeData(element.attribute(u"to"_s), data);
 }
 
 /// Route an XMPP packet.
-///
-/// \param packet
-
 bool QXmppServer::sendPacket(const QXmppStanza &packet)
 {
     // serialize data
@@ -685,34 +623,27 @@ bool QXmppServer::sendPacket(const QXmppStanza &packet)
     return d->routeData(packet.to(), data);
 }
 
+///
 /// Add a new incoming client \a stream.
 ///
 /// This method can be used for instance to implement BOSH support
 /// as a server extension.
-
+///
 void QXmppServer::addIncomingClient(QXmppIncomingClient *stream)
 {
 
     stream->setPasswordChecker(d->passwordChecker);
 
-    connect(stream, &QXmppStream::connected,
-            this, &QXmppServer::_q_clientConnected);
-
-    connect(stream, &QXmppStream::disconnected,
-            this, &QXmppServer::_q_clientDisconnected);
-
-    connect(stream, &QXmppIncomingClient::elementReceived,
-            this, &QXmppServer::handleElement);
+    connect(stream, &QXmppIncomingClient::connected, this, &QXmppServer::_q_clientConnected);
+    connect(stream, &QXmppIncomingClient::disconnected, this, &QXmppServer::_q_clientDisconnected);
+    connect(stream, &QXmppIncomingClient::elementReceived, this, &QXmppServer::handleElement);
 
     // add stream
     d->incomingClients.insert(stream);
-    Q_EMIT setGauge("incoming-client.count", d->incomingClients.size());
+    Q_EMIT setGauge(u"incoming-client.count"_s, d->incomingClients.size());
 }
 
 /// Handle a new incoming TCP connection from a client.
-///
-/// \param socket
-
 void QXmppServer::_q_clientConnection(QSslSocket *socket)
 {
     // check the socket didn't die since the signal was emitted
@@ -728,8 +659,6 @@ void QXmppServer::_q_clientConnection(QSslSocket *socket)
 }
 
 /// Handle a successful stream connection for a client.
-///
-
 void QXmppServer::_q_clientConnected()
 {
     auto *client = qobject_cast<QXmppIncomingClient *>(sender());
@@ -754,7 +683,6 @@ void QXmppServer::_q_clientConnected()
 }
 
 /// Handle a stream disconnection for a client.
-
 void QXmppServer::_q_clientDisconnected()
 {
     auto *client = qobject_cast<QXmppIncomingClient *>(sender());
@@ -787,7 +715,7 @@ void QXmppServer::_q_clientDisconnected()
         }
 
         // update counter
-        Q_EMIT setGauge("incoming-client.count", d->incomingClients.size());
+        Q_EMIT setGauge(u"incoming-client.count"_s, d->incomingClients.size());
     }
 }
 
@@ -811,7 +739,7 @@ void QXmppServer::_q_dialbackRequestReceived(const QXmppDialback &dialback)
             verify.setId(dialback.id());
             verify.setTo(dialback.from());
             verify.setFrom(d->domain);
-            verify.setType(isValid ? "valid" : "invalid");
+            verify.setType(isValid ? u"valid"_s : u"invalid"_s);
             stream->sendPacket(verify);
             return;
         }
@@ -819,14 +747,12 @@ void QXmppServer::_q_dialbackRequestReceived(const QXmppDialback &dialback)
 }
 
 /// Handle an incoming XML element.
-
 void QXmppServer::handleElement(const QDomElement &element)
 {
     handleStanza(this, element);
 }
 
 /// Handle a stream disconnection for an outgoing server.
-
 void QXmppServer::_q_outgoingServerDisconnected()
 {
     auto *outgoing = qobject_cast<QXmppOutgoingServer *>(sender());
@@ -836,14 +762,11 @@ void QXmppServer::_q_outgoingServerDisconnected()
 
     if (d->outgoingServers.remove(outgoing)) {
         outgoing->deleteLater();
-        Q_EMIT setGauge("outgoing-server.count", d->outgoingServers.size());
+        Q_EMIT setGauge(u"outgoing-server.count"_s, d->outgoingServers.size());
     }
 }
 
 /// Handle a new incoming TCP connection from a server.
-///
-/// \param socket
-
 void QXmppServer::_q_serverConnection(QSslSocket *socket)
 {
 
@@ -856,7 +779,7 @@ void QXmppServer::_q_serverConnection(QSslSocket *socket)
     auto *stream = new QXmppIncomingServer(socket, d->domain, this);
     socket->setParent(stream);
 
-    connect(stream, &QXmppStream::disconnected,
+    connect(stream, &QXmppIncomingServer::disconnected,
             this, &QXmppServer::_q_serverDisconnected);
 
     connect(stream, &QXmppIncomingServer::dialbackRequestReceived,
@@ -867,11 +790,10 @@ void QXmppServer::_q_serverConnection(QSslSocket *socket)
 
     // add stream
     d->incomingServers.insert(stream);
-    Q_EMIT setGauge("incoming-server.count", d->incomingServers.size());
+    Q_EMIT setGauge(u"incoming-server.count"_s, d->incomingServers.size());
 }
 
 /// Handle a stream disconnection for an incoming server.
-
 void QXmppServer::_q_serverDisconnected()
 {
     auto *incoming = qobject_cast<QXmppIncomingServer *>(sender());
@@ -881,7 +803,7 @@ void QXmppServer::_q_serverDisconnected()
 
     if (d->incomingServers.remove(incoming)) {
         incoming->deleteLater();
-        Q_EMIT setGauge("incoming-server.count", d->incomingServers.size());
+        Q_EMIT setGauge(u"incoming-server.count"_s, d->incomingServers.size());
     }
 }
 
@@ -894,22 +816,13 @@ public:
 };
 
 /// Constructs a new SSL server instance.
-///
-/// \param parent
-
 QXmppSslServer::QXmppSslServer(QObject *parent)
     : QTcpServer(parent),
-      d(new QXmppSslServerPrivate)
+      d(std::make_unique<QXmppSslServerPrivate>())
 {
 }
 
-/// Destroys an SSL server instance.
-///
-
-QXmppSslServer::~QXmppSslServer()
-{
-    delete d;
-}
+QXmppSslServer::~QXmppSslServer() = default;
 
 void QXmppSslServer::incomingConnection(qintptr socketDescriptor)
 {
@@ -931,29 +844,22 @@ void QXmppSslServer::incomingConnection(qintptr socketDescriptor)
     Q_EMIT newConnection(socket);
 }
 
+///
 /// Adds the given certificates to the CA certificate database to be used
 /// for incoming connections.
 ///
-/// \param certificates
-
 void QXmppSslServer::addCaCertificates(const QList<QSslCertificate> &certificates)
 {
     d->caCertificates += certificates;
 }
 
 /// Sets the local certificate to be used for incoming connections.
-///
-/// \param certificate
-
 void QXmppSslServer::setLocalCertificate(const QSslCertificate &certificate)
 {
     d->localCertificate = certificate;
 }
 
 /// Sets the local private key to be used for incoming connections.
-///
-/// \param key
-
 void QXmppSslServer::setPrivateKey(const QSslKey &key)
 {
     d->privateKey = key;
