@@ -1,26 +1,7 @@
-/*
- * Copyright (C) 2008-2021 The QXmpp developers
- *
- * Authors:
- *  Jeremy Lainé
- *  Manjeet Dahiya
- *
- * Source:
- *  https://github.com/qxmpp-project/qxmpp
- *
- * This file is a part of QXmpp library.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- */
+// SPDX-FileCopyrightText: 2012 Jeremy Lainé <jeremy.laine@m4x.org>
+// SPDX-FileCopyrightText: 2012 Manjeet Dahiya <manjeetdahiya@gmail.com>
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include "QXmppArchiveIq.h"
 
@@ -30,14 +11,14 @@ class tst_QXmppArchiveIq : public QObject
 {
     Q_OBJECT
 
-private slots:
-    void testArchiveList_data();
-    void testArchiveList();
-    void testArchiveChat_data();
-    void testArchiveChat();
-    void testArchiveRemove();
-    void testArchiveRetrieve_data();
-    void testArchiveRetrieve();
+private:
+    Q_SLOT void testArchiveList_data();
+    Q_SLOT void testArchiveList();
+    Q_SLOT void testArchiveChat_data();
+    Q_SLOT void testArchiveChat();
+    Q_SLOT void testArchiveRemove();
+    Q_SLOT void testArchiveRetrieve_data();
+    Q_SLOT void testArchiveRetrieve();
 };
 
 void tst_QXmppArchiveIq::testArchiveList_data()
@@ -72,8 +53,8 @@ void tst_QXmppArchiveIq::testArchiveList()
     QXmppArchiveListIq iq;
     parsePacket(iq, xml);
     QCOMPARE(iq.type(), QXmppIq::Get);
-    QCOMPARE(iq.id(), QLatin1String("list_1"));
-    QCOMPARE(iq.with(), QLatin1String("juliet@capulet.com"));
+    QCOMPARE(iq.id(), u"list_1");
+    QCOMPARE(iq.with(), u"juliet@capulet.com");
     QCOMPARE(iq.start(), QDateTime(QDate(1469, 7, 21), QTime(2, 0, 0), Qt::UTC));
     QCOMPARE(iq.end(), QDateTime(QDate(1479, 7, 21), QTime(4, 0, 0), Qt::UTC));
     QCOMPARE(iq.resultSetQuery().max(), max);
@@ -95,7 +76,7 @@ void tst_QXmppArchiveIq::testArchiveChat_data()
                                    ">"
                                    "<from secs=\"0\"><body>Art thou not Romeo, and a Montague?</body></from>"
                                    "<to secs=\"11\"><body>Neither, fair saint, if either thee dislike.</body></to>"
-                                   "<from secs=\"7\"><body>How cam'st thou hither, tell me, and wherefore?</body></from>"
+                                   "<from secs=\"7\"><body>How cam&apos;st thou hither, tell me, and wherefore?</body></from>"
                                    "</chat>"
                                    "</iq>")
                             << -1;
@@ -110,7 +91,7 @@ void tst_QXmppArchiveIq::testArchiveChat_data()
                                      ">"
                                      "<from secs=\"0\"><body>Art thou not Romeo, and a Montague?</body></from>"
                                      "<to secs=\"11\"><body>Neither, fair saint, if either thee dislike.</body></to>"
-                                     "<from secs=\"7\"><body>How cam'st thou hither, tell me, and wherefore?</body></from>"
+                                     "<from secs=\"7\"><body>How cam&apos;st thou hither, tell me, and wherefore?</body></from>"
                                      "<set xmlns=\"http://jabber.org/protocol/rsm\">"
                                      "<count>3</count>"
                                      "</set>"
