@@ -1,21 +1,25 @@
 // SPDX-FileCopyrightText: 2012 Jeremy Lainé <jeremy.laine@m4x.org>
 // SPDX-FileCopyrightText: 2012 Manjeet Dahiya <manjeetdahiya@gmail.com>
 // SPDX-FileCopyrightText: 2021 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2023 Tibor Csötönyi <work@taibsu.de>
 //
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include "QXmppBitsOfBinaryContentId.h"
 #include "QXmppBitsOfBinaryDataList.h"
 #include "QXmppEncryptedFileSource.h"
+#include "QXmppFallback.h"
+#include "QXmppJingleData.h"
 #include "QXmppMessage.h"
 #include "QXmppMessageReaction.h"
 #include "QXmppMixInvitation.h"
 #include "QXmppOutOfBandUrl.h"
 #include "QXmppTrustMessageElement.h"
 
+#include "util.h"
+
 #include <optional>
 
-#include "util.h"
 #include <QObject>
 
 class tst_QXmppMessage : public QObject
@@ -49,6 +53,7 @@ private:
     Q_SLOT void testProcessingHints();
     Q_SLOT void testBobData();
     Q_SLOT void testFallbackIndication();
+    Q_SLOT void fallbackBodies();
     Q_SLOT void testStanzaIds();
     Q_SLOT void testSlashMe_data();
     Q_SLOT void testSlashMe();
@@ -58,6 +63,8 @@ private:
     Q_SLOT void testE2eeFallbackBody();
     Q_SLOT void testFileSharing();
     Q_SLOT void testEncryptedFileSource();
+    Q_SLOT void testReplies();
+    Q_SLOT void testJingleMessageInitiationElement();
 };
 
 void tst_QXmppMessage::testBasic_data()
@@ -126,8 +133,8 @@ void tst_QXmppMessage::testBasic()
 
     QXmppMessage message;
     parsePacket(message, xml);
-    QCOMPARE(message.to(), QString("foo@example.com/QXmpp"));
-    QCOMPARE(message.from(), QString("bar@example.com/QXmpp"));
+    QCOMPARE(message.to(), u"foo@example.com/QXmpp"_s);
+    QCOMPARE(message.from(), u"bar@example.com/QXmpp"_s);
     QVERIFY(message.extendedAddresses().isEmpty());
     QCOMPARE(int(message.type()), type);
     QCOMPARE(message.body(), body);
@@ -146,14 +153,21 @@ void tst_QXmppMessage::testBasic()
     QVERIFY(!message.hasHint(QXmppMessage::NoCopy));
     QVERIFY(!message.hasHint(QXmppMessage::Store));
     QCOMPARE(message.bitsOfBinaryData(), QXmppBitsOfBinaryDataList());
-    QVERIFY(!message.isFallback());
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_DEPRECATED
     QVERIFY(message.stanzaId().isNull());
     QVERIFY(message.stanzaIdBy().isNull());
+    QT_WARNING_POP
+    QVERIFY(message.stanzaIds().isEmpty());
     QVERIFY(message.originId().isNull());
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_DEPRECATED
+    QVERIFY(!message.isFallback());
+    QT_WARNING_POP
 
     message = QXmppMessage();
-    message.setTo(QStringLiteral("foo@example.com/QXmpp"));
-    message.setFrom(QStringLiteral("bar@example.com/QXmpp"));
+    message.setTo(u"foo@example.com/QXmpp"_s);
+    message.setFrom(u"bar@example.com/QXmpp"_s);
     message.setType(QXmppMessage::Type(type));
     message.setBody(body);
     message.setSubject(subject);
@@ -190,8 +204,8 @@ void tst_QXmppMessage::testMessageAttention()
     QXmppMessage message;
     // parsing
     parsePacket(message, xml);
-    QCOMPARE(message.to(), QString("foo@example.com/QXmpp"));
-    QCOMPARE(message.from(), QString("bar@example.com/QXmpp"));
+    QCOMPARE(message.to(), u"foo@example.com/QXmpp"_s);
+    QCOMPARE(message.from(), u"bar@example.com/QXmpp"_s);
     QVERIFY(message.extendedAddresses().isEmpty());
     QCOMPARE(message.type(), QXmppMessage::Normal);
     QCOMPARE(message.body(), QString());
@@ -200,7 +214,7 @@ void tst_QXmppMessage::testMessageAttention()
     QCOMPARE(message.receiptId(), QString());
 
     // serialization
-    message = QXmppMessage(QStringLiteral("bar@example.com/QXmpp"), QStringLiteral("foo@example.com/QXmpp"));
+    message = QXmppMessage(u"bar@example.com/QXmpp"_s, u"foo@example.com/QXmpp"_s);
     message.setType(QXmppMessage::Normal);
     message.setAttentionRequested(true);
     serializePacket(message, xml);
@@ -217,23 +231,23 @@ void tst_QXmppMessage::testMessageReceipt()
     QXmppMessage message;
     // parsing
     parsePacket(message, xml);
-    QCOMPARE(message.id(), QString("richard2-4.1.247"));
-    QCOMPARE(message.to(), QString("kingrichard@royalty.england.lit/throne"));
-    QCOMPARE(message.from(), QString("northumberland@shakespeare.lit/westminster"));
+    QCOMPARE(message.id(), u"richard2-4.1.247"_s);
+    QCOMPARE(message.to(), u"kingrichard@royalty.england.lit/throne"_s);
+    QCOMPARE(message.from(), u"northumberland@shakespeare.lit/westminster"_s);
     QVERIFY(message.extendedAddresses().isEmpty());
     QCOMPARE(message.type(), QXmppMessage::Normal);
-    QCOMPARE(message.body(), QString("My lord, dispatch; read o'er these articles."));
+    QCOMPARE(message.body(), u"My lord, dispatch; read o'er these articles."_s);
     QCOMPARE(message.isAttentionRequested(), false);
     QCOMPARE(message.isReceiptRequested(), true);
     QCOMPARE(message.receiptId(), QString());
 
     // serialization
     message = QXmppMessage();
-    message.setId(QStringLiteral("richard2-4.1.247"));
-    message.setTo(QStringLiteral("kingrichard@royalty.england.lit/throne"));
-    message.setFrom(QStringLiteral("northumberland@shakespeare.lit/westminster"));
+    message.setId(u"richard2-4.1.247"_s);
+    message.setTo(u"kingrichard@royalty.england.lit/throne"_s);
+    message.setFrom(u"northumberland@shakespeare.lit/westminster"_s);
     message.setType(QXmppMessage::Normal);
-    message.setBody(QStringLiteral("My lord, dispatch; read o'er these articles."));
+    message.setBody(u"My lord, dispatch; read o'er these articles."_s);
     message.setReceiptRequested(true);
     serializePacket(message, xml);
 
@@ -244,22 +258,22 @@ void tst_QXmppMessage::testMessageReceipt()
 
     QXmppMessage receipt;
     parsePacket(receipt, receiptXml);
-    QCOMPARE(receipt.id(), QString("bi29sg183b4v"));
-    QCOMPARE(receipt.to(), QString("northumberland@shakespeare.lit/westminster"));
-    QCOMPARE(receipt.from(), QString("kingrichard@royalty.england.lit/throne"));
+    QCOMPARE(receipt.id(), u"bi29sg183b4v"_s);
+    QCOMPARE(receipt.to(), u"northumberland@shakespeare.lit/westminster"_s);
+    QCOMPARE(receipt.from(), u"kingrichard@royalty.england.lit/throne"_s);
     QVERIFY(receipt.extendedAddresses().isEmpty());
     QCOMPARE(receipt.type(), QXmppMessage::Normal);
     QCOMPARE(receipt.body(), QString());
     QCOMPARE(receipt.isAttentionRequested(), false);
     QCOMPARE(receipt.isReceiptRequested(), false);
-    QCOMPARE(receipt.receiptId(), QString("richard2-4.1.247"));
+    QCOMPARE(receipt.receiptId(), u"richard2-4.1.247"_s);
 
     receipt = QXmppMessage();
-    receipt.setId(QStringLiteral("bi29sg183b4v"));
-    receipt.setTo(QStringLiteral("northumberland@shakespeare.lit/westminster"));
-    receipt.setFrom(QStringLiteral("kingrichard@royalty.england.lit/throne"));
+    receipt.setId(u"bi29sg183b4v"_s);
+    receipt.setTo(u"northumberland@shakespeare.lit/westminster"_s);
+    receipt.setFrom(u"kingrichard@royalty.england.lit/throne"_s);
     receipt.setType(QXmppMessage::Normal);
-    receipt.setReceiptId(QStringLiteral("richard2-4.1.247"));
+    receipt.setReceiptId(u"richard2-4.1.247"_s);
     serializePacket(receipt, receiptXml);
 
     const QByteArray oldXml(
@@ -269,15 +283,15 @@ void tst_QXmppMessage::testMessageReceipt()
 
     QXmppMessage old;
     parsePacket(old, oldXml);
-    QCOMPARE(old.id(), QString("richard2-4.1.247"));
-    QCOMPARE(old.to(), QString("northumberland@shakespeare.lit/westminster"));
-    QCOMPARE(old.from(), QString("kingrichard@royalty.england.lit/throne"));
+    QCOMPARE(old.id(), u"richard2-4.1.247"_s);
+    QCOMPARE(old.to(), u"northumberland@shakespeare.lit/westminster"_s);
+    QCOMPARE(old.from(), u"kingrichard@royalty.england.lit/throne"_s);
     QVERIFY(old.extendedAddresses().isEmpty());
     QCOMPARE(old.type(), QXmppMessage::Normal);
     QCOMPARE(old.body(), QString());
     QCOMPARE(old.isAttentionRequested(), false);
     QCOMPARE(old.isReceiptRequested(), false);
-    QCOMPARE(old.receiptId(), QString("richard2-4.1.247"));
+    QCOMPARE(old.receiptId(), u"richard2-4.1.247"_s);
 
     // test that an ID is generated
     QXmppMessage message2;
@@ -371,12 +385,12 @@ void tst_QXmppMessage::testMucInvitation()
     QCOMPARE(message.mucInvitationReason(), QLatin1String("Hey Hecate, this is the place for all good witches!"));
 
     message = QXmppMessage();
-    message.setTo(QStringLiteral("hecate@shakespeare.lit"));
-    message.setFrom(QStringLiteral("crone1@shakespeare.lit/desktop"));
+    message.setTo(u"hecate@shakespeare.lit"_s);
+    message.setFrom(u"crone1@shakespeare.lit/desktop"_s);
     message.setType(QXmppMessage::Normal);
-    message.setMucInvitationJid(QStringLiteral("darkcave@macbeth.shakespeare.lit"));
-    message.setMucInvitationPassword(QStringLiteral("cauldronburn"));
-    message.setMucInvitationReason(QStringLiteral("Hey Hecate, this is the place for all good witches!"));
+    message.setMucInvitationJid(u"darkcave@macbeth.shakespeare.lit"_s);
+    message.setMucInvitationPassword(u"cauldronburn"_s);
+    message.setMucInvitationReason(u"Hey Hecate, this is the place for all good witches!"_s);
     serializePacket(message, xml);
 }
 
@@ -441,9 +455,9 @@ void tst_QXmppMessage::testXhtml()
     QCOMPARE(message.xhtml(), QLatin1String("<p style=\"font-weight:bold\">hi!</p>"));
 
     message = QXmppMessage();
-    message.setBody(QStringLiteral("hi!"));
+    message.setBody(u"hi!"_s);
     message.setType(QXmppMessage::Normal);
-    message.setXhtml(QStringLiteral("<p style=\"font-weight:bold\">hi!</p>"));
+    message.setXhtml(u"<p style=\"font-weight:bold\">hi!</p>"_s);
     serializePacket(message, xml);
 }
 
@@ -488,9 +502,9 @@ void tst_QXmppMessage::testChatMarkers()
     parsePacket(markableMessage, markableXml);
     QCOMPARE(markableMessage.isMarkable(), true);
     QCOMPARE(markableMessage.marker(), QXmppMessage::NoMarker);
-    QCOMPARE(markableMessage.id(), QString("message-1"));
+    QCOMPARE(markableMessage.id(), u"message-1"_s);
     QCOMPARE(markableMessage.markedId(), QString());
-    QCOMPARE(markableMessage.thread(), QString("sleeping"));
+    QCOMPARE(markableMessage.thread(), u"sleeping"_s);
     QCOMPARE(markableMessage.markedThread(), QString());
 
     const QByteArray receivedXml(
@@ -507,10 +521,10 @@ void tst_QXmppMessage::testChatMarkers()
     parsePacket(receivedMessage, receivedXml);
     QCOMPARE(receivedMessage.isMarkable(), false);
     QCOMPARE(receivedMessage.marker(), QXmppMessage::Received);
-    QCOMPARE(receivedMessage.id(), QString("message-2"));
-    QCOMPARE(receivedMessage.markedId(), QString("message-1"));
+    QCOMPARE(receivedMessage.id(), u"message-2"_s);
+    QCOMPARE(receivedMessage.markedId(), u"message-1"_s);
     QCOMPARE(receivedMessage.thread(), QString());
-    QCOMPARE(receivedMessage.markedThread(), QString("sleeping"));
+    QCOMPARE(receivedMessage.markedThread(), u"sleeping"_s);
 
     const QByteArray displayedXml(
         "<message "
@@ -526,10 +540,10 @@ void tst_QXmppMessage::testChatMarkers()
     parsePacket(displayedMessage, displayedXml);
     QCOMPARE(displayedMessage.isMarkable(), false);
     QCOMPARE(displayedMessage.marker(), QXmppMessage::Displayed);
-    QCOMPARE(displayedMessage.id(), QString("message-2"));
-    QCOMPARE(displayedMessage.markedId(), QString("message-1"));
+    QCOMPARE(displayedMessage.id(), u"message-2"_s);
+    QCOMPARE(displayedMessage.markedId(), u"message-1"_s);
     QCOMPARE(displayedMessage.thread(), QString());
-    QCOMPARE(displayedMessage.markedThread(), QString("sleeping"));
+    QCOMPARE(displayedMessage.markedThread(), u"sleeping"_s);
 
     const QByteArray acknowledgedXml(
         "<message "
@@ -545,10 +559,10 @@ void tst_QXmppMessage::testChatMarkers()
     parsePacket(acknowledgedMessage, acknowledgedXml);
     QCOMPARE(acknowledgedMessage.isMarkable(), false);
     QCOMPARE(acknowledgedMessage.marker(), QXmppMessage::Acknowledged);
-    QCOMPARE(acknowledgedMessage.id(), QString("message-2"));
-    QCOMPARE(acknowledgedMessage.markedId(), QString("message-1"));
+    QCOMPARE(acknowledgedMessage.id(), u"message-2"_s);
+    QCOMPARE(acknowledgedMessage.markedId(), u"message-1"_s);
     QCOMPARE(acknowledgedMessage.thread(), QString());
-    QCOMPARE(acknowledgedMessage.markedThread(), QString("sleeping"));
+    QCOMPARE(acknowledgedMessage.markedThread(), u"sleeping"_s);
 
     const QByteArray emptyThreadXml(
         "<message "
@@ -563,8 +577,8 @@ void tst_QXmppMessage::testChatMarkers()
     parsePacket(emptyThreadMessage, emptyThreadXml);
     QCOMPARE(emptyThreadMessage.isMarkable(), false);
     QCOMPARE(emptyThreadMessage.marker(), QXmppMessage::Received);
-    QCOMPARE(emptyThreadMessage.id(), QString("message-2"));
-    QCOMPARE(emptyThreadMessage.markedId(), QString("message-1"));
+    QCOMPARE(emptyThreadMessage.id(), u"message-2"_s);
+    QCOMPARE(emptyThreadMessage.markedId(), u"message-1"_s);
     QCOMPARE(emptyThreadMessage.thread(), QString());
     QCOMPARE(emptyThreadMessage.markedThread(), QString());
 
@@ -671,7 +685,7 @@ void tst_QXmppMessage::testPrivateMessage()
     QCOMPARE(message.isPrivate(), true);
 
     message = QXmppMessage();
-    message.setBody(QStringLiteral("My lord, dispatch; read o'er these articles."));
+    message.setBody(u"My lord, dispatch; read o'er these articles."_s);
     message.setPrivate(true);
     serializePacket(message, xml);
 
@@ -705,11 +719,11 @@ void tst_QXmppMessage::testOutOfBandUrl()
 
     QCOMPARE(oobMessage.outOfBandUrls().size(), 2);
 
-    QCOMPARE(oobMessage.outOfBandUrls().front().url(), QStringLiteral("http://www.jabber.org/images/psa-license.jpg"));
+    QCOMPARE(oobMessage.outOfBandUrls().front().url(), u"http://www.jabber.org/images/psa-license.jpg"_s);
     QVERIFY(!oobMessage.outOfBandUrls().front().description().has_value());
 
-    QCOMPARE(oobMessage.outOfBandUrls().at(1).url(), QStringLiteral("https://xmpp.org/images/logos/xmpp-logo.svg"));
-    QCOMPARE(oobMessage.outOfBandUrls().at(1).description().value(), QStringLiteral("XMPP logo"));
+    QCOMPARE(oobMessage.outOfBandUrls().at(1).url(), u"https://xmpp.org/images/logos/xmpp-logo.svg"_s);
+    QCOMPARE(oobMessage.outOfBandUrls().at(1).description().value(), u"XMPP logo"_s);
 
     serializePacket(oobMessage, oobXml);
 
@@ -728,11 +742,11 @@ void tst_QXmppMessage::testMessageCorrect()
 
     QXmppMessage message;
     parsePacket(message, xml);
-    QCOMPARE(message.replaceId(), QString("badmessage"));
+    QCOMPARE(message.replaceId(), u"badmessage"_s);
     serializePacket(message, xml);
 
     message.setReplaceId("someotherid");
-    QCOMPARE(message.replaceId(), QString("someotherid"));
+    QCOMPARE(message.replaceId(), u"someotherid"_s);
 }
 
 void tst_QXmppMessage::testMessageAttaching()
@@ -745,11 +759,11 @@ void tst_QXmppMessage::testMessageAttaching()
 
     QXmppMessage message;
     parsePacket(message, xml);
-    QCOMPARE(message.attachId(), QString("SD24VCzSYQ"));
+    QCOMPARE(message.attachId(), u"SD24VCzSYQ"_s);
     serializePacket(message, xml);
 
     message.setAttachId("someotherid");
-    QCOMPARE(message.attachId(), QString("someotherid"));
+    QCOMPARE(message.attachId(), u"someotherid"_s);
 }
 
 void tst_QXmppMessage::testMix()
@@ -765,17 +779,33 @@ void tst_QXmppMessage::testMix()
         "<body>Harpier cries: &apos;tis time, &apos;tis time.</body>"
         "</message>");
 
-    QXmppMessage message;
-    parsePacket(message, xml);
-    serializePacket(message, xml);
+    QXmppMessage message1;
 
-    QCOMPARE(message.mixUserJid(), QString("hag66@shakespeare.example"));
-    QCOMPARE(message.mixUserNick(), QString("thirdwitch"));
+    QVERIFY(message1.mixParticipantId().isEmpty());
+    QVERIFY(message1.mixUserJid().isEmpty());
+    QVERIFY(message1.mixUserNick().isEmpty());
 
-    message.setMixUserJid("alexander@example.org");
-    QCOMPARE(message.mixUserJid(), QString("alexander@example.org"));
-    message.setMixUserNick("erik");
-    QCOMPARE(message.mixUserNick(), QString("erik"));
+    parsePacket(message1, xml);
+    serializePacket(message1, xml);
+
+    QCOMPARE(message1.mixParticipantId(), u"123456"_s);
+    QCOMPARE(message1.mixUserJid(), u"hag66@shakespeare.example"_s);
+    QCOMPARE(message1.mixUserNick(), u"thirdwitch"_s);
+
+    QXmppMessage message2;
+
+    message2.setType(QXmppMessage::GroupChat);
+    message2.setFrom(u"coven@mix.shakespeare.example/123456"_s);
+    message2.setTo(u"hag66@shakespeare.example"_s);
+    message2.setBody(u"Harpier cries: 'tis time, 'tis time."_s);
+    message2.setMixUserJid(u"hag66@shakespeare.example"_s);
+    message2.setMixUserNick(u"thirdwitch"_s);
+
+    QCOMPARE(message2.mixParticipantId(), u"123456"_s);
+    QCOMPARE(message2.mixUserJid(), u"hag66@shakespeare.example"_s);
+    QCOMPARE(message2.mixUserNick(), u"thirdwitch"_s);
+
+    serializePacket(message2, xml);
 }
 
 void tst_QXmppMessage::testEme()
@@ -789,9 +819,9 @@ void tst_QXmppMessage::testEme()
 
     QXmppMessage messageOmemo;
     parsePacket(messageOmemo, xmlOmemo);
-    QCOMPARE(messageOmemo.encryptionMethodNs(), QString("eu.siacs.conversations.axolotl"));
+    QCOMPARE(messageOmemo.encryptionMethodNs(), u"eu.siacs.conversations.axolotl"_s);
     QCOMPARE(messageOmemo.encryptionMethod(), QXmpp::OMEMO);
-    QCOMPARE(messageOmemo.encryptionName(), QString("OMEMO"));
+    QCOMPARE(messageOmemo.encryptionName(), u"OMEMO"_s);
     serializePacket(messageOmemo, xmlOmemo);
 
     // test custom encryption
@@ -803,23 +833,23 @@ void tst_QXmppMessage::testEme()
 
     QXmppMessage messageCustom;
     parsePacket(messageCustom, xmlCustom);
-    QCOMPARE(messageCustom.encryptionMethodNs(), QString("im:example:customcrypt:1"));
+    QCOMPARE(messageCustom.encryptionMethodNs(), u"im:example:customcrypt:1"_s);
     QCOMPARE(messageCustom.encryptionMethod(), QXmpp::UnknownEncryption);
-    QCOMPARE(messageCustom.encryptionName(), QString("CustomCrypt"));
+    QCOMPARE(messageCustom.encryptionName(), u"CustomCrypt"_s);
     serializePacket(messageCustom, xmlCustom);
 
     // test setters/getters
     QXmppMessage message;
     message.setEncryptionMethod(QXmpp::LegacyOpenPGP);
     QCOMPARE(message.encryptionMethod(), QXmpp::LegacyOpenPGP);
-    QCOMPARE(message.encryptionMethodNs(), QString("jabber:x:encrypted"));
-    QCOMPARE(message.encryptionName(), QString("Legacy OpenPGP"));
+    QCOMPARE(message.encryptionMethodNs(), u"jabber:x:encrypted"_s);
+    QCOMPARE(message.encryptionName(), u"Legacy OpenPGP"_s);
 
     message.setEncryptionMethodNs("fancyorg:encryption:fancycrypt:0");
     message.setEncryptionName("FancyCrypt");
     QCOMPARE(message.encryptionMethod(), QXmpp::UnknownEncryption);
-    QCOMPARE(message.encryptionMethodNs(), QString("fancyorg:encryption:fancycrypt:0"));
-    QCOMPARE(message.encryptionName(), QString("FancyCrypt"));
+    QCOMPARE(message.encryptionMethodNs(), u"fancyorg:encryption:fancycrypt:0"_s);
+    QCOMPARE(message.encryptionName(), u"FancyCrypt"_s);
 }
 
 void tst_QXmppMessage::testSpoiler()
@@ -834,7 +864,7 @@ void tst_QXmppMessage::testSpoiler()
     QXmppMessage messageWithHint;
     parsePacket(messageWithHint, xmlWithHint);
     QVERIFY(messageWithHint.isSpoiler());
-    QCOMPARE(messageWithHint.spoilerHint(), QString("Love story end"));
+    QCOMPARE(messageWithHint.spoilerHint(), u"Love story end"_s);
     serializePacket(messageWithHint, xmlWithHint);
 
     // test parsing without hint
@@ -847,7 +877,7 @@ void tst_QXmppMessage::testSpoiler()
     QXmppMessage messageWithoutHint;
     parsePacket(messageWithoutHint, xmlWithoutHint);
     QVERIFY(messageWithoutHint.isSpoiler());
-    QCOMPARE(messageWithoutHint.spoilerHint(), QString(""));
+    QCOMPARE(messageWithoutHint.spoilerHint(), u""_s);
     serializePacket(messageWithoutHint, xmlWithoutHint);
 
     // test setters
@@ -857,7 +887,7 @@ void tst_QXmppMessage::testSpoiler()
 
     message.setIsSpoiler(false);
     message.setSpoilerHint("test hint");
-    QCOMPARE(message.spoilerHint(), QString("test hint"));
+    QCOMPARE(message.spoilerHint(), u"test hint"_s);
     QVERIFY(message.isSpoiler());
 }
 
@@ -885,9 +915,9 @@ void tst_QXmppMessage::testProcessingHints()
     // test serialization
     QXmppMessage message2;
     message2.setType(QXmppMessage::Chat);
-    message2.setFrom(QString("romeo@montague.lit/laptop"));
-    message2.setTo(QString("juliet@capulet.lit/laptop"));
-    message2.setBody(QString("V unir avtug'f pybnx gb uvqr zr sebz gurve fvtug"));
+    message2.setFrom(u"romeo@montague.lit/laptop"_s);
+    message2.setTo(u"juliet@capulet.lit/laptop"_s);
+    message2.setBody(u"V unir avtug'f pybnx gb uvqr zr sebz gurve fvtug"_s);
     message2.addHint(QXmppMessage::NoPermanentStore);
     message2.addHint(QXmppMessage::NoStore);
     message2.addHint(QXmppMessage::NoCopy);
@@ -939,8 +969,8 @@ void tst_QXmppMessage::testBobData()
 
     QXmppBitsOfBinaryData data;
     data.setCid(QXmppBitsOfBinaryContentId::fromContentId(
-        QStringLiteral("sha1+5a4c38d44fc64805cbb2d92d8b208be13ff40c0f@bob.xmpp.org")));
-    data.setContentType(QMimeDatabase().mimeTypeForName(QStringLiteral("image/png")));
+        u"sha1+5a4c38d44fc64805cbb2d92d8b208be13ff40c0f@bob.xmpp.org"_s));
+    data.setContentType(QMimeDatabase().mimeTypeForName(u"image/png"_s));
     data.setData(QByteArray::fromBase64(QByteArrayLiteral(
         "iVBORw0KGgoAAAANSUhEUgAAALQAAAA8BAMAAAA9AI20AAAAG1BMVEX///8AAADf39+"
         "/v79/f39fX1+fn58/Pz8fHx/8ACGJAAAACXBIWXMAAA7EAAAOxAGVKw4bAAADS0lEQV"
@@ -967,7 +997,7 @@ void tst_QXmppMessage::testBobData()
     QXmppMessage message;
     parsePacket(message, xml);
     QCOMPARE(message.type(), QXmppMessage::Chat);
-    QCOMPARE(message.id(), QStringLiteral(""));
+    QCOMPARE(message.id(), u""_s);
     QCOMPARE(message.bitsOfBinaryData().size(), 1);
     QCOMPARE(message.bitsOfBinaryData().first().cid().algorithm(), data.cid().algorithm());
     QCOMPARE(message.bitsOfBinaryData().first().cid().hash(), data.cid().hash());
@@ -980,7 +1010,7 @@ void tst_QXmppMessage::testBobData()
 
     QXmppMessage msg;
     msg.setType(QXmppMessage::Chat);
-    msg.setId(QStringLiteral(""));
+    msg.setId(u""_s);
     QXmppBitsOfBinaryDataList bobDataList;
     bobDataList << data;
     msg.setBitsOfBinaryData(bobDataList);
@@ -988,7 +1018,7 @@ void tst_QXmppMessage::testBobData()
 
     QXmppMessage msg2;
     msg2.setType(QXmppMessage::Chat);
-    msg2.setId(QStringLiteral(""));
+    msg2.setId(u""_s);
     msg2.bitsOfBinaryData() << data;
     serializePacket(msg2, xml);
 
@@ -999,11 +1029,15 @@ void tst_QXmppMessage::testBobData()
 
 void tst_QXmppMessage::testFallbackIndication()
 {
-    const QByteArray xml = QByteArrayLiteral(
+    using Fallback = QXmppFallback;
+
+    QByteArray xml = QByteArrayLiteral(
         "<message type=\"chat\">"
         "<fallback xmlns=\"urn:xmpp:fallback:0\"/>"
         "</message>");
 
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_DEPRECATED
     QXmppMessage message;
     parsePacket(message, xml);
     QVERIFY(message.isFallback());
@@ -1012,27 +1046,108 @@ void tst_QXmppMessage::testFallbackIndication()
     QXmppMessage message2;
     message2.setIsFallback(true);
     serializePacket(message2, xml);
+    QT_WARNING_POP
+
+    xml =
+        "<message type='chat'>"
+        "<fallback xmlns='urn:xmpp:fallback:0' for='urn:xmpp:reply:0'>"
+        "<body start='0' end='33'/>"
+        "<subject start='5' end='10'/>"
+        "<body/>"
+        "</fallback>"
+        "</message>";
+    message = {};
+    parsePacket(message, xml);
+    QCOMPARE(message.fallbackMarkers().size(), 1);
+    auto marker = message.fallbackMarkers().first();
+    QCOMPARE(marker.forNamespace(), "urn:xmpp:reply:0");
+    const auto &refs = marker.references();
+    QCOMPARE(refs.size(), 3);
+    QCOMPARE(refs.at(0).element, Fallback::Body);
+    QCOMPARE(refs.at(1).element, Fallback::Subject);
+    QCOMPARE(refs.at(2).element, Fallback::Body);
+    QVERIFY(refs.at(0).range.has_value());
+    QVERIFY(refs.at(1).range.has_value());
+    QVERIFY(!refs.at(2).range.has_value());
+    QCOMPARE(refs.at(0).range->start, 0);
+    QCOMPARE(refs.at(0).range->end, 33);
+
+    serializePacket(message, xml);
+
+    message = {};
+    message.setFallbackMarkers({
+        Fallback {
+            "urn:xmpp:reply:0",
+            { Fallback::Reference { Fallback::Body, Fallback::Range { 0, 33 } },
+              Fallback::Reference { Fallback::Subject, Fallback::Range { 5, 10 } },
+              Fallback::Reference { Fallback::Body, {} } },
+        },
+    });
+    serializePacket(message, xml);
+}
+
+void tst_QXmppMessage::fallbackBodies()
+{
+    const QByteArray xml = QByteArrayLiteral(
+        "<message to='anna@example.com' id='message-id2' type='groupchat'>"
+        "<body>"
+        "> Anna wrote:\n"
+        "> Hi, how are you?\n"
+        "Great"
+        "</body>"
+        "<reply to='anna@example.com' id='message-id1' xmlns='urn:xmpp:reply:0'/>"
+        "<fallback xmlns='urn:xmpp:fallback:0' for='urn:xmpp:reply:0'>"
+        "<body start='0' end='33' />"
+        "</fallback>"
+        "</message>");
+
+    QXmppMessage message;
+    parsePacket(message, xml);
+
+    QCOMPARE(message.readFallbackRemovedText(QXmppFallback::Body, { "urn:xmpp:reply:0" }), "Great");
+    QCOMPARE(message.readFallbackRemovedText(QXmppFallback::Subject, { "urn:xmpp:reply:0" }), QString());
+
+    QCOMPARE(message.readFallbackText(QXmppFallback::Body, u"urn:xmpp:reply:0"), "> Anna wrote:\n> Hi, how are you?\n");
+    QCOMPARE(message.readFallbackText(QXmppFallback::Subject, u"urn:xmpp:reply:0"), QString());
+
+    QCOMPARE(message.readReplyQuoteFromBody(), "Anna wrote:\nHi, how are you?\n");
 }
 
 void tst_QXmppMessage::testStanzaIds()
 {
     const QByteArray xml = QByteArrayLiteral(
         "<message type=\"chat\">"
+        "<stanza-id xmlns=\"urn:xmpp:sid:0\" id=\"123\" by=\"room@mix.qxmpp.org\"/>"
         "<stanza-id xmlns=\"urn:xmpp:sid:0\" id=\"1236\" by=\"server.tld\"/>"
         "<origin-id xmlns=\"urn:xmpp:sid:0\" id=\"5678\"/>"
         "</message>");
 
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_DEPRECATED
     QXmppMessage msg;
     parsePacket(msg, xml);
-    QCOMPARE(msg.stanzaId(), QStringLiteral("1236"));
-    QCOMPARE(msg.stanzaIdBy(), QStringLiteral("server.tld"));
-    QCOMPARE(msg.originId(), QStringLiteral("5678"));
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_DEPRECATED
+    QCOMPARE(msg.stanzaId(), u"1236"_s);
+    QCOMPARE(msg.stanzaIdBy(), u"server.tld"_s);
+    QT_WARNING_POP
+    QCOMPARE(msg.stanzaIds().size(), 2);
+    QCOMPARE(msg.originId(), u"5678"_s);
     serializePacket(msg, xml);
 
     QXmppMessage msg2;
-    msg2.setStanzaId(QStringLiteral("1236"));
-    msg2.setStanzaIdBy(QStringLiteral("server.tld"));
-    msg2.setOriginId(QStringLiteral("5678"));
+
+    // test both setters (single id / multiple ids)
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_DEPRECATED
+    msg2.setStanzaId(u"1236"_s);
+    msg2.setStanzaIdBy(u"server.tld"_s);
+    QT_WARNING_POP
+
+    auto ids = msg2.stanzaIds();
+    ids.prepend(QXmppStanzaId { "123", "room@mix.qxmpp.org" });
+    msg2.setStanzaIds(ids);
+    msg2.setOriginId(u"5678"_s);
     serializePacket(msg2, xml);
 }
 
@@ -1192,19 +1307,19 @@ void tst_QXmppMessage::testE2eeFallbackBody()
 
     QXmppMessage message1;
     parsePacket(message1, xml);
-    QCOMPARE(message1.e2eeFallbackBody(), QStringLiteral("This message is encrypted with OMEMO 2 but could not be decrypted"));
+    QCOMPARE(message1.e2eeFallbackBody(), u"This message is encrypted with OMEMO 2 but could not be decrypted"_s);
     serializePacket(message1, xml);
 
     QXmppMessage message2;
-    message2.setE2eeFallbackBody(QStringLiteral("This message is encrypted with OMEMO 2 but could not be decrypted"));
+    message2.setE2eeFallbackBody(u"This message is encrypted with OMEMO 2 but could not be decrypted"_s);
     serializePacket(message2, xml);
 }
 
 void tst_QXmppMessage::testFileSharing()
 {
-    const QByteArray xml(
+    QByteArray xml(
         "<message id='sharing-a-file' to='juliet@shakespeare.lit' from='romeo@montague.lit/resource' type='normal'>"
-        "<file-sharing xmlns='urn:xmpp:sfs:0' disposition='inline'>"
+        "<file-sharing xmlns='urn:xmpp:sfs:0' disposition='inline' id='abc23'>"
         "<file xmlns='urn:xmpp:file:metadata:0'>"
         "<desc>Photo from the summit.</desc>"
         "<hash xmlns='urn:xmpp:hashes:2' algo='sha3-256'>2XarmwTlNxDAMkvymloX3S5+VbylNrJt/l5QyPa+YoU=</hash>"
@@ -1220,10 +1335,30 @@ void tst_QXmppMessage::testFileSharing()
         "</file-sharing>"
         "</message>");
 
-    QXmppMessage message1;
-    parsePacket(message1, xml);
-    QVERIFY(!message1.sharedFiles().empty());
-    serializePacket(message1, xml);
+    QXmppMessage message;
+    parsePacket(message, xml);
+    QVERIFY(!message.sharedFiles().empty());
+    QCOMPARE(message.sharedFiles().first().id(), "abc23");
+    serializePacket(message, xml);
+
+    xml = "<message id='adding-photo1' to='juliet@shakespeare.lit' from='romeo@montague.lit/resource' type='normal'>"
+          "<body>https://download.montague.lit/4a771ac1-f0b2-4a4a-9700-f2a26fa2bb67/photo1.jpg</body>"
+          "<x xmlns='jabber:x:oob'><url>https://download.montague.lit/4a771ac1-f0b2-4a4a-9700-f2a26fa2bb67/photo1.jpg</url></x>"
+          "<attach-to xmlns='urn:xmpp:message-attaching:1' id='sharing-files'/>"
+          "<sources xmlns='urn:xmpp:sfs:0' id='photo1.jpg'>"
+          "<url-data xmlns='http://jabber.org/protocol/url-data' target='https://download.montague.lit/4a771ac1-f0b2-4a4a-9700-f2a26fa2bb67/photo1.jpg'/>"
+          "</sources>"
+          "<fallback xmlns='urn:xmpp:fallback:0' for='urn:xmpp:sfs:0'><body/></fallback>"
+          "</message>";
+    message = {};
+
+    parsePacket(message, xml);
+    QVERIFY(message.sharedFiles().empty());
+    QCOMPARE(message.fileSourcesAttachments().size(), 1);
+    auto sources = message.fileSourcesAttachments().first();
+    QCOMPARE(sources.httpSources().size(), 1);
+    QCOMPARE(sources.httpSources().first().url(), QUrl::fromEncoded("https://download.montague.lit/4a771ac1-f0b2-4a4a-9700-f2a26fa2bb67/photo1.jpg"));
+    serializePacket(message, xml);
 }
 
 void tst_QXmppMessage::testEncryptedFileSource()
@@ -1285,5 +1420,42 @@ void tst_QXmppMessage::testEncryptedFileSource()
     }
 }
 
+void tst_QXmppMessage::testReplies()
+{
+    const QByteArray xml =
+        "<message id='message-id2' to='anna@example.com' type='chat'>"
+        "<body>Great idea!</body>"
+        "<reply xmlns='urn:xmpp:reply:0' to='anna@example.com/tablet' id='message-id1'/>"
+        "</message>";
+
+    QXmppMessage m;
+    parsePacket(m, xml);
+    QVERIFY(m.reply().has_value());
+    QCOMPARE(m.reply()->id, "message-id1");
+    QCOMPARE(m.reply()->to, "anna@example.com/tablet");
+    serializePacket(m, xml);
+}
+
+void tst_QXmppMessage::testJingleMessageInitiationElement()
+{
+    const QByteArray xml(
+        "<message to='romeo@montague.example' from='juliet@capulet.example/phone' type='chat'>"
+        "<store xmlns=\"urn:xmpp:hints\"/>"
+        "<proceed xmlns='urn:xmpp:jingle-message:0' id='ca3cf894-5325-482f-a412-a6e9f832298d'/>"
+        "</message>");
+
+    QXmppMessage message1;
+    QVERIFY(!message1.jingleMessageInitiationElement());
+
+    parsePacket(message1, xml);
+    QVERIFY(message1.jingleMessageInitiationElement());
+    serializePacket(message1, xml);
+
+    QXmppMessage message2;
+    message2.addHint(QXmppMessage::Store);
+    message2.setJingleMessageInitiationElement(QXmppJingleMessageInitiationElement());
+    QVERIFY(message2.jingleMessageInitiationElement());
+}
+
 QTEST_MAIN(tst_QXmppMessage)
 #include "tst_qxmppmessage.moc"
