@@ -10,6 +10,7 @@
 
 #include "pubsubutil.h"
 #include "util.h"
+
 #include <QObject>
 
 using namespace QXmpp::Private;
@@ -101,8 +102,8 @@ void tst_QXmppPubSubIq::testItemsResponse()
     QVERIFY(iq.itemsContinuation().has_value());
     QCOMPARE(iq.itemsContinuation()->count(), 19);
     QCOMPARE(iq.itemsContinuation()->index(), 0);
-    QCOMPARE(iq.itemsContinuation()->first(), QStringLiteral("current"));
-    QCOMPARE(iq.itemsContinuation()->last(), QStringLiteral("otheritemid"));
+    QCOMPARE(iq.itemsContinuation()->first(), u"current"_s);
+    QCOMPARE(iq.itemsContinuation()->last(), u"otheritemid"_s);
     serializePacket(iq, xml);
 }
 
@@ -117,7 +118,7 @@ void tst_QXmppPubSubIq::testCreateNode()
 
     PubSubIq iq;
     parsePacket(iq, xml);
-    QCOMPARE(iq.id(), QString("create1"));
+    QCOMPARE(iq.id(), u"create1"_s);
     QCOMPARE(iq.to(), QLatin1String("pubsub.shakespeare.lit"));
     QCOMPARE(iq.from(), QLatin1String("hamlet@denmark.lit/elsinore"));
     QCOMPARE(iq.type(), QXmppIq::Set);
@@ -147,7 +148,7 @@ void tst_QXmppPubSubIq::testDeleteNode()
 
     PubSubIq iq;
     parsePacket(iq, xml);
-    QCOMPARE(iq.id(), QString("delete1"));
+    QCOMPARE(iq.id(), u"delete1"_s);
     QCOMPARE(iq.to(), QLatin1String("pubsub.shakespeare.lit"));
     QCOMPARE(iq.from(), QLatin1String("hamlet@denmark.lit/elsinore"));
     QCOMPARE(iq.type(), QXmppIq::Set);
@@ -193,7 +194,7 @@ void tst_QXmppPubSubIq::testPublish()
     serializePacket(iq, xml);
 
     // serialize using setters
-    QXmppPubSubBaseItem item(QStringLiteral("current"));
+    QXmppPubSubBaseItem item(u"current"_s);
 
     iq = PubSubIq();
     iq.setId(QLatin1String("items1"));
@@ -225,7 +226,7 @@ void tst_QXmppPubSubIq::testRetractItem()
 
     PubSubIq<> iq;
     parsePacket(iq, xml);
-    QCOMPARE(iq.id(), QString("retract1"));
+    QCOMPARE(iq.id(), u"retract1"_s);
     QCOMPARE(iq.to(), QLatin1String("pubsub.shakespeare.lit"));
     QCOMPARE(iq.from(), QLatin1String("hamlet@denmark.lit/elsinore"));
     QCOMPARE(iq.type(), QXmppIq::Set);
@@ -233,7 +234,7 @@ void tst_QXmppPubSubIq::testRetractItem()
     QCOMPARE(iq.queryJid(), QString());
     QCOMPARE(iq.queryNode(), QLatin1String("princely_musings"));
     QCOMPARE(iq.items().size(), 1);
-    QCOMPARE(iq.items().first().id(), QStringLiteral("ae890ac52d0df67ed7cfdf51b644e901"));
+    QCOMPARE(iq.items().first().id(), u"ae890ac52d0df67ed7cfdf51b644e901"_s);
     serializePacket(iq, xml);
 
     iq = PubSubIq();
@@ -246,7 +247,7 @@ void tst_QXmppPubSubIq::testRetractItem()
     iq.setQueryNode(QLatin1String("princely_musings"));
 
     QXmppPubSubBaseItem item;
-    item.setId(QStringLiteral("ae890ac52d0df67ed7cfdf51b644e901"));
+    item.setId(u"ae890ac52d0df67ed7cfdf51b644e901"_s);
     iq.setItems({ item });
 
     serializePacket(iq, xml);
@@ -294,14 +295,14 @@ void tst_QXmppPubSubIq::testSubscription()
 
     PubSubIq iq;
     parsePacket(iq, xml);
-    QCOMPARE(iq.id(), QString("sub1"));
-    QCOMPARE(iq.to(), QString("francisco@denmark.lit/barracks"));
-    QCOMPARE(iq.from(), QString("pubsub.shakespeare.lit"));
+    QCOMPARE(iq.id(), u"sub1"_s);
+    QCOMPARE(iq.to(), u"francisco@denmark.lit/barracks"_s);
+    QCOMPARE(iq.from(), u"pubsub.shakespeare.lit"_s);
     QCOMPARE(iq.type(), QXmppIq::Result);
     QCOMPARE(iq.queryType(), PubSubIq<>::Subscription);
-    QCOMPARE(iq.subscription()->jid(), QString("francisco@denmark.lit"));
-    QCOMPARE(iq.subscription()->node(), QString("princely_musings"));
-    QCOMPARE(iq.subscription()->subId(), QStringLiteral("ba49252aaa4f5d320c24d3766f0bdcade78c78d3"));
+    QCOMPARE(iq.subscription()->jid(), u"francisco@denmark.lit"_s);
+    QCOMPARE(iq.subscription()->node(), u"princely_musings"_s);
+    QCOMPARE(iq.subscription()->subId(), u"ba49252aaa4f5d320c24d3766f0bdcade78c78d3"_s);
     serializePacket(iq, xml);
 
     iq = PubSubIq();
@@ -373,11 +374,7 @@ void tst_QXmppPubSubIq::testIsPubSubIq()
     QFETCH(QByteArray, xml);
     QFETCH(bool, isValid);
 
-    QDomDocument doc;
-    QVERIFY(doc.setContent(xml, true));
-    QDomElement element = doc.documentElement();
-
-    QCOMPARE(PubSubIq<>::isPubSubIq(element), isValid);
+    QCOMPARE(PubSubIq<>::isPubSubIq(xmlToDom(xml)), isValid);
 }
 
 void tst_QXmppPubSubIq::testCustomItem()
@@ -402,10 +399,10 @@ void tst_QXmppPubSubIq::testCustomItem()
     QCOMPARE(iq.type(), QXmppIq::Result);
     QCOMPARE(iq.queryType(), PubSubIq<>::Items);
     QCOMPARE(iq.queryJid(), QString());
-    QCOMPARE(iq.queryNode(), QStringLiteral("blah"));
+    QCOMPARE(iq.queryNode(), u"blah"_s);
     QCOMPARE(iq.items().size(), 2);
-    QCOMPARE(iq.items().at(0).id(), QStringLiteral("42"));
-    QCOMPARE(iq.items().at(1).id(), QStringLiteral("23"));
+    QCOMPARE(iq.items().at(0).id(), u"42"_s);
+    QCOMPARE(iq.items().at(1).id(), u"23"_s);
     QCOMPARE(iq.items().at(0).publisher(), QString());
     QCOMPARE(iq.items().at(1).publisher(), QString());
 
